//
// hf.h
//
// Create a fractal landscape
//
// Copyright (c) J. Belson	1997.12.29
//
// $Author: jon $	:	$Date: 2003/09/17 20:59:35 $
//


#ifndef _HF_H_
#define	_HF_H_

#include <cstdlib>
#include <iostream>

#include "settings.h"
#include "point3d.h"
#include "vector3d.h"


enum test_type {
	enumWAVE,
	enumBOX,
	enumTRIANGLE,
	enumFLAT
};


/**
 * Base class of all heightfields. To create a new type of
 *  heightfield, derive from hf and implement the virtual
 *  members as appropriate.
 */
class hf {

private:


protected:

	enum {
		TERRAIN_WIDTH	= 256,
		MAX_NUM_OCTAVES	= 10
	};

	// Width of map
	int map_size;
	int map_order;

	float offset;

	// Seed for random number generator, set to
	//  a -ve value on first function call
	long idum;

	// Seed for random number generator
	unsigned long seed;

	settings* my_settings;

	// Function to give array offset from x and y...
	virtual int index(int x, int y) {
		return (y*(map_size+1) + x);
	}

	// Scale vertical range
	float scale(float f);

	// Interpolation routines
	float interpolate(float a, float b, float x);
	float cosine_interpolate(float a, float b, float x);
	float linear_interpolate(float a, float b, float x);


public:
	// Create an instance of a heightfield
	hf(void);
	virtual ~hf();

	// Any fractral specific initialisation here
	virtual void init(void) {};

	// Effective size of terrain
	virtual int get_width(void)	{ return TERRAIN_WIDTH; }

	// Get height at arbitrary heightfield point...
	virtual float get_height(float x, float y) = 0;

	// Apply offset to calculated height
	void set_offset(float f);

	// Return approximation to surface normal at position
	virtual vector3d get_normal(float x, float y);

	// Trace ray to see if it intersects terrain
	virtual bool trace_path(point3d& p, const vector3d view);

	virtual void seed_rand(unsigned long int s);
};


#endif	// _HF_H
