//
// hf_multifactal.cc
//
// Create multifractal heightfield using Perlin noise
// From 'Procedural Texturing and Modelling'
//
// My bits Copyright (C) J. Belson	2001.06.01
//
// $Author: jon $ : $Date: 2003/09/17 20:59:35 $
//

#define WANT_LOGGING

#include "hf_multifractal.h"
#include "logging.h"
#include "noise.h"
#include "settings.h"


/**
 * Constructor
 */
hf_multifractal::hf_multifractal(void)
	: hf(),
	  _lacunarity(2.0),
	  _offset(0.8),
	  _iterations(4)
{
	float f;
	int val;

	my_settings->get_float(MULTIFRACTAL_FD, &f);
	my_settings->get_int(MULTIFRACTAL_ITERATIONS, &val);
	my_settings->get_float(MULTIFRACTAL_LACUNARITY, &_lacunarity);

	_iterations = val;
	_exponent_array = new double[int (_iterations) + 1];

	for (int i=0; i<_iterations; i++) {
		_exponent_array[i] = pow(_lacunarity, -i*f);
	}

	srandom(seed);

	my_settings->get_float(MULTIFRACTAL_SCALE, &_scaling);
	_scaling = 0.1 + _scaling/100.0;

	_noise = new my_noise();
}


/**
 * Destructor
 */
hf_multifractal::~hf_multifractal()
{
	delete _noise;
	delete [] _exponent_array;
}



/**
 * Get height at arbitrary heightfield point...
 */
float hf_multifractal::get_height(float x, float y)
{
	if (seed == 0) {
		return -0.1 + offset;
	}

	double value = 1.0;

	x *= _scaling;
	y *= _scaling;

	float adj_x	= x * float(TERRAIN_WIDTH)/map_size;
	float adj_y	= y * float(TERRAIN_WIDTH)/map_size;

	point2d point(adj_x, adj_y);

	// Inner loop of spectral construction
	int i;
	for (i=0; i<_iterations; i++) {
		value *= _noise->noise2(point)*_exponent_array[i] + 1.0;

		point = point * _lacunarity;
	}

	double remainder = _iterations - int (_iterations);

	if (remainder) {		// Add in 'octaves' remainder
		value += remainder*_noise->noise2(point)*_exponent_array[i];
	}


	return offset + -18.0 + scale(value/2.5);
}


