//
// hf_perlin.cc
//
// Create heightfield using Perlin noise
//
// Copyright (C) J. Belson	2000.1.16
//
// $Author: jon $ : $Date: 2003/07/26 17:01:33 $
//


#include "assert.h"
#include <cmath>

#include "hf_perlin.h"
#include "perlin.h"
#include "settings.h"
#include "point3d.h"


/**
 * Lookup tables containing prime numbers
 */
const long int hf_perlin::primes[3][NUM_PRIMES] = {

{
	#include "primes1.cc"
},

{
	#include "primes2.cc"
},

{
	#include "primes3.cc"
}

};


/**
 * Constructor
 */
hf_perlin::hf_perlin(void)
	: hf(),
	  set(0)
{
	puts("hf_perlin::hf_perlin()");

	my_settings = settings::get_instance();

	// Read user's settings...
	my_settings->get_int(PERLIN_ITERATIONS, &iterations);
	my_settings->get_float(PERLIN_SCALE, &scaling);
	my_settings->get_float(PERLIN_PERSISTENCE, &persistence);

	scaling /= 50.0;
	persistence /= 100.0;

	noise = new perlin();

	noise->set_seed(seed);
	noise->set_iterations(iterations);
	noise->set_freq_scale(scaling);
	noise->set_persistence(persistence);
}


/**
 * Destructor
 */
hf_perlin::~hf_perlin()
{
	delete noise;
}



/**
 * Override basic seed_rand
 */
void hf_perlin::seed_rand(unsigned long int s)
{
	noise->set_seed(s);
	hf::seed_rand(s);
}



/**
 * Get height at arbitrary heightfield point...
 */
float hf_perlin::get_height(float x, float y)
{
	if (seed == 0) {
		return -0.1 + offset;
	}

	// XXX p.set_seed(seed);

	float adj_x	= x * float(TERRAIN_WIDTH)/map_size;
	float adj_y	= y * float(TERRAIN_WIDTH)/map_size;
	float height = noise->perlin_noise_2d(256+adj_x/20.0, 256+adj_y/20.0);

	return scale(height) + offset;
}


