//
// hf_perlin.h
//
// Create heightfield using Perlin noise
//
// Copyright (C) J. Belson	2000.1.16
//

#ifndef _HF_PERLIN_H_
#define _HF_PERLIN_H_

#include "hf.h"
#include "perlin.h"

class settings;


/**
 * Create a heightfield using perlin noise
 */
class hf_perlin : public hf {

private:
	enum {
		MAX_ITERATIONS		= 6,
		DEFAULT_ITERATIONS	= 4
	};
	enum { NUM_PRIMES = 600 };

	// Perlin noise generator
	perlin* noise;

	// Which set of primes?
	int set;
	int sets[MAX_ITERATIONS];	// Four random primes from arrays

	long int prime1, prime2, prime3;

	float persistence, scaling;

	// How many noise functions should we sum?
	int iterations;

	static const long int primes[3][NUM_PRIMES];

	virtual int index(int x, int y) {
		return (y*(map_size+1) + x);
	}

	settings* my_settings;

public:
	hf_perlin(void);
	~hf_perlin();

	// Get height at arbitrary heightfield point...
	virtual float get_height(float x, float y);

	// Override basic seed_rand
	virtual void seed_rand(unsigned long int s);

	void set_iterations(int val)	{ iterations = val;		};
	void set_persistence(float val)	{ persistence = val;	};
	void set_scaling(float val)		{ scaling = val;		};
};

#endif	// _HF_PERLIN_H_


