//
// hf_perlin2.cc
//
// From 'Procedural Texturing and Modelling'
//
// My bits Copyright (C) J. Belson 2001.06.17
//
// $Author: jon $ : $Date: 2003/07/26 17:01:33 $
//


#include "hf_perlin2.h"
#include "settings.h"

#include "noise.h"

/**
 * Constructor
 */
hf_perlin2::hf_perlin2(void)
	: hf(),
	  lacunarity(2.0),
	  octaves(10),
	  noise(0)
{
	puts("hf_perlin2::hf_perlin2()");

	// Get required fractal dimension
	float f;
	my_settings->get_float(PERLIN2_FD, &f);

	exponent_array = new double[(int (octaves) + 1)];

	int val;
	my_settings->get_int(PERLIN2_OCTAVES, &val);
	octaves = val;

	my_settings->get_float(PERLIN2_LACUNARITY, &lacunarity);

	// Compute weight for each frequency
	for (int i=0; i<=octaves; i++) {
		exponent_array[i] = std::pow(lacunarity, -i*f);
	}

	srandom(seed);

	my_settings->get_float(PERLIN2_SCALE, &scaling);
	scaling = 0.1 + scaling/120.0;

	noise = new my_noise();
}



/**
 * Destructor
 */
hf_perlin2::~hf_perlin2()
{
	delete noise;
	delete [] exponent_array;
}



/**
 * Get height at arbitrary heightfield point...
 */
float hf_perlin2::get_height(float x, float y)
{
	if (seed == 0) {
		return -0.1 + offset;
	}

	// XXX perl->set_seed(seed);

	double value = 0.0;

	x *= scaling;
	y *= scaling;

	float adj_x	= x * float(TERRAIN_WIDTH)/map_size;
	float adj_y	= y * float(TERRAIN_WIDTH)/map_size;

	//point2d point(x, y);
	point2d point(adj_x, adj_y);

	// Inner loop of spectral construction
	int i;
	for (i=0; i<octaves; i++) {
		value += noise->noise2(point) * exponent_array[i];

		point = point * lacunarity;
	}

	double remainder = octaves - int (octaves);
	
	if (remainder) {		// Add in 'octaves' remainder
		value += remainder*noise->noise2(point)*exponent_array[i];
	}

	return scale(value/1.5) + offset;
}


/**
 * Set number of octaves of noise to use.
 */
void hf_perlin2::set_iterations(int val)
{
	octaves = val;
}



