//
// hf_skydome.h
//
// Create skydome heightfield
//
// Copyright (C) J. Belson	2002.08.16
//

#include <cmath>

#include "hf_skydome.h"

#include "settings.h"


/**
 * Constructor
 */
hf_skydome::hf_skydome(void)
	: hf()
{
	;
}


/**
 * Get height at arbitrary heightfield point...
 */
float hf_skydome::get_height(float x, float y)
{
#if 1
	float radius = SCALE*TERRAIN_WIDTH;
	//if (x*x + y*y > radius*radius)
	//	return -SKYDOME_OFFSET;
	float h = -SKYDOME_OFFSET + std::sqrt(radius*radius - x*x - y*y);

#else

	float h = -SKYDOME_OFFSET +
		SKYDOME_SCALE*std::cos(y/2.0 * M_PI / TERRAIN_WIDTH)*
		std::cos(x/2.0 * M_PI / TERRAIN_WIDTH);
#endif

//printf("%f %f %f\n", x, y, h);

	return h;
}


/**
 * Trace ray to see where it intersects skydome.  This is different
 *  to the terrain case since we want to know when we're above it
 *  rather than below it.
 *
 * On exit	:	false for no intersection
 *				true if path intesects, with p set to the point
 */
bool hf_skydome::trace_path(point3d& p, const vector3d view)
{
	bool got_hit = false;

	vector3d step = view;
	step.normalise();
	step = view * 10.0;
	step.normalise();

	// While we're still beneath skydome...
	const int min = SCALE * -TERRAIN_WIDTH/2;
	const int max = SCALE * TERRAIN_WIDTH/2;
	while (p.x >= min && p.x < max && p.z >= min && p.z < max && p.y < 80) {

		float height = get_height(p.x, p.z);

		// Have reached skydome?
		if (p.y > -5.0 && height <= p.y) {
			got_hit = true;
			break;
		}

		p = p + step;
	}

	return got_hit;
}
