//
// hf_synthesis.cc
//
// Create heightfield using spectral synthesis
//
// Copyright (C) J. Belson	2000.02.05
//

#include "assert.h"
#include <cmath>

#include "hf_synthesis.h"
#include "settings.h"


void cdft2d(int n1, int n2, int isgn, double **a, int *ip, double *w);


hf_synthesis::hf_synthesis(int order, float fd) : hf(order, fd)
{
	;
}

hf_synthesis::~hf_synthesis()
{
	;
}


double **alloc_2d_double(int n1, int n2)
{
    double **dd, *d;
    int j;
    
    dd = (double **) malloc(n1 * sizeof(double *));
    if (dd == NULL) {
//        printf("Allocation Failure!\n");
        exit(1);
    }
    d = (double *) malloc(n1 * n2 * sizeof(double));
    if (d == NULL) {
//        printf("Allocation Failure!\n");
        exit(1);
    }

	for (int i=0; i<n1*n2; i++) {
		d[i] = 0.0;
	}

    dd[0] = d;
    for (j = 1; j < n1; j++) {
        dd[j] = &d[n2 * j];
    }
    return dd;
}


void free_2d_double(double **dd)
{
    free(dd[0]);
    free(dd);
}


int *alloc_1d_int(int n1)
{
    int *i;
    
    i = (int *) malloc(n1 * sizeof(int));
    if (i == NULL) {
//        printf("Allocation Failure!\n");
        exit(1);
    }
    return i;
}


void free_1d_int(int *i)
{
    free(i);
}

double *alloc_1d_double(int n1)
{
    double *d;
    
    d = (double *) malloc(n1 * sizeof(double));
    if (d == NULL) {
//        printf("Allocation Failure!\n");
        exit(1);
    }
    return d;
}


void free_1d_double(double *d)
{
    free(d);
}


//
// Generate height field using spectral synthesis
// Note that generated size is always TERRAIN_WIDTH
//  rather than 'map_size'
//
void hf_synthesis::generate(float std_dev, float h)
{
	if (seed == 0) {
		return;
	}


	// Get required fractal dimension
	my_settings.get_float(SYNTHESIS_FD, &h);

	int val;
	my_settings.get_int(SYNTHESIS_ITERATIONS, &val);
	int iterations = int (pow(2, val));




#if 0
	// Create array for calculation
	float* a;

	int num_elements = (TERRAIN_WIDTH*TERRAIN_WIDTH + 1) * 2;
	a = new float[num_elements];
	memset(a, 0, num_elements * sizeof(float));

	int i0, j0;

	for (int i=0; i<TERRAIN_WIDTH/2; i++) {
		for (int j=0; j<TERRAIN_WIDTH/2; j++) {
		
			float phase = 2.0 * M_PI * ran1();
			float rad;
			if (i != 0 || j != 0) {
				rad = pow(i*i + j*j, -(h+1)/2.0) * gasdev();
			} else {
				rad = 0.0;
			}

			Real(a, i, j) = rad * cos(phase);
			Imag(a, i, j) = rad * sin(phase);
			if (i == 0) {
				i0 = 0;
			} else {
				i0 = TERRAIN_WIDTH - i;
			}
			if (j == 0) {
				j0 = 0;
			} else {
				j0 = TERRAIN_WIDTH - j;
			}

			Real(a, i0, j0) = rad * cos(phase);
			Imag(a, i0, j0) = -rad * sin(phase);
		}
	}

	Imag(a, TERRAIN_WIDTH/2, 0) = 0;
	Imag(a, 0, TERRAIN_WIDTH/2) = 0;
	Imag(a, TERRAIN_WIDTH/2, TERRAIN_WIDTH/2) = 0;

	for (int i=1; i<TERRAIN_WIDTH/2 - 1; i++) {
		for (int j=1; j<TERRAIN_WIDTH/2 - 1; j++) {
			float phrase = 2.0 * M_PI * ran1();
			float rad = pow(i*i + j*j, -(h+1)/2.0) * gasdev();

			Real(a, i, TERRAIN_WIDTH - j) = rad * cos(phrase);
			Imag(a, i, TERRAIN_WIDTH - j) = rad * sin(phrase);
			Real(a, TERRAIN_WIDTH - i, j) = rad * cos(phrase);
			Imag(a, TERRAIN_WIDTH - i, j) = -rad * sin(phrase);
		}
	}


	// Using Numerical Recipes code
	int nsize[3] = { 0, TERRAIN_WIDTH, TERRAIN_WIDTH };

	// Inverse 2d transform
	fourn(a, nsize, 2, -1);	

	// Convert from {real, imaginary} format to array of {real},
	//  scaling as necessary
	float scale = TERRAIN_WIDTH/float(map_size);
	for (int i=0; i<map_size; i++) {
		for (int j=0; j<map_size; j++) {
			relief[index(i, j)] = 6*Real(a, (scale*i)+1, (scale*j)+1);
		}
	}

	delete a;
#endif









#if 1

#define MAX(x,y) ((x) > (y) ? (x) : (y))

    int *ip, n1, n2, n;	//, i;
    double **a, **t, *w;

	n1 = 256;
	n2 = 256;

    a = alloc_2d_double(n1+1, n2*2);
    t = alloc_2d_double(n1+1, n2*2);
    n = MAX(n1, n2 / 2);
    ip = alloc_1d_int(2 + (int) sqrt(n + 0.5));
    n = MAX(n1 / 2, n2 / 4) + MAX(n1, n2);
    w = alloc_1d_double(n);
    ip[0] = 0;


#define R(x,y)	a[(x)][2*(y)]
#define I(x,y)	a[(x)][2*(y) + 1]

	int i0, j0;

	for (int i=0; i<iterations /*TERRAIN_WIDTH*//2; i++) {
		for (int j=0; j<iterations /*TERRAIN_WIDTH*//2; j++) {
		
			float phase = 2.0 * M_PI * ran1();
			float rad;
			if (i != 0 || j != 0) {
				rad = pow(i*i + j*j, -(h+1)/2.0) * gasdev();
			} else {
				rad = 0.0;
			}

			R(i, j) = rad * cos(phase);
			I(i, j) = rad * sin(phase);
			if (i == 0) {
				i0 = 0;
			} else {
				i0 = TERRAIN_WIDTH - i;
			}
			if (j == 0) {
				j0 = 0;
			} else {
				j0 = TERRAIN_WIDTH - j;
			}

			R(i0, j0) = rad * cos(phase);
			I(i0, j0) = -rad * sin(phase);
		}
	}

	I(TERRAIN_WIDTH/2, 0) = 0;
	I(0, TERRAIN_WIDTH/2) = 0;
	I(TERRAIN_WIDTH/2, TERRAIN_WIDTH/2) = 0;

	for (int i=1; i</*TERRAIN_WIDTH*/ iterations/2 - 1; i++) {
		for (int j=1; j</*TERRAIN_WIDTH*/ iterations/2 - 1; j++) {
			float phrase = 2.0 * M_PI * ran1();
			float rad = pow(i*i + j*j, -(h+1)/2.0) * gasdev();

			R(i, TERRAIN_WIDTH - j) = rad * cos(phrase);
			I(i, TERRAIN_WIDTH - j) = rad * sin(phrase);
			R(TERRAIN_WIDTH - i, j) = rad * cos(phrase);
			I(TERRAIN_WIDTH - i, j) = -rad * sin(phrase);
		}
	}



	// Do the transform
    cdft2d(n1, n2*2, 1, a, ip, w);


	// Convert from {real, imaginary} format to array of {real},
	//  scaling as necessary
	float scale = TERRAIN_WIDTH/float(map_size);
	for (int i=0; i<map_size; i++) {
		for (int j=0; j<map_size; j++) {
			relief[i][j] = 6*R(int ((scale*i)+1), int ((scale*j)+1));
		}
	}



#endif



	free_2d_double(t);
	free_2d_double(a);




}

void makewt(int nw, int *ip, double *w)
{
    void bitrv2(int n, int *ip, double *a);
    int nwh, j;
    double delta, x, y;
    
    ip[0] = nw;
    ip[1] = 1;
    if (nw > 2) {
        nwh = nw >> 1;
        delta = atan(1.0) / nwh;
        w[0] = 1;
        w[1] = 0;
        w[nwh] = cos(delta * nwh);
        w[nwh + 1] = w[nwh];
        for (j = 2; j <= nwh - 2; j += 2) {
            x = cos(delta * j);
            y = sin(delta * j);
            w[j] = x;
            w[j + 1] = y;
            w[nw - j] = y;
            w[nw - j + 1] = x;
        }
        bitrv2(nw, ip + 2, w);
    }
}

void bitrv2col(int n1, int n, int *ip, double **a)
{
    int i, j, j1, k, k1, l, m, m2;
    double xr, xi;
    
    ip[0] = 0;
    l = n;
    m = 1;
    while ((m << 2) < l) {
        l >>= 1;
        for (j = 0; j <= m - 1; j++) {
            ip[m + j] = ip[j] + l;
        }
        m <<= 1;
    }
    if ((m << 2) > l) {
        for (i = 0; i <= n1 - 1; i++) {
            for (k = 1; k <= m - 1; k++) {
                for (j = 0; j <= k - 1; j++) {
                    j1 = (j << 1) + ip[k];
                    k1 = (k << 1) + ip[j];
                    xr = a[i][j1];
                    xi = a[i][j1 + 1];
                    a[i][j1] = a[i][k1];
                    a[i][j1 + 1] = a[i][k1 + 1];
                    a[i][k1] = xr;
                    a[i][k1 + 1] = xi;
                }
            }
        }
    } else {
        m2 = m << 1;
        for (i = 0; i <= n1 - 1; i++) {
            for (k = 1; k <= m - 1; k++) {
                for (j = 0; j <= k - 1; j++) {
                    j1 = (j << 1) + ip[k];
                    k1 = (k << 1) + ip[j];
                    xr = a[i][j1];
                    xi = a[i][j1 + 1];
                    a[i][j1] = a[i][k1];
                    a[i][j1 + 1] = a[i][k1 + 1];
                    a[i][k1] = xr;
                    a[i][k1 + 1] = xi;
                    j1 += m2;
                    k1 += m2;
                    xr = a[i][j1];
                    xi = a[i][j1 + 1];
                    a[i][j1] = a[i][k1];
                    a[i][j1 + 1] = a[i][k1 + 1];
                    a[i][k1] = xr;
                    a[i][k1 + 1] = xi;
                }
            }
        }
    }
}


void bitrv2row(int n, int n2, int *ip, double **a)
{
    int i, j, j1, k, k1, l, m;
    double xr, xi;
    
    ip[0] = 0;
    l = n;
    m = 1;
    while ((m << 1) < l) {
        l >>= 1;
        for (j = 0; j <= m - 1; j++) {
            ip[m + j] = ip[j] + l;
        }
        m <<= 1;
    }
    if ((m << 1) > l) {
        for (k = 1; k <= m - 1; k++) {
            for (j = 0; j <= k - 1; j++) {
                j1 = j + ip[k];
                k1 = k + ip[j];
                for (i = 0; i <= n2 - 2; i += 2) {
                    xr = a[j1][i];
                    xi = a[j1][i + 1];
                    a[j1][i] = a[k1][i];
                    a[j1][i + 1] = a[k1][i + 1];
                    a[k1][i] = xr;
                    a[k1][i + 1] = xi;
                }
            }
        }
    } else {
        for (k = 1; k <= m - 1; k++) {
            for (j = 0; j <= k - 1; j++) {
                j1 = j + ip[k];
                k1 = k + ip[j];
                for (i = 0; i <= n2 - 2; i += 2) {
                    xr = a[j1][i];
                    xi = a[j1][i + 1];
                    a[j1][i] = a[k1][i];
                    a[j1][i + 1] = a[k1][i + 1];
                    a[k1][i] = xr;
                    a[k1][i + 1] = xi;
                }
                j1 += m;
                k1 += m;
                for (i = 0; i <= n2 - 2; i += 2) {
                    xr = a[j1][i];
                    xi = a[j1][i + 1];
                    a[j1][i] = a[k1][i];
                    a[j1][i + 1] = a[k1][i + 1];
                    a[k1][i] = xr;
                    a[k1][i + 1] = xi;
                }
            }
        }
    }
}

void cftbcol(int n1, int n, double **a, double *w)
{
    int i, j, j1, j2, j3, k, k1, ks, l, m;
    double wk1r, wk1i, wk2r, wk2i, wk3r, wk3i;
    double x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    
    for (i = 0; i <= n1 - 1; i++) {
        l = 2;
        while ((l << 1) < n) {
            m = l << 2;
            for (j = 0; j <= l - 2; j += 2) {
                j1 = j + l;
                j2 = j1 + l;
                j3 = j2 + l;
                x0r = a[i][j] + a[i][j1];
                x0i = a[i][j + 1] + a[i][j1 + 1];
                x1r = a[i][j] - a[i][j1];
                x1i = a[i][j + 1] - a[i][j1 + 1];
                x2r = a[i][j2] + a[i][j3];
                x2i = a[i][j2 + 1] + a[i][j3 + 1];
                x3r = a[i][j2] - a[i][j3];
                x3i = a[i][j2 + 1] - a[i][j3 + 1];
                a[i][j] = x0r + x2r;
                a[i][j + 1] = x0i + x2i;
                a[i][j2] = x0r - x2r;
                a[i][j2 + 1] = x0i - x2i;
                a[i][j1] = x1r - x3i;
                a[i][j1 + 1] = x1i + x3r;
                a[i][j3] = x1r + x3i;
                a[i][j3 + 1] = x1i - x3r;
            }
            if (m < n) {
                wk1r = w[2];
                for (j = m; j <= l + m - 2; j += 2) {
                    j1 = j + l;
                    j2 = j1 + l;
                    j3 = j2 + l;
                    x0r = a[i][j] + a[i][j1];
                    x0i = a[i][j + 1] + a[i][j1 + 1];
                    x1r = a[i][j] - a[i][j1];
                    x1i = a[i][j + 1] - a[i][j1 + 1];
                    x2r = a[i][j2] + a[i][j3];
                    x2i = a[i][j2 + 1] + a[i][j3 + 1];
                    x3r = a[i][j2] - a[i][j3];
                    x3i = a[i][j2 + 1] - a[i][j3 + 1];
                    a[i][j] = x0r + x2r;
                    a[i][j + 1] = x0i + x2i;
                    a[i][j2] = x2i - x0i;
                    a[i][j2 + 1] = x0r - x2r;
                    x0r = x1r - x3i;
                    x0i = x1i + x3r;
                    a[i][j1] = wk1r * (x0r - x0i);
                    a[i][j1 + 1] = wk1r * (x0r + x0i);
                    x0r = x3i + x1r;
                    x0i = x3r - x1i;
                    a[i][j3] = wk1r * (x0i - x0r);
                    a[i][j3 + 1] = wk1r * (x0i + x0r);
                }
                k1 = 1;
                ks = -1;
                for (k = (m << 1); k <= n - m; k += m) {
                    k1++;
                    ks = -ks;
                    wk1r = w[k1 << 1];
                    wk1i = w[(k1 << 1) + 1];
                    wk2r = ks * w[k1];
                    wk2i = w[k1 + ks];
                    wk3r = wk1r - 2 * wk2i * wk1i;
                    wk3i = 2 * wk2i * wk1r - wk1i;
                    for (j = k; j <= l + k - 2; j += 2) {
                        j1 = j + l;
                        j2 = j1 + l;
                        j3 = j2 + l;
                        x0r = a[i][j] + a[i][j1];
                        x0i = a[i][j + 1] + a[i][j1 + 1];
                        x1r = a[i][j] - a[i][j1];
                        x1i = a[i][j + 1] - a[i][j1 + 1];
                        x2r = a[i][j2] + a[i][j3];
                        x2i = a[i][j2 + 1] + a[i][j3 + 1];
                        x3r = a[i][j2] - a[i][j3];
                        x3i = a[i][j2 + 1] - a[i][j3 + 1];
                        a[i][j] = x0r + x2r;
                        a[i][j + 1] = x0i + x2i;
                        x0r -= x2r;
                        x0i -= x2i;
                        a[i][j2] = wk2r * x0r - wk2i * x0i;
                        a[i][j2 + 1] = wk2r * x0i + wk2i * x0r;
                        x0r = x1r - x3i;
                        x0i = x1i + x3r;
                        a[i][j1] = wk1r * x0r - wk1i * x0i;
                        a[i][j1 + 1] = wk1r * x0i + wk1i * x0r;
                        x0r = x1r + x3i;
                        x0i = x1i - x3r;
                        a[i][j3] = wk3r * x0r - wk3i * x0i;
                        a[i][j3 + 1] = wk3r * x0i + wk3i * x0r;
                    }
                }
            }
            l = m;
        }
        if (l < n) {
            for (j = 0; j <= l - 2; j += 2) {
                j1 = j + l;
                x0r = a[i][j] - a[i][j1];
                x0i = a[i][j + 1] - a[i][j1 + 1];
                a[i][j] += a[i][j1];
                a[i][j + 1] += a[i][j1 + 1];
                a[i][j1] = x0r;
                a[i][j1 + 1] = x0i;
            }
        }
    }
}


void cftbrow(int n, int n2, double **a, double *w)
{
    int i, j, j1, j2, j3, k, k1, ks, l, m;
    double wk1r, wk1i, wk2r, wk2i, wk3r, wk3i;
    double x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    
    l = 1;
    while ((l << 1) < n) {
        m = l << 2;
        for (j = 0; j <= l - 1; j++) {
            j1 = j + l;
            j2 = j1 + l;
            j3 = j2 + l;
            for (i = 0; i <= n2 - 2; i += 2) {
                x0r = a[j][i] + a[j1][i];
                x0i = a[j][i + 1] + a[j1][i + 1];
                x1r = a[j][i] - a[j1][i];
                x1i = a[j][i + 1] - a[j1][i + 1];
                x2r = a[j2][i] + a[j3][i];
                x2i = a[j2][i + 1] + a[j3][i + 1];
                x3r = a[j2][i] - a[j3][i];
                x3i = a[j2][i + 1] - a[j3][i + 1];
                a[j][i] = x0r + x2r;
                a[j][i + 1] = x0i + x2i;
                a[j2][i] = x0r - x2r;
                a[j2][i + 1] = x0i - x2i;
                a[j1][i] = x1r - x3i;
                a[j1][i + 1] = x1i + x3r;
                a[j3][i] = x1r + x3i;
                a[j3][i + 1] = x1i - x3r;
            }
        }
        if (m < n) {
            wk1r = w[2];
            for (j = m; j <= l + m - 1; j++) {
                j1 = j + l;
                j2 = j1 + l;
                j3 = j2 + l;
                for (i = 0; i <= n2 - 2; i += 2) {
                    x0r = a[j][i] + a[j1][i];
                    x0i = a[j][i + 1] + a[j1][i + 1];
                    x1r = a[j][i] - a[j1][i];
                    x1i = a[j][i + 1] - a[j1][i + 1];
                    x2r = a[j2][i] + a[j3][i];
                    x2i = a[j2][i + 1] + a[j3][i + 1];
                    x3r = a[j2][i] - a[j3][i];
                    x3i = a[j2][i + 1] - a[j3][i + 1];
                    a[j][i] = x0r + x2r;
                    a[j][i + 1] = x0i + x2i;
                    a[j2][i] = x2i - x0i;
                    a[j2][i + 1] = x0r - x2r;
                    x0r = x1r - x3i;
                    x0i = x1i + x3r;
                    a[j1][i] = wk1r * (x0r - x0i);
                    a[j1][i + 1] = wk1r * (x0r + x0i);
                    x0r = x3i + x1r;
                    x0i = x3r - x1i;
                    a[j3][i] = wk1r * (x0i - x0r);
                    a[j3][i + 1] = wk1r * (x0i + x0r);
                }
            }
            k1 = 1;
            ks = -1;
            for (k = (m << 1); k <= n - m; k += m) {
                k1++;
                ks = -ks;
                wk1r = w[k1 << 1];
                wk1i = w[(k1 << 1) + 1];
                wk2r = ks * w[k1];
                wk2i = w[k1 + ks];
                wk3r = wk1r - 2 * wk2i * wk1i;
                wk3i = 2 * wk2i * wk1r - wk1i;
                for (j = k; j <= l + k - 1; j++) {
                    j1 = j + l;
                    j2 = j1 + l;
                    j3 = j2 + l;
                    for (i = 0; i <= n2 - 2; i += 2) {
                        x0r = a[j][i] + a[j1][i];
                        x0i = a[j][i + 1] + a[j1][i + 1];
                        x1r = a[j][i] - a[j1][i];
                        x1i = a[j][i + 1] - a[j1][i + 1];
                        x2r = a[j2][i] + a[j3][i];
                        x2i = a[j2][i + 1] + a[j3][i + 1];
                        x3r = a[j2][i] - a[j3][i];
                        x3i = a[j2][i + 1] - a[j3][i + 1];
                        a[j][i] = x0r + x2r;
                        a[j][i + 1] = x0i + x2i;
                        x0r -= x2r;
                        x0i -= x2i;
                        a[j2][i] = wk2r * x0r - wk2i * x0i;
                        a[j2][i + 1] = wk2r * x0i + wk2i * x0r;
                        x0r = x1r - x3i;
                        x0i = x1i + x3r;
                        a[j1][i] = wk1r * x0r - wk1i * x0i;
                        a[j1][i + 1] = wk1r * x0i + wk1i * x0r;
                        x0r = x1r + x3i;
                        x0i = x1i - x3r;
                        a[j3][i] = wk3r * x0r - wk3i * x0i;
                        a[j3][i + 1] = wk3r * x0i + wk3i * x0r;
                    }
                }
            }
        }
        l = m;
    }
    if (l < n) {
        for (j = 0; j <= l - 1; j++) {
            j1 = j + l;
            for (i = 0; i <= n2 - 2; i += 2) {
                x0r = a[j][i] - a[j1][i];
                x0i = a[j][i + 1] - a[j1][i + 1];
                a[j][i] += a[j1][i];
                a[j][i + 1] += a[j1][i + 1];
                a[j1][i] = x0r;
                a[j1][i + 1] = x0i;
            }
        }
    }
}

void cftfcol(int n1, int n, double **a, double *w)
{
    int i, j, j1, j2, j3, k, k1, ks, l, m;
    double wk1r, wk1i, wk2r, wk2i, wk3r, wk3i;
    double x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    
    for (i = 0; i <= n1 - 1; i++) {
        l = 2;
        while ((l << 1) < n) {
            m = l << 2;
            for (j = 0; j <= l - 2; j += 2) {
                j1 = j + l;
                j2 = j1 + l;
                j3 = j2 + l;
                x0r = a[i][j] + a[i][j1];
                x0i = a[i][j + 1] + a[i][j1 + 1];
                x1r = a[i][j] - a[i][j1];
                x1i = a[i][j + 1] - a[i][j1 + 1];
                x2r = a[i][j2] + a[i][j3];
                x2i = a[i][j2 + 1] + a[i][j3 + 1];
                x3r = a[i][j2] - a[i][j3];
                x3i = a[i][j2 + 1] - a[i][j3 + 1];
                a[i][j] = x0r + x2r;
                a[i][j + 1] = x0i + x2i;
                a[i][j2] = x0r - x2r;
                a[i][j2 + 1] = x0i - x2i;
                a[i][j1] = x1r + x3i;
                a[i][j1 + 1] = x1i - x3r;
                a[i][j3] = x1r - x3i;
                a[i][j3 + 1] = x1i + x3r;
            }
            if (m < n) {
                wk1r = w[2];
                for (j = m; j <= l + m - 2; j += 2) {
                    j1 = j + l;
                    j2 = j1 + l;
                    j3 = j2 + l;
                    x0r = a[i][j] + a[i][j1];
                    x0i = a[i][j + 1] + a[i][j1 + 1];
                    x1r = a[i][j] - a[i][j1];
                    x1i = a[i][j + 1] - a[i][j1 + 1];
                    x2r = a[i][j2] + a[i][j3];
                    x2i = a[i][j2 + 1] + a[i][j3 + 1];
                    x3r = a[i][j2] - a[i][j3];
                    x3i = a[i][j2 + 1] - a[i][j3 + 1];
                    a[i][j] = x0r + x2r;
                    a[i][j + 1] = x0i + x2i;
                    a[i][j2] = x0i - x2i;
                    a[i][j2 + 1] = x2r - x0r;
                    x0r = x1r + x3i;
                    x0i = x1i - x3r;
                    a[i][j1] = wk1r * (x0i + x0r);
                    a[i][j1 + 1] = wk1r * (x0i - x0r);
                    x0r = x3i - x1r;
                    x0i = x3r + x1i;
                    a[i][j3] = wk1r * (x0r + x0i);
                    a[i][j3 + 1] = wk1r * (x0r - x0i);
                }
                k1 = 1;
                ks = -1;
                for (k = (m << 1); k <= n - m; k += m) {
                    k1++;
                    ks = -ks;
                    wk1r = w[k1 << 1];
                    wk1i = w[(k1 << 1) + 1];
                    wk2r = ks * w[k1];
                    wk2i = w[k1 + ks];
                    wk3r = wk1r - 2 * wk2i * wk1i;
                    wk3i = 2 * wk2i * wk1r - wk1i;
                    for (j = k; j <= l + k - 2; j += 2) {
                        j1 = j + l;
                        j2 = j1 + l;
                        j3 = j2 + l;
                        x0r = a[i][j] + a[i][j1];
                        x0i = a[i][j + 1] + a[i][j1 + 1];
                        x1r = a[i][j] - a[i][j1];
                        x1i = a[i][j + 1] - a[i][j1 + 1];
                        x2r = a[i][j2] + a[i][j3];
                        x2i = a[i][j2 + 1] + a[i][j3 + 1];
                        x3r = a[i][j2] - a[i][j3];
                        x3i = a[i][j2 + 1] - a[i][j3 + 1];
                        a[i][j] = x0r + x2r;
                        a[i][j + 1] = x0i + x2i;
                        x0r -= x2r;
                        x0i -= x2i;
                        a[i][j2] = wk2r * x0r + wk2i * x0i;
                        a[i][j2 + 1] = wk2r * x0i - wk2i * x0r;
                        x0r = x1r + x3i;
                        x0i = x1i - x3r;
                        a[i][j1] = wk1r * x0r + wk1i * x0i;
                        a[i][j1 + 1] = wk1r * x0i - wk1i * x0r;
                        x0r = x1r - x3i;
                        x0i = x1i + x3r;
                        a[i][j3] = wk3r * x0r + wk3i * x0i;
                        a[i][j3 + 1] = wk3r * x0i - wk3i * x0r;
                    }
                }
            }
            l = m;
        }
        if (l < n) {
            for (j = 0; j <= l - 2; j += 2) {
                j1 = j + l;
                x0r = a[i][j] - a[i][j1];
                x0i = a[i][j + 1] - a[i][j1 + 1];
                a[i][j] += a[i][j1];
                a[i][j + 1] += a[i][j1 + 1];
                a[i][j1] = x0r;
                a[i][j1 + 1] = x0i;
            }
        }
    }
}


void cftfrow(int n, int n2, double **a, double *w)
{
    int i, j, j1, j2, j3, k, k1, ks, l, m;
    double wk1r, wk1i, wk2r, wk2i, wk3r, wk3i;
    double x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    
    l = 1;
    while ((l << 1) < n) {
        m = l << 2;
        for (j = 0; j <= l - 1; j++) {
            j1 = j + l;
            j2 = j1 + l;
            j3 = j2 + l;
            for (i = 0; i <= n2 - 2; i += 2) {
                x0r = a[j][i] + a[j1][i];
                x0i = a[j][i + 1] + a[j1][i + 1];
                x1r = a[j][i] - a[j1][i];
                x1i = a[j][i + 1] - a[j1][i + 1];
                x2r = a[j2][i] + a[j3][i];
                x2i = a[j2][i + 1] + a[j3][i + 1];
                x3r = a[j2][i] - a[j3][i];
                x3i = a[j2][i + 1] - a[j3][i + 1];
                a[j][i] = x0r + x2r;
                a[j][i + 1] = x0i + x2i;
                a[j2][i] = x0r - x2r;
                a[j2][i + 1] = x0i - x2i;
                a[j1][i] = x1r + x3i;
                a[j1][i + 1] = x1i - x3r;
                a[j3][i] = x1r - x3i;
                a[j3][i + 1] = x1i + x3r;
            }
        }
        if (m < n) {
            wk1r = w[2];
            for (j = m; j <= l + m - 1; j++) {
                j1 = j + l;
                j2 = j1 + l;
                j3 = j2 + l;
                for (i = 0; i <= n2 - 2; i += 2) {
                    x0r = a[j][i] + a[j1][i];
                    x0i = a[j][i + 1] + a[j1][i + 1];
                    x1r = a[j][i] - a[j1][i];
                    x1i = a[j][i + 1] - a[j1][i + 1];
                    x2r = a[j2][i] + a[j3][i];
                    x2i = a[j2][i + 1] + a[j3][i + 1];
                    x3r = a[j2][i] - a[j3][i];
                    x3i = a[j2][i + 1] - a[j3][i + 1];
                    a[j][i] = x0r + x2r;
                    a[j][i + 1] = x0i + x2i;
                    a[j2][i] = x0i - x2i;
                    a[j2][i + 1] = x2r - x0r;
                    x0r = x1r + x3i;
                    x0i = x1i - x3r;
                    a[j1][i] = wk1r * (x0i + x0r);
                    a[j1][i + 1] = wk1r * (x0i - x0r);
                    x0r = x3i - x1r;
                    x0i = x3r + x1i;
                    a[j3][i] = wk1r * (x0r + x0i);
                    a[j3][i + 1] = wk1r * (x0r - x0i);
                }
            }
            k1 = 1;
            ks = -1;
            for (k = (m << 1); k <= n - m; k += m) {
                k1++;
                ks = -ks;
                wk1r = w[k1 << 1];
                wk1i = w[(k1 << 1) + 1];
                wk2r = ks * w[k1];
                wk2i = w[k1 + ks];
                wk3r = wk1r - 2 * wk2i * wk1i;
                wk3i = 2 * wk2i * wk1r - wk1i;
                for (j = k; j <= l + k - 1; j++) {
                    j1 = j + l;
                    j2 = j1 + l;
                    j3 = j2 + l;
                    for (i = 0; i <= n2 - 2; i += 2) {
                        x0r = a[j][i] + a[j1][i];
                        x0i = a[j][i + 1] + a[j1][i + 1];
                        x1r = a[j][i] - a[j1][i];
                        x1i = a[j][i + 1] - a[j1][i + 1];
                        x2r = a[j2][i] + a[j3][i];
                        x2i = a[j2][i + 1] + a[j3][i + 1];
                        x3r = a[j2][i] - a[j3][i];
                        x3i = a[j2][i + 1] - a[j3][i + 1];
                        a[j][i] = x0r + x2r;
                        a[j][i + 1] = x0i + x2i;
                        x0r -= x2r;
                        x0i -= x2i;
                        a[j2][i] = wk2r * x0r + wk2i * x0i;
                        a[j2][i + 1] = wk2r * x0i - wk2i * x0r;
                        x0r = x1r + x3i;
                        x0i = x1i - x3r;
                        a[j1][i] = wk1r * x0r + wk1i * x0i;
                        a[j1][i + 1] = wk1r * x0i - wk1i * x0r;
                        x0r = x1r - x3i;
                        x0i = x1i + x3r;
                        a[j3][i] = wk3r * x0r + wk3i * x0i;
                        a[j3][i + 1] = wk3r * x0i - wk3i * x0r;
                    }
                }
            }
        }
        l = m;
    }
    if (l < n) {
        for (j = 0; j <= l - 1; j++) {
            j1 = j + l;
            for (i = 0; i <= n2 - 2; i += 2) {
                x0r = a[j][i] - a[j1][i];
                x0i = a[j][i + 1] - a[j1][i + 1];
                a[j][i] += a[j1][i];
                a[j][i + 1] += a[j1][i + 1];
                a[j1][i] = x0r;
                a[j1][i + 1] = x0i;
            }
        }
    }
}






//
// Main FFT routine
//
void cdft2d(int n1, int n2, int isgn, double **a, int *ip, double *w)
{

    int n;
    
    n = n1 << 1;
    if (n < n2) {
        n = n2;
    }
    if (n > (ip[0] << 2)) {
        makewt(n >> 2, ip, w);
    }
    if (n2 > 4) {
        bitrv2col(n1, n2, ip + 2, a);
    }
    if (n1 > 2) {
        bitrv2row(n1, n2, ip + 2, a);
    }
    if (isgn < 0) {
        cftfcol(n1, n2, a, w);
        cftfrow(n1, n2, a, w);
    } else {
        cftbcol(n1, n2, a, w);
        cftbrow(n1, n2, a, w);
    }
}


void bitrv2(int n, int *ip, double *a)
{
    int j, j1, k, k1, l, m, m2;
    double xr, xi;
    
    ip[0] = 0;
    l = n;
    m = 1;
    while ((m << 2) < l) {
        l >>= 1;
        for (j = 0; j <= m - 1; j++) {
            ip[m + j] = ip[j] + l;
        }
        m <<= 1;
    }
    if ((m << 2) > l) {
        for (k = 1; k <= m - 1; k++) {
            for (j = 0; j <= k - 1; j++) {
                j1 = (j << 1) + ip[k];
                k1 = (k << 1) + ip[j];
                xr = a[j1];
                xi = a[j1 + 1];
                a[j1] = a[k1];
                a[j1 + 1] = a[k1 + 1];
                a[k1] = xr;
                a[k1 + 1] = xi;
            }
        }
    } else {
        m2 = m << 1;
        for (k = 1; k <= m - 1; k++) {
            for (j = 0; j <= k - 1; j++) {
                j1 = (j << 1) + ip[k];
                k1 = (k << 1) + ip[j];
                xr = a[j1];
                xi = a[j1 + 1];
                a[j1] = a[k1];
                a[j1 + 1] = a[k1 + 1];
                a[k1] = xr;
                a[k1 + 1] = xi;
                j1 += m2;
                k1 += m2;
                xr = a[j1];
                xi = a[j1 + 1];
                a[j1] = a[k1];
                a[j1 + 1] = a[k1 + 1];
                a[k1] = xr;
                a[k1 + 1] = xi;
            }
        }
    }
}








































void hf_synthesis::InvFFT(float* relief, float* relief_buffer, int map_size)
{
	;
}



/*      FOURN  --  Multi-dimensional fast Fourier transform 

        Called with arguments: 

           data       A  one-dimensional  array  of  floats  (NOTE!!!   NOT 
                      DOUBLES!!), indexed from one (NOTE!!!   NOT  ZERO!!), 
                      containing  pairs of numbers representing the complex 
                      valued samples.  The Fourier transformed results  are 
                      returned in the same array. 

           nn         An  array specifying the edge size in each dimension. 
                      THIS ARRAY IS INDEXED FROM  ONE,  AND  ALL  THE  EDGE 
                      SIZES MUST BE POWERS OF TWO!!! 

           ndim       Number of dimensions of FFT to perform.  Set to 2 for 
                      two dimensional FFT. 

           isign      If 1, a Fourier transform is done; if -1 the  inverse 
                      transformation is performed. 

        This  function  is essentially as given in Press et al., "Numerical 
        Recipes In C", Section 12.11, pp.  467-470. 
*/ 

void hf_synthesis::fourn(float* data, int* nn, int ndim, int isign) 
{ 
    register int i1, i2, i3; 
    int i2rev, i3rev, ip1, ip2, ip3, ifp1, ifp2; 
    int ibit, idim, k1, k2, n, nprev, nrem, ntot; 
    float tempi, tempr; 
    double theta, wi, wpi, wpr, wr, wtemp; 

#define SWAP(a,b) tempr=(a); (a) = (b); (b) = tempr 

    ntot = 1; 
    for (idim = 1; idim <= ndim; idim++) 
        ntot *= nn[idim]; 
    nprev = 1; 
    for (idim = ndim; idim >= 1; idim--) { 
        n = nn[idim]; 
        nrem = ntot / (n * nprev); 
        ip1 = nprev << 1; 
        ip2 = ip1 * n; 
        ip3 = ip2 * nrem; 
        i2rev = 1; 
        for (i2 = 1; i2 <= ip2; i2 += ip1) { 
            if (i2 < i2rev) { 
                for (i1 = i2; i1 <= i2 + ip1 - 2; i1 += 2) { 
                    for (i3 = i1; i3 <= ip3; i3 += ip2) { 
                        i3rev = i2rev + i3 - i2; 
                        SWAP(data[i3], data[i3rev]); 
                        SWAP(data[i3 + 1], data[i3rev + 1]); 
                    } 
                } 
            } 
            ibit = ip2 >> 1; 
            while (ibit >= ip1 && i2rev > ibit) { 
                i2rev -= ibit; 
                ibit >>= 1; 
            } 
            i2rev += ibit; 
        } 
        ifp1 = ip1; 
        while (ifp1 < ip2) { 
            ifp2 = ifp1 << 1; 
            theta = isign * (M_PI * 2) / (ifp2 / ip1); 
            wtemp = sin(0.5 * theta); 
            wpr = -2.0 * wtemp * wtemp; 
            wpi = sin(theta); 
            wr = 1.0; 
            wi = 0.0; 
            for (i3 = 1; i3 <= ifp1; i3 += ip1) { 
                for (i1 = i3; i1 <= i3 + ip1 - 2; i1 += 2) { 
                    for (i2 = i1; i2 <= ip3; i2 += ifp2) { 
                        k1 = i2; 
                        k2 = k1 + ifp1; 
                        tempr = wr * data[k2] - wi * data[k2 + 1]; 
                        tempi = wr * data[k2 + 1] + wi * data[k2]; 
                        data[k2] = data[k1] - tempr; 
                        data[k2 + 1] = data[k1 + 1] - tempi; 
                        data[k1] += tempr; 
                        data[k1 + 1] += tempi; 
                    } 
                } 
                wr = (wtemp = wr) * wpr - wi * wpi + wr; 
                wi = wi * wpr + wtemp * wpi + wi; 
            } 
            ifp1 = ifp2; 
        } 
        nprev *= n; 
    } 
} 
#undef SWAP


