//
// hfield.cc
//
// Front end to fractal generation classes
//
// Copyright (c) J. Belson	2001.03.13
//


#include <assert.h>

#include "hf.h"
#include "hfield.h"
#include "hf_multifractal.h"
#include "hf_perlin.h"
#include "hf_perlin2.h"
#include "hf_skydome.h"
#include "hf_ridged_perlin.h"
#include "settings.h"


/**
 * Constructor
 */
hfield::hfield(hf_type t)
{
	switch (t) {
		case HF_PERLIN:
			heightfield = new hf_perlin();
			break;

		case HF_PERLIN2:
			heightfield = new hf_perlin2();
			break;

		case HF_MULTIFRACTAL:
			heightfield = new hf_multifractal();
			break;

		case HF_RIDGED_PERLIN:
			heightfield = new hf_ridged_perlin();
			break;

		case HF_SKYDOME:
			heightfield = new hf_skydome();
			break;

		default:
			std::cerr << "Panic!  Unknown fractal type!" << std::endl;
			exit (1);
	}
}


/**
 * Destructor.
 */
hfield::~hfield()
{
	delete heightfield;
}


/**
 * Effective size of terrain
 */
int hfield::get_width(void) const
{
	return heightfield->get_width();
}


/**
 * Get height at point...
 */
float hfield::get_height(float x, float y) const
{
	float val = heightfield->get_height(x, y);
return val;
	return 12.0 * (val - height_min)/(height_max - height_min);
}


/**
 * Get height at point...
 */
vector3d hfield::get_normal(float x, float y) const
{
	return heightfield->get_normal(x, y);
}


/**
 * Vertical offset
 */
void hfield::set_offset(float f)
{
	heightfield->set_offset(f);
}


/**
 * Trace ray to see if it intersects terrain
 */
bool hfield::trace_path(point3d& p, vector3d view)
{
	return heightfield->trace_path(p, view);
}


/**
 * Seed heighfield's random number generator
 */
void hfield::seed_rand(unsigned long int s)
{
	printf("hfield::seed_rand(%ld)\n", s);

	heightfield->seed_rand(s);
	get_stats();
	heightfield->seed_rand(s);
}


/**
 * Get approximation to min/max height
 */
void hfield::get_stats(void)
{
	height_min = 100.0;
	height_max = -100.0;

	for (int i=0; i<1000; i++) {
		float val = heightfield->get_height(
						(random()/double(RAND_MAX/2)) - 1.0,
						(random()/double(RAND_MAX/2)) - 1.0
						);
		if (val > height_max)	height_max = val;
		if (val < height_min)	height_min = val;
	}

	printf("min = %f, max = %f, range = %f\n",
			height_min, height_max, height_max-height_min);
}

