//
// hfield.h
//
// Front end to fractal generation classes
//
// Copyright (c) J. Belson	2001.03.13
//

#ifndef _HFIELD_H_
#define _HFIELD_H_

#include <iostream>

#include "point3d.h"
#include "settings.h"
#include "vector3d.h"

class hf;


/**
 * Interface to heightfield classes
 */
class hfield {

private:
	hf_type type;

	hf* heightfield;

	// Statistics for height values read	
	float		height_min;
	float		height_max;

public:
	hfield(hf_type t);
	~hfield();

	// Effective size of terrain
	int get_width(void) const;

	// Get height at arbitrary heightfield point...
	float get_height(float x, float y) const;

	// Get height at point...
	vector3d get_normal(float x, float y) const;

	// Vertical offset
	void set_offset(float f);

	// Trace ray to see if it intersects terrain
	bool trace_path(point3d& p, vector3d view);

	void seed_rand(unsigned long int s);

	void get_stats(void);

};

#endif	// _HFIELD_H_


