//
// perlin.h
//
// Perlin noise generator
//
// Copyright (C) J. Belson	2000.02.19
//


#ifndef _PERLIN_H_
#define _PERLIN_H_

#include "perlin.h"


///
/// Perlin noise generator derived from Hugo Elias's
///  web page.
///
class perlin {

private:
	enum {
		NUM_PRIMES		= 600,
		MAX_ITERATIONS	= 6,
		DEFAULT_WIDTH	= 256
	};

	static const float	DEFAULT_PERSISTENCE;
	static const float	DEFAULT_FREQ_SCALE;
	static const int	DEFAULT_ITERATIONS;

	// Used for noise generation
	static const long int primes[3][NUM_PRIMES];
	long int prime1, prime2, prime3;
	int sets[MAX_ITERATIONS];

	float persistence;
	float freq_scale;
	int iterations;
	int width;

	inline float noise(long int x, long int y);
	float interpolate(float a, float b, float x);
	float linear_interpolate(float a, float b, float x);
	float smoothed_noise(int x, int y);
	float interpolated_noise(float x, float y);

public:
	perlin();

	// Return noise value at x, y
	float perlin_noise_2d(float x, float y);

	void set_iterations(int val);
	void set_persistence(float val);
	void set_freq_scale(float val);
	void set_width(int val);
	void set_seed(int val);
};

#endif	// _PERLIN_H_
