//
// skylight.h
//
// Model for skylight based on sun position and turbidity
//
// Implemented from paper "A Practical Analytic Model for Daylight"
// A. J. Preetham, Peter Shirley, Brian Smits, Siggraph 99
// (I left out all the difficult bits :-)
//
// Copyright (C) J. Belson	2000.05.19
//
// $Author: jon $	:	$Date: 2003/04/09 14:54:12 $
//

#ifndef _SKYLIGHT_H_
#define _SKYLIGHT_H_

#include "fcolour.h"
#include "vector3d.h"



/**
 * Mathematical model of skydome colour
 */
class skylight {

private:
	struct coeff {
		double A, B, C, D, E, F;
	};

	// 'Density' of the atmosphere
	double turbidity;

	float lum_scale;

	// Coefficients for current turbidity
	coeff c_lum;
	coeff c_x;
	coeff c_y;

	vector3d sun;

	double largest;

	// Calculate parameters for given turbidity
	coeff coeff_luminance(void);
	coeff coeff_x(void);
	coeff coeff_y(void);

	// Return value for specified viewing angle
	double zenith_luminance(double theta);
	double zenith_x(double theta);
	double zenith_y(double theta);

	// Convert xyY to RGB
	fcolour cie_to_rgb(double x, double y, double lum) const;

	// Evaluate Perez function
	double perez_function(double theta, double gamma, coeff c);

	// Perform exposure function on 'val'
	float exposure(float val);

public:
	skylight(double t = 3.0);

	void set_luminance(float lum) { lum_scale = lum; }
	void set_turbidity(float t);
	void set_sun_direction(const vector3d& v);

	// Calculate colour in specified direction
	fcolour get_colour(const vector3d& v);

};


#endif	// _SKYLIGHT_H_

