//
// zbuffer.cc
//
// Handle depth buffer operations
//
// Copyright (c) J. Belson	1999.03.14
//


#include "zbuffer.h"


///
/// Constructor
///
zbuffer::zbuffer(int w, int h) : width(w), height(h)
{
	buffer = new float [width * height];
}


///
/// Destructor
///
zbuffer::~zbuffer()
{
	delete [] buffer;
}


///
/// Initialise zbuffer to large negative value, corresponding
///  to large distance.
///
void zbuffer::clear_buffer(void)
{
	for (int i=0; i<width*height; i++) {
		buffer[i] = -10000.0;
	}
}


///
/// Set a whole line of z buffer to a height value.
/// @param line	y coordinate of row
/// @param x1		start of line
/// @param x2		end of line
/// @param depth	depth value ot set
///
void zbuffer::set_line(int line, int x1, int x2, float depth)
{
	// Make sure x1 < x2
	if (x1 > x2) {
		int temp = x1;
		x1 = x2;
		x2 = temp;		
	}

	float *pos = buffer + line*width + x1;
	
	for (int i=0; i<x2-x1; i++) {
		*pos++ = depth;
	}
}


///
/// Return address of pixel 'x' on row 'line'
///
float *zbuffer::get_buffer(int line, int x)
{
	return buffer + line*width + x;
}


