//
// interpolate.cc
//
// Provide smooth steps between values
//
// Copyright (C) J. Belson	2000.02.03
//


#include "interpolate.h"



/**
 * Three floating point values
 */
interpolate_3f::interpolate_3f(float start1, float start2, float start3,
				float end1, float end2, float end3, int step)
{
	n1 = start1;
	n2 = start2;
	n3 = start3;

	n1_inc = n2_inc = n3_inc = 0.0;

	if (step) {
		n1_inc = (end1 - start1)/step;
		n2_inc = (end2 - start2)/step;
		n3_inc = (end3 - start3)/step;
	}
}

void interpolate_3f::set(float start1, float start2, float start3,
				float end1, float end2, float end3, int step)
{
	n1 = start1;
	n2 = start2;
	n3 = start3;

	n1_inc = n2_inc = n3_inc = 0.0;

	if (step) {
		n1_inc = (end1 - start1)/step;
		n2_inc = (end2 - start2)/step;
		n3_inc = (end3 - start3)/step;
	}
}

void interpolate_3f::next(void)
{
	n1 += n1_inc;
	n2 += n2_inc;
	n3 += n3_inc;
}


/**
 * Two floating point values
 */
interpolate_2f::interpolate_2f(float start1, float start2,
				float end1, float end2, int step)
{
	n1 = start1;
	n2 = start2;

	n1_inc = n2_inc = 0.0;

	if (step) {
		n1_inc = (end1 - start1)/step;
		n2_inc = (end2 - start2)/step;
	}
}

void interpolate_2f::set(float start1, float start2,
				float end1, float end2, int step)
{
	n1 = start1;
	n2 = start2;

	n1_inc = n2_inc = 0.0;

	if (step) {
		n1_inc = (end1 - start1)/step;
		n2_inc = (end2 - start2)/step;
	}
}

void interpolate_2f::next(void)
{
	n1 += n1_inc;
	n2 += n2_inc;
}



/**
 * Single floating point
 */
interpolate_f::interpolate_f(float start, float end, int step)
{
	n = start;

	n_inc = 0.0;

	if (step) {
		n_inc = (end - start)/step;
	}
}

void interpolate_f::set(float start, float end, int step)
{
	n = start;

	n_inc = 0.0;

	if (step) {
		n_inc = (end - start)/step;
	}
}

void interpolate_f::next(void)
{
	n += n_inc;
}



/**
 * Single integer
 */
interpolate_i::interpolate_i(int start, int end, int steps)
{
	a = start << SHIFT;

	inc = 0;
	
	if (steps) {
		inc = ((end - start)<<SHIFT)/steps;
	}
}

void interpolate_i::set(int start, int end, int steps)
{
	a = start << SHIFT;

	inc = 0;
	
	if (steps) {
		inc = ((end - start)<<SHIFT)/steps;
	}
}


void interpolate_i::next(void)
{
	a += inc;
}



/**
 * Three integers
 */
interpolate_3i::interpolate_3i(
		int a1, int a2, int a3,
		int b1, int b2, int b3,
		int steps)
{
	a = a1 << SHIFT;
	b = a2 << SHIFT;
	c = a3 << SHIFT;

	inc_a = inc_b = inc_c = 0;

	if (steps) {
		inc_a = ((b1 - a1)<<SHIFT)/steps;
		inc_b = ((b2 - a2)<<SHIFT)/steps;
		inc_c = ((b3 - a3)<<SHIFT)/steps;
	}
}

void interpolate_3i::set(
		int a1, int a2, int a3,
		int b1, int b2, int b3,
		int steps)
{
	a = a1 << SHIFT;
	b = a2 << SHIFT;
	c = a3 << SHIFT;

	inc_a = inc_b = inc_c = 0;

	if (steps) {
		inc_a = ((b1 - a1)<<SHIFT)/steps;
		inc_b = ((b2 - a2)<<SHIFT)/steps;
		inc_c = ((b3 - a3)<<SHIFT)/steps;
	}
}

void interpolate_3i::next(void)
{
	a += inc_a;
	b += inc_b;
	c += inc_c;
}




