//
// OS-independent code for OpenGL rendering
//
// Copyright (c) J. Belson	2000.11.04
//
// $Author: jon $	:	$Date: 2003/10/11 18:11:00 $
//

#ifndef _BASE_GLWINDOW_H_
#define _BASE_GLWINDOW_H_


#include "fcolour.h"
#include "frustrum.h"
#include "gl_include.h"
#include "matrix.h"
#include "point3d.h"
#include "vector3d.h"


class hfield;
class settings;


/**
 * Platform independent base class for an OpenGL preview display window
 */
class base_GLwindow
{

private:
	float		light_i, light_j, light_k;

	// Current viewer position and view direction
	point3d		viewpoint;
	matrix		camera;
	vector3d	heading;

	float		fov;
	frustrum	view_frustrum;

	// Vertices and surface normals for a model
	struct object {
		point3d*	points;
		vector3d*	normals;
		object()		{ points = 0; normals = 0;				}
		~object()		{ clear();								}
		void clear()	{ delete [] points; delete [] normals;	} 
	} land, sky;

	int		map_width;		// 2^detail
	int		detail;
	bool	texturing;
	bool	show_normals;

	// Window size
	int width, height;

	float pos_x, pos_y, pos_z;	// Current position
	float rot_a, rot_b, rot_c;	// Current orientation

	static const GLfloat fog_colour[];
	static const GLfloat sky_colour[];

	// Cloud settings
	fcolour horizon_colour;
	fcolour zenith_colour;
	fcolour	cloud_colour;
	float	persistence;
	float	presence;
	float	cloud_cover;
	float	scale;
	int		seed;

	// Draw some surface normals for debugging purposes
	void draw_normals(void);

	/* Execute GL commands for given vertex details */
	void add_point(const vector3d& normal, const point3d& vertex, const fcolour& col);
	void add_point(object& obj, int idx) const;

protected:
	settings* my_settings;

public:
	base_GLwindow(int w, int h);
	virtual ~base_GLwindow();

	virtual void GLinit(void);

	// Create model from supplied heightfield
	void create_land(const hfield* heightfield);
	void create_sky(const hfield* heightfield);

	// Render land and sky
	virtual void render(bool draw_land=true, bool draw_sky=true);

	// Set viewpoint and viewing direction
	void set_view(const point3d& eye, const point3d& target);

	void set_detail(int val);
	void set_texture(bool state);

	// Switch between rendering modes
	virtual void wireframe(void);
	virtual void flat(void);
	virtual void smooth(void);

	// Various rendering states
	virtual void set_lighting(bool state);
	virtual void set_fogging(bool state);
	virtual void set_shadows(bool state);
	virtual void set_light_position(float x, float y, float z);
};

#endif	// _BASE_GLWINDOW_H_

