//
// base_cloud_window.cc
//
// Prefs for cloud generation
//
// Copyright (c) J. Belson	1999.04.05
//
// $Author: jon $	:	$Date: 2003/10/11 18:11:00 $
//

#include <cmath>

#include "base_cloud_window.h"
#include "fcolour.h"
#include "perlin.h"
#include "texture_cloud.h"
#include "types.h"


const float base_cloud_window::DEFAULT_PERSISTENCE	= 0.5;
const float base_cloud_window::DEFAULT_PRESENCE		= 5.0;
const float base_cloud_window::DEFAULT_CLOUD_COVER	= 0.0;
const float base_cloud_window::DEFAULT_SCALE		= 0.1;


/**
 * Constructor
 */
base_cloud_window::base_cloud_window()
{
	my_settings = settings::get_instance();

	load_settings();

	quit_requested = false;
}


/**
 * Destructor
 */
base_cloud_window::~base_cloud_window()
{
	;
}


/**
 * Render a cloud preview using current settings
 */
void base_cloud_window::render_thread(void)
{
	textset_cloud ts;
	textset_cloud::load_settings(ts);
/*
	ts.seed			= seed;
	ts.fg			= cloud_colour;
	ts.persistence	= persistence;
	ts.scale		= scale;
	ts.cloud_cover	= cloud_cover;
	ts.presence		= presence;
	ts.zenith_colour = fcolour(90/255.0, 150/255.0, 231/255.0);
	ts.horizon_colour = fcolour(90/255.0, 150/255.0, 231/255.0);
*/
	texture* cloud = new texture_cloud(ts);

	const int bpr	= cloud_view->get_bytes_per_row();
	uint32* p		= (uint32*) cloud_view->get_bitmap_data();

	fcolour sky(90/255.0, 150/255.0, 231/255.0);

	for (int y=0; y<CLOUD_WIDTH; y++) {
		if (quit_requested) {
			return;
		}

		for (int x=0; x<CLOUD_WIDTH; x++) {
			fcolour colour = cloud->colour(x, y);
			colour = sky.blend(colour);
			p[x] = fcolour::get_rgb32(colour);
		}

		p += bpr/4;	// Next row...
	}

	delete cloud;
}


/**
 * Cloud colour changed
 */
void base_cloud_window::changed_colour(const QColor& col)
{
	unsigned char r = col.red();
	unsigned char g = col.green();
	unsigned char b = col.blue();

	cloud_colour = fcolour(r/255.0, g/255.0, b/255.0);
}


/**
 * Cloud persistence changed
 */
void base_cloud_window::changed_persistence(int val)
{
	persistence = float(val)/MAX_VALUE;
}


/**
 * Cloud presence changed
 */
void base_cloud_window::changed_presence(int val)
{
	presence = 10.0*float(val)/MAX_VALUE;
}


/**
 * Cloud cover changed
 */
void base_cloud_window::changed_cover(int val)
{
	cloud_cover = -4.0 + 8.0*float(val)/MAX_VALUE;
}


/**
 * Cloud scale changed
 */
void base_cloud_window::changed_scale(int val)
{
	scale = 0.2 * float(val)/MAX_VALUE;
}


/**
 * Read current preferences
 */
void base_cloud_window::load_settings(void)
{
	my_settings->get_colour(CLOUD_COLOUR, &cloud_colour);
	my_settings->get_float(CLOUD_PERSISTENCE, &persistence);
	my_settings->get_float(CLOUD_PRESENCE, &presence);
	my_settings->get_float(CLOUD_COVER, &cloud_cover);
	my_settings->get_float(CLOUD_SCALE, &scale);

	my_settings->get_int(CLOUD_SEED, &seed);
}

