//
// base_main_window.cc
//
// Main control window
//
// Copyright (c) J. Belson	1998.12.12
//


#include "colour.h"
#include "fcolour.h"
#include "main_window.h"
#include "point3d.h"
#include "settings.h"
#include "texture.h"


static fcolour col_array[] = {

    fcolour(0.0,    	0.0,		0.0),
    fcolour(78/256.0,   196/256.0,  74/256.0),
    fcolour(40/256.0,   166/256.0,  40/256.0),
    fcolour(136/256.0,  0.31,       0.31),
    fcolour(0.87,       0.84,       0.96),
    fcolour(1.0,        1.0,        1.0)
};


/**
 * Constructor
 */
base_main_window::base_main_window()
{
	my_settings = settings::get_instance();

	load_settings();

	seed	= 0;			// Ditto rng seed
	randomise = true;		// Randomise seed for each new generation

	w_render		= 0;
	w_preview		= 0;
	w_sky			= 0;
	w_cloud			= 0;
	w_lighting		= 0;
	w_palette		= 0;
	w_trees			= 0;
	w_atmospherics	= 0;
	w_texture		= 0;
	w_water			= 0;

	my_map = 0;
}


/**
 * Destructor
 */
base_main_window::~base_main_window()
{
	if (my_map)	delete my_map;
}


/**
 * Render relief view using current hfield
 * @param	p32				Pointer to start of 32 bit buffer
 * @param	bytes_per_row	Bytes offset from start of line 'n' to
 *							 start of line 'n+1'
 */
void base_main_window::render_relief(uint32* p32, int bytes_per_row)
{
	float water_level;
	my_settings->get_float(WATER_LEVEL, &water_level);

	if (my_map) {
		delete my_map;
	}
	my_map = new hfield(get_type());
	my_map->seed_rand(get_seed());
	my_map->set_offset(-water_level);

	static const int terrain_width = my_map->get_width();

	const int min	= -terrain_width/2;
	const int max	=  terrain_width/2;
	const float step= 1.0;

	int line = 0;
	int offset = 0;

	for (float y=min; y<max; y+=step) {

		offset = 0;
		p32 += bytes_per_row/4;

		for (float x=min; x<max; x+=step) {

			point3d p(x, 0, y);
			float height = my_map->get_height(p.x, p.z);

			if (height < -4.0) {
				p32[offset] = fcolour::make_rgb32(55, 55, 230);
			} else if (height < 0.0) {
				p32[offset] = fcolour::make_rgb32(60, 60, 245);
			} else {
				int h = int (10.0 * height);

				if (h > 250)	h = 250;
				if (h < 0)		h = 0;

				float param = h/256.0;

				fcolour c = texture::spline(param,	
									sizeof(col_array)/sizeof(col_array[0]),
									col_array);
				c.clamp();

				p32[offset] = fcolour::get_rgb32(c);
			}
			offset++;
		}
		line++;
	}

}


/**
 * Read current preferences
 */
void base_main_window::load_settings(void)
{
	int val;
	my_settings->get_int(PREVIEW_HF_TYPE, &val);

	type = hf_type (val);

	my_settings->get_bool(DEBUG_UNCONSTRAINED_CAMERA, &unconstrained_camera);
}





