//
// base_main_window.h
//
// Main control window
//
// Copyright (c) J. Belson	1998.12.12
//


#ifndef _BASE_MAIN_WINDOW_H_
#define _BASE_MAIN_WINDOW_H_

#include "hfield.h"


// Forward declarations
class hfield;
class settings;
class preview_window;
class render_window;
class sky_window;
class cloud_window;
class palette_window;
class lighting_window;
class trees_window;
class atmospherics_window;
class texture_window;
class water_window;


/**
 * Platform independent functionality for main window.  Each port
 *  should derive from this to create a main window class.
 */
class base_main_window
{
private:

	uchar	palette[256];

	int		seed;
	hf_type	type;			// How are we generating heightfields?
	float	fd;				// Fractal dimension


protected:
	settings* my_settings;

	hfield*	my_map;
	bool	randomise;		// Change seed every render?

	// XXX Render relief view
	void render_relief(uint32* p32, int bytes_per_row);

	int get_seed(void)		{ return seed;	}
	void set_seed(int s)	{ seed = s;		}
	hf_type get_type(void)	{ return type;	}
	void set_type(hf_type t){ type = t;		}
	float get_fd(void)		{ return fd;	}
	void set_fd(float f)	{ fd = f;		}

	// Read settings for main window
	void load_settings(void);

	virtual void update_widgets(void) = 0;

	// True if camera limited to being within terrain area
	bool unconstrained_camera;

	// Other windows...
	preview_window*			w_preview;
	render_window*			w_render;
	sky_window*				w_sky;
	cloud_window*			w_cloud;
	palette_window*			w_palette;
	lighting_window*		w_lighting;
	trees_window*			w_trees;
	atmospherics_window*	w_atmospherics;
	texture_window*			w_texture;
	water_window*			w_water;

public:
	base_main_window();
	virtual ~base_main_window();

	enum { RELIEF_WIDTH = 256, RELIEF_ORDER = 8 };

	virtual void generate_map(void) = 0;

};



#endif	// _BASE_MAIN_WINDOW_H_






