//
// base_preview_window.cc
//
// Adjustable preview window
//
// Copyright (c) J. Belson	2000.11.04
//
// $Author: jon $	:	$Date: 2003/09/14 11:08:23 $
//

#include "base_preview_window.h"
#include "hfield.h"
#include "point3d.h"
#include "settings.h"
#include "vector3d.h"


/**
 * Constructor
 */
base_preview_window::base_preview_window()
	: w_width(START_WIDTH),
	  w_height(START_HEIGHT)
{
	my_settings = settings::get_instance();

	// Initial rendering states
	smoothing		= true;
	normals			= false;
	lighting		= true;
	texture			= true;
	fogging			= false;
	shadows			= false;
	track_camera	= true;
	draw_sky		= true;
	draw_land		= true;

	need_recalc	= true;
	fd			= 0.65;
	detail		= 7;

	land		= 0;
	sky			= 0;

	my_settings->get_int(PREVIEW_DETAIL, &detail);
}


/**
 * Destructor
 */
base_preview_window::~base_preview_window()
{
	;
}


/**
 * This routine does all the hard work of generating and
 *  rendering the landscape.
 */
void base_preview_window::render_thread(void)
{
	// Get camera position
	float cam_x, cam_y, cam_z;
	float tar_x, tar_y, tar_z;

	my_settings->get_float3(CAMERA_POSITION, &cam_x, &cam_y, &cam_z);
	my_settings->get_float3(TARGET_POSITION, &tar_x, &tar_y, &tar_z);

	// Where we are, and where we're looking
	win->set_view(point3d(cam_x, cam_y, cam_z), point3d(tar_x, tar_y, tar_z));

	float i, j, k;
	my_settings->get_float3(LIGHT_RAY_VECTOR, &i, &j, &k);
	win->set_light_position(i, j, k);


	// Regenerate terrain if settings have changed
	if (need_recalc) {
		delete land;
		delete sky;

		int type;
		my_settings->get_int(PREVIEW_HF_TYPE, &type);

		float water_level;
		my_settings->get_float(WATER_LEVEL, &water_level);
	
		land = new hfield(hf_type(type));
		land->seed_rand(seed);
		land->set_offset(-water_level);

		win->set_detail(detail);	// Set detail level for preview
		win->create_land(land);

		sky	= new hfield(HF_SKYDOME);
		win->create_sky(sky);

		win->render(draw_land, draw_sky);

		need_recalc = false;

	} else {
		win->render(draw_land, draw_sky);
	}

}
