//
// render_window.h
//
// Render window
//
// Copyright (c) J. Belson	1999.01.04
//


#ifndef _BASE_RENDER_WINDOW_H_
#define _BASE_RENDER_WINDOW_H_

#include "hfield.h"

#include "settings.h"
#include "bm_window.h"
#include "renderer.h"
#include "zbuffer.h"

class settings;

/**
 * Platform independent base class for render window. 
 */
class base_render_window {

private:

	// Handle display window, using w_width/w_height
	virtual void open_output_window(void) = 0;

protected:
	enum {
		START_WIDTH			= 640,
		START_HEIGHT		= 480
	};


	uchar	background;		// Background colour
	uchar	palette[4];
	int		seed;

	hfield*	my_map;
	float	fd;

	int		w_width, w_height;	// Window size

	// How many rendering threads?
	int		num_threads;

	// Number of redraws per second
	int		redraw_rate;

	// Current positions of camera and target
	float	camera_x, camera_y, camera_z;
	float	rot_a, rot_b, rot_c;
	float	lift_a, lift_b, lift_c;

	// Rendering preferences
	bool textures;
	bool lighting;
	bool shadows;
	bool reflections;
	bool sky;
	bool ground;
	bool trees;
	bool atmospherics;
	bool backface_cull;
	bool frustrum_cull;

	settings* my_settings;

	// Generating and render the landscape.
	virtual void main_render_thread(void);

	// Render option widgets should be disabled while rendering
	virtual void disable_widgets(void) = 0;
	virtual void enable_widgets(void) = 0;

public:
	base_render_window();
	virtual ~base_render_window();

	// Move window to (x, y)
	virtual void set_position(int x, int y) = 0;

	// Store current window position
	virtual void save_position(void) = 0;

};

#endif	// _BASE_RENDER_WINDOW_H_

