//
// ConvertColourHSV.cc
//
// J. Belson	2000.06.08
//

#include <cmath>

#include "ConvertColourHSV.h"


ConvertColourHSV::ConvertColourHSV()
	: ConvertColour()
{
	letter[0] = 'H';
	letter[1] = 'S';
	letter[2] = 'V';
}


col ConvertColourHSV::FromRGB(uint8 r, uint8 g, uint8 b)
{
	float red = r/255.0;
	float blue = b/255.0;
	float green = g/255.0;

	float hue, saturation, value;

	float max = red;
	if (blue > max) {
		max = blue;
	}
	if (green > max) {
		max = green;
	}

	float min = red;
	if (blue < min) {
		min = blue;
	}
	if (green < min) {
		min = green;
	}


	value = max;

	float diff = max - min;

	saturation = 0;
	if (max) {
		saturation = diff/max;
	}

	if (saturation == 0) {
		hue = 0;	// Undefined?
	} else {
		float r_dist = (max - red)/diff;
		float g_dist = (max - green)/diff;
		float b_dist = (max - blue)/diff;

		if (red == max) {
			hue = b_dist - g_dist;
		}
		else if (green == max) {
			hue = 2 + r_dist - b_dist;
		}
		else /*if (rgb.blue == max)*/ {
			hue = 4 + g_dist - r_dist;
		}
		hue *= 60;
		if (hue < 0) {
			hue += 360;
		}
	}

	col colour;
	colour.a = hue/360.0;
	colour.b = saturation;
	colour.c = value;

	return colour;
}



void ConvertColourHSV::ToRGB(col colour, uint8* r, uint8* g, uint8* b)
{
	float hue			= colour.a * 360.0;
	float saturation	= colour.b;
	float value			= colour.c;

	float red, green, blue;

	if (saturation == 0.0) {
		red		= value;
		green	= value;
		blue	= value;
	} else {
		if (hue == 360.0) {
			hue = 0.0;
		}
		hue /= 60.0;
		int i = int (floor(hue));
		float f = hue - i;

		float p = value * (1.0 - saturation);
		float q = value * (1.0 - (saturation * f));
		float t = value * (1.0 - (saturation * (1.0 - f)));

		switch (i) {
			case 0:
				red		= value;
				green	= t;
				blue	= p;
				break;
			case 1:
				red		= q;
				green	= value;
				blue	= p;
				break;
			case 2:
				red		= p;
				green	= value;
				blue	= t;
				break;
			case 3:
				red		= p;
				green	= q;
				blue	= value;
				break;
			case 4:
				red		= t;
				green	= p;
				blue	= value;
				break;
			default:
				red		= value;
				green	= p;
				blue	= q;
				break;
		}
	}

	*r = uint32 (255 * red);
	*g = uint32 (255 * green);
	*b = uint32 (255 * blue);

}




