//
// GLview.cc
//
// GL View that can redraw its contents
//
// Copyright (c) J. Belson	1999.1.2
//

#include <iostream.h>
#include <stdlib.h>
#include <Bitmap.h>
#include <string.h>

#include "GLview.h"
#include "settings.h"
#include "terrain.h"

#include "types.h"


const GLfloat GLview::fog_colour[] = { 0.75, 0.75, 0.75, 1.0 };
const GLfloat GLview::sky_colour[] = { 82/255.0, 134/255.0, 214/255.0, 1.0 };


//
// Constructor/destructor
//
GLview::GLview(BRect rect)
	: BGLView(rect, "bitmap_GLview", B_FOLLOW_ALL, 0 /*B_WILL_DRAW*/,
					BGL_RGB|BGL_DEPTH|BGL_DOUBLE)
{
	cout << "GLview::GLview" << endl;

	points = 0;

	light_i = 0.0;
	light_j = 1.0;
	light_k = 0.0;

	pos_x = pos_y = pos_z = 0.0;
	rot_a = rot_b = rot_c = 0.0;
}



void GLview::AttachedToWindow(void)
{
	cout << "GLview::AttachedToWindow()" << endl;

	BGLView::AttachedToWindow();

	LockGL();

	// Start with clean slate...
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Take your pick of projections
//	glOrtho(-100, 100, -40, 40, -40, 80);
//	glOrtho(-100, 100, -40, 40, 40, -80);
//	glFrustum(-30,30, -30,30, 10,50);
	gluPerspective(  40  /*60*/, 1.3, 0.0015, 320);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();


	// Ambient light for scene
	GLfloat light_model_ambient[]	= { 0.25, 0.25, 0.25, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light_model_ambient);

	// Sun
	float ambient;
	my_settings.get_float(AMBIENT_INTENSITY, &ambient);

//	GLfloat light_position[]		= { light_i, light_j, light_k, 0.0 };
//	GLfloat light_diffuse[]			= { 0.75, 0.75,  0.75 };
//	GLfloat light_ambient[]			= { ambient, ambient, ambient  };

//	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
//	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
//	glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);


	// Fog...
	glFogi(GL_FOG_MODE, GL_LINEAR /*GL_EXP*/);
	glFogfv(GL_FOG_COLOR, fog_colour);
	glFogf(GL_FOG_DENSITY, /*0.35*/ 0.01);
	glHint(GL_FOG_HINT, GL_DONT_CARE);
	glFogf(GL_FOG_START, 0.0 /*15.0*/);
	glFogf(GL_FOG_END, 400.0);


	// enable backface culling
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

	// zbuffering
	glEnable(GL_DEPTH_TEST);

	// Default to smooth shading
	glShadeModel(GL_SMOOTH);

	// Vaguely sky cloured background
	glClearColor(sky_colour[0], sky_colour[1], sky_colour[2], 1.0);


	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	UnlockGL();
}



//
// Change type of rendering
//
void GLview::wireframe(void)
{
	cout << "GLview::wireframe" << endl;
	LockGL();
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	//glShadeModel(GL_FLAT);
	UnlockGL();
}



void GLview::flat(void)
{
	cout << "GLview::flat" << endl;
	LockGL();
	glPolygonMode(GL_FRONT, GL_FILL );
	glShadeModel(GL_FLAT);
	UnlockGL();
}



void GLview::smooth(void)
{
	cout << "GLview::smooth" << endl;
	LockGL();
	glPolygonMode(GL_FRONT, GL_FILL);
	glShadeModel(GL_SMOOTH);
	UnlockGL();
}



void GLview::set_lighting(bool state)
{
	(state) ? glEnable(GL_LIGHTING)
	        : glDisable(GL_LIGHTING);
}


void GLview::set_fogging(bool state)
{
	if (state) {
		glClearColor(fog_colour[0], fog_colour[1], fog_colour[2], 1.0);
		glEnable(GL_FOG);
	} else {
		glClearColor(sky_colour[0], sky_colour[1], sky_colour[2], 1.0);
		glDisable(GL_FOG);
	}
}


void GLview::set_shadows(bool state)
{
	// Unimplemented
}


void GLview::set_light_position(float i, float j, float k)
{
	i = -i;
	j = -j;
	k = -k;

	cout << "glLight = (" << i << ", " << j << ", " << k << ")\n";

	light_i = i;
	light_j = j;
	light_k = k;

	GLfloat light_position[] = { light_i, light_j, light_k, 0.0 };

	glLightfv(GL_LIGHT0, GL_POSITION, light_position);
}


void GLview::set_rotation(float a, float b, float c)
{
	// Convert radians into degrees
	a = a/(2*M_PI) * 360.0;
	b = b/(2*M_PI) * 360.0;
	c = c/(2*M_PI) * 360.0;

	rot_a = a;
	rot_b = b;
	rot_c = c;
}


void GLview::set_translation(float x, float y, float z)
{
	pos_x = x;
	pos_y = y;
	pos_z = z;
}








void GLview::rerender(void)
{
	cout << "GLview::rerender" << endl;

	if (points) {
		make_model(points, normals, map_width);
	}
}



