//
// GLview.h
//
// GL View that can redraw its contents
//
// Copyright (c) J. Belson	1999.1.2
//

#ifndef _GLVIEW_H_
#define _GLVIEW_H_

#include <GLView.h>
#include <iostream.h>
#include <stdlib.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "point3d.h"
#include "vector3d.h"


class GLview : public BGLView {

private:
	// Used for re-rendering
	const point3d*	points;
	const vector3d*	normals;
	int				map_width;
	float light_i, light_j, light_k;

	float pos_x, pos_y, pos_z;	// Current position
	float rot_a, rot_b, rot_c;	// Current orientation

	static const GLfloat fog_colour[];
	static const GLfloat sky_colour[];


//	virtual void Draw(BRect rect);
	void AttachedToWindow(void);

public:
	GLview(BRect rect);

	// Create and render OpenGL model...
	void make_model(const point3d *p, const vector3d *n, int width);
	void rerender(void);

	// Rendering type
	void wireframe(void);
	void flat(void);
	void smooth(void);

	// Various rendering states
	void set_lighting(bool state);
	void set_fogging(bool state);
	void set_shadows(bool state);
	void set_light_position(float x, float y, float z);
	void set_rotation(float a, float b, float c);
	void set_translation(float x, float y, float z);


//	void refresh(BRect rect);
};


#endif	// _BM_GLVIEW_H_
