//
// GLwindow.cc
//
// Window that supports GL rendering
//
// Copyright (c) J. Belson	1999.1.2
//


#include <Application.h>
#include <Point.h>
#include "point2d.h"
#include "GLwindow.h"

#include <GL/gl.h>
#include <GL/glu.h>


// OpenGL initialisation based on Chris Herborth's ports of the NeHe tutorials


//
// Constructor/destructor
//
GLwindow::GLwindow(BRect r, char *name, window_type t, ulong mode)
	: smart_GLwindow(r, name, t, mode),
	  base_GLwindow(r.IntegerWidth(), r.IntegerHeight())
{
	opengl_device = BGL_DEVICE_SOFTWARE;

	// Initialise the OpenGL doodah
	EnumerateDevices( BGL_MONITOR_ANY, BGL_ANY | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE );
	InitializeGL(opengl_device, BGL_ANY | BGL_DOUBLE, BGL_ANY, BGL_NONE, BGL_NONE );

	puts("initialised opengl");

	// Magically, the window will now appear on-screen.
	Show();
	Sync();
}



GLwindow::~GLwindow()
{
	Hide();
	Sync();
}



void GLwindow::DeviceInfo( uint32 device_id, uint32 monitor, const char *name, bool depth, bool stencil, bool accum )
{
	if( device_id != BGL_DEVICE_SOFTWARE 		&& 
		opengl_device == BGL_DEVICE_SOFTWARE 	) {
		opengl_device = device_id;
		
		printf( "Using OpenGL device #%ld: %s\n", device_id, name );
	}

}





void GLwindow::FrameResized(float width, float height )
{
printf("frameResized  %f, %f \n", width, height );
	MakeCurrent();
	//glViewport( 0, 0, int(width-1), int(height-1));
	ReleaseCurrent();
}




void GLwindow::Quit(void)
{
	smart_GLwindow::Quit();
}



//
// Deal with incoming messages to window
//
void GLwindow::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		default:
			//cout << "Message" << endl;
			BWindow::MessageReceived(message);
			break;
	}
}



void GLwindow::GLinit(void)
{
	MakeCurrent();
	base_GLwindow::GLinit();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	SwapBuffers();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	ReleaseCurrent();
}



// Create and render OpenGL model...
void GLwindow::make_model(const point3d *p, const vector3d *n, int width)
{
	MakeCurrent();
	base_GLwindow::make_model(p, n, width);
	SwapBuffers();
	ReleaseCurrent();
}


void GLwindow::rerender(void)
{
	//view->rerender();
}



// Switch between rendering modes
void GLwindow::wireframe(void)
{
	MakeCurrent();
	base_GLwindow::wireframe();
	ReleaseCurrent();
};


void GLwindow::flat(void)
{
	MakeCurrent();
	base_GLwindow::flat();
	ReleaseCurrent();
};


void GLwindow::smooth(void)
{
	MakeCurrent();
	base_GLwindow::smooth();
	ReleaseCurrent();
};


// Set viewpoint and viewing direction
void GLwindow::set_view(const point3d& p, const vector3d& v)
{
	MakeCurrent();
	base_GLwindow::set_view(p, v);
	ReleaseCurrent();
}



// Various rendering states
void GLwindow::set_lighting(bool state)
{
	MakeCurrent();
	base_GLwindow::set_lighting(state);
	ReleaseCurrent();
}


void GLwindow::set_fogging(bool state)
{
	MakeCurrent();
	base_GLwindow::set_fogging(state);
	ReleaseCurrent();
}


void GLwindow::set_shadows(bool state)
{
	MakeCurrent();
	base_GLwindow::set_shadows(state);
	ReleaseCurrent();
}


void GLwindow::set_light_position(float x, float y, float z)
{
	MakeCurrent();
	base_GLwindow::set_light_position(x, y, z);
	ReleaseCurrent();
}


void GLwindow::set_rotation(float a, float b, float c)
{
	MakeCurrent();
	base_GLwindow::set_rotation(a, b, c);
	ReleaseCurrent();
}


void GLwindow::set_translation(float x, float y, float z)
{
	MakeCurrent();
	base_GLwindow::set_translation(x, y, z);
	ReleaseCurrent();
}


