//
// GLwindow.h
//
// Window that supports GL rendering
//
// Copyright (c) J. Belson	1999.1.2
//


#ifndef _GLWINDOW_H_
#define _GLWINDOW_H_

#include "smart_window.h"

#include <GL/gl.h>
#include <GL/glu.h>
#include "base_GLwindow.h"

#include <DirectGLWindow.h>



#define MAX_DEVICES 16
typedef struct ModeInfoRec ModeInfo;
struct ModeInfoRec
{
	ModeInfo *next;
	uint32 xres;
	uint32 yres;
};

typedef struct GfxDeviceRec
{
	uint32 deviceID;
	ModeInfo *first;
} GfxDevice;






class GLwindow : public smart_GLwindow /*BDirectGLWindow*/, base_GLwindow
{
private:
	// Which OpenGL device should we render on?
	uint32 opengl_device;

	int32 currentDevice;
	GfxDevice devices[MAX_DEVICES];

public:
	GLwindow(BRect r, char *name, window_type t, ulong mode);
	~GLwindow();
	
	virtual void GLinit(void);

	// Create and render OpenGL model...
	virtual void make_model(const point3d *p, const vector3d *n, int width);
//	virtual void render_model(const point3d *p, const vector3d *n, int width);

	// Set viewpoint and viewing direction
	void set_view(const point3d& p, const vector3d& v);

	void rerender(void);

	// OpenGL initialisation routines
	void DeviceInfo(uint32 device_id, uint32 monitor, const char *name, bool depth, bool stencil, bool accum);

	// Switch between rendering modes
	void wireframe(void);
	void flat(void);
	void smooth(void);


	// Various rendering states
	void set_lighting(bool state);
	void set_fogging(bool state);
	void set_shadows(bool state);
	void set_light_position(float x, float y, float z);
	void set_rotation(float a, float b, float c);
	void set_translation(float x, float y, float z);

	virtual void Quit(void);
	virtual void FrameResized( float width, float height);
	virtual void MessageReceived(BMessage *);
};


#endif	// _GLWINDOW_H_
