//
// atmospherics_window.cc
//
// Prefs for atmospheric effects
//
// Copyright (c) J. Belson	2000.01.19
//

#include <ColorControl.h>
#include <Message.h>
#include <iostream.h>

#include "MBorder.h"
#include "MBViewWrapper.h"
#include "MSlider.h"
#include "MStringView.h"
#include "Space.h"
#include "HGroup.h"
#include "VGroup.h"
#include "Colour.h"
#include "ConvertColourRGB.h"
#include "atmospherics_window.h"
#include "messages.h"
#include "settings.h"

#define MSG_PALETTE_CHANGED			'AW-1'
#define MSG_INTENSITY_CHANGED		'AW-2'


//
// Constructor/destructor
//
atmospherics_window::atmospherics_window(
				BWindow *window,
				BRect r,
				char *name,
				window_type t,
				ulong mode)
	: smart_window(r, name, t, mode),
	  parent(window)
{

	MSlider*		slider;

	ConvertColour* conv = new ConvertColourRGB();
	Colour* cp = new Colour(BRect(0, 0, 200, 64), conv, new BMessage(MSG_PALETTE_CHANGED));

	slider = new MSlider(
					"Haze Intensity",
					0,
					MAX_VALUE,
					1,
					new BMessage(MSG_INTENSITY_CHANGED));


	MView* top_view =
		new VGroup(
			new Space(minimax(4, 4, 4, 4)),
			new MBorder(
				M_LABELED_BORDER, 12, "Haze",
				new VGroup(
					new HGroup(
						new MBViewWrapper(cp),
						0
					),
					new Space(minimax(8, 8, 8, 8)),
					slider,
					0
				)
			),
			new Space(),
			0
		);

	slider->SetLimitLabels("Less", "More");
	slider->SetHashMarkCount(10);
	slider->SetHashMarks(B_HASH_MARKS_BOTTOM);


	AddChild(dynamic_cast<BView*>(top_view));

	int r, g, b;
	my_settings.get_int3(HAZE_COLOUR, &r, &g, &b);
	cp->SetColour(r, g, b);

	float val;
	my_settings.get_float(HAZE_INTENSITY, &val);
	slider->SetValue(MAX_VALUE * (val - 0.5));
}



atmospherics_window::~atmospherics_window()
{
	BMessage msg(MSG_ATMOSPHERICS_CLOSED);
	parent->PostMessage(&msg);
}



//
// Deal with window close gadget
//
bool atmospherics_window::QuitRequested()
{
	//if (win->Lock()) {
	//	win->Quit();
	//}

	return TRUE;
}



//
// Deal with incoming messages to window
//
void atmospherics_window::MessageReceived(BMessage *message)
{

	switch (message->what) {

		case MSG_PALETTE_CHANGED:
			{
				int8 val;
				uint8 red, green, blue;
			
				red = green = blue = 0;
			
				if (B_OK == message->FindInt8("red", &val)) {
					red = uint8 (val);
				}
			
				if (B_OK == message->FindInt8("green", &val)) {
					green = uint8 (val);
				}
			
				if (B_OK == message->FindInt8("blue", &val)) {
					blue = uint8 (val);
				}
				my_settings.set_int3(HAZE_COLOUR, red, green, blue);


			}
			break;

		case MSG_INTENSITY_CHANGED:
			{
				int32 i;
				if (B_OK == message->FindInt32("be:value", &i)) {
					float val = 0.5 + float(i)/MAX_VALUE;
					my_settings.set_float(HAZE_INTENSITY, val);
				} else {
					cout << "Couldn't find intensity!?" << endl;
				}
			}
			break;

		default:
			MWindow::MessageReceived(message);
			break;

	}
}


