//
// bm_window.cc
//
// Simple window with a bitmap to render into
//
// Copyright (c) J. Belson	1998.12.15
//


#include <Application.h>
#include <Point.h>
#include <Polygon.h>

#include "point2d.h"
#include "vector3d.h"
#include "bm_window.h"
#include "gradient.h"

#include <stdlib.h>
#include <unistd.h>



//
// Constructor/destructor
//
bm_window::bm_window(BRect r, char *name, window_type t, ulong mode, int bit_depth) :
	smart_window(r, name, t, mode)
{
	window_width	= int (r.right - r.left + 1);
	window_height	= int (r.bottom - r.top + 1);

	cout << "bm_window()" << endl;

	view = new bm_view(BRect(0, 0, r.right - r.left, r.bottom - r.top), bit_depth);

	bytes_per_row = view->get_bytes_per_row();

	// Draw a kind of 'sky' effect
	//clear_window();

	AddChild(view);
	//view->SetLowColor(255, 0, 0);
	//view->SetHighColor(0, 255, 0);

	depth_buffer = new zbuffer(window_width, window_height);

	set_redraw_rate(0);
}



bm_window::~bm_window()
{
//	delete view;	

	delete depth_buffer;
}



void bm_window::Quit()
{
	smart_window::Quit();
}


void bm_window::refresh(void)
{
	if (view->LockLooper()) {
		view->Draw(Bounds());
		view->UnlockLooper();
	}
}



//
// Rate at which window contents is updated
// On entry:	rate is pulses per sec
//				0 stops pulsing
//
void bm_window::set_redraw_rate(int rate)
{
	// Pulse events are measued in microseconds, so...
	if (rate) {
		SetPulseRate(1.0/rate * 1000000);
	} else {
		SetPulseRate(0);
	}
}


//
// Deal with incoming messages to window
//
void bm_window::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		default:
			//cout << "Message" << endl;
			BWindow::MessageReceived(message);
			break;
	}
}



//
// Set background to a graduated 'sky' kind of effect
//
void bm_window::draw_sky(bool grad, int offset)
{
	unsigned long int *gfx = (unsigned long int *) view->get_bitmap_data();

	if (Lock()) {
		gradient sky;
		if (grad) {
			sky = gradient(90,125,180, 230,230,230,  window_height/2 - offset);

			int i;
			for (i=0; i<window_height/2 - offset; i++) {
				unsigned char r, g, b;
				sky.get_next(r, g, b);
				for (int j=0; j<window_width; j++) {
					gfx[j] = MAKE_COLOUR(r, g, b);
				}
				gfx += view->get_bytes_per_row() >> 2;	// 32-bit bitmap
			}

			sky = gradient(230,230,230, 255, 255, 255 /*80,120,60*/,  window_height/2 + offset);
			for (i=0; i<window_height/2 + offset; i++) {
				unsigned char r, g, b;
				sky.get_next(r, g, b);
				for (int j=0; j<window_width; j++) {
					gfx[j] = MAKE_COLOUR(r, g, b);
				}
				gfx += view->get_bytes_per_row() >> 2;	// 32-bit bitmap
			}

		} else {
			sky = gradient(100,130,180, 200,200,200,  window_height);

			for (int i=0; i<window_height; i++) {
				unsigned char r, g, b;
				sky.get_next(r, g, b);
				for (int j=0; j<window_width; j++) {
					gfx[j] = MAKE_COLOUR(r, g, b);
				}
				gfx += view->get_bytes_per_row() >> 2;	// 32-bit bitmap
			}
		}
		Unlock();
	}
}



//
// Initialise depth buffer before use
//
void bm_window::clear_zbuffer()
{
	depth_buffer->clear_buffer();
}



//
// Get width/height for clipping purposes or whatever
//
void bm_window::get_window_size(int *x, int *y)
{
	*x = window_width;
	*y = window_height;
}






void bm_window::set_colour(unsigned char r, unsigned char g, unsigned char b)
{
	if (Lock()) {
		view->SetHighColor(r, g, b);
		Unlock();
	}
}


uint32* bm_window::get_buffer(void)
{
	return (uint32*) view->get_bitmap_data();
}


// Get bitmap containing rendered image
const BBitmap* bm_window::get_bitmap(void)
{
	return (const BBitmap*) view->get_bitmap();
}




