//
// bm_window.h
//
// Simple window with a bitmap to render into
//
// Copyright (c) J. Belson	1998.12.15
//


#ifndef _BM_WINDOW_H_
#define _BM_WINDOW_H_

//#include "Window.h"
#include "smart_window.h"
#include "Polygon.h"

#include "bm_view.h"
#include "colour.h"
#include "triangle2d.h"
#include "vector3d.h"
#include "zbuffer.h"


class bm_window : public smart_window
{
	private:
		bm_view *view;
		int window_width, window_height;

		enum {
			MAX_HEIGHT = 1024
		};

		enum orientation {
			LEFT,
			RIGHT
		};

		// Edge tables
		float		table_start[MAX_HEIGHT],  table_end[MAX_HEIGHT];
		float		depth_start[MAX_HEIGHT],  depth_end[MAX_HEIGHT];
		colour_rgb	colour_start[MAX_HEIGHT], colour_end[MAX_HEIGHT];
		vector3d	normal_start[MAX_HEIGHT], normal_end[MAX_HEIGHT];

		zbuffer *depth_buffer;

	public:
		bm_window(BRect r, char *name, window_type t, ulong mode, int bit_depth);
		~bm_window();

		uchar	*bm_data;
		int		bytes_per_row;

		void refresh(void);
		void draw_sky(bool grad, int offset = 0);
		void clear_zbuffer(void);
		void get_window_size(int *x, int *y);
//		void draw_triangle(int w_width, int w_height, int map_size, /*const*/ triangle2d *tri);
//		void plot_point(float x, float y);
//		void draw_line(float x, float y, float depth1, float x2, float y2, float depth2, orientation side);
//		void draw_poly(float x, float y, float x2, float y2, float x3, float y3);
		void set_colour(unsigned char r, unsigned char g, unsigned char b);

		// Rate at which window contents is updated
		void set_redraw_rate(int rate);

		// Access raw image data
		uint32* get_buffer(void);

		// Get bitmap containing rendered image
		const BBitmap* get_bitmap(void);

		//virtual bool QuitRequested();
		virtual void Quit();
		virtual void MessageReceived(BMessage *);
};


#endif	// _BM_WINDOW_H_
