//
// cloud_window.cc
//
// Prefs for cloud generation
//
// Copyright (c) J. Belson	1999.04.05
//


#include <iostream.h>
#include <time.h>
#include <Bitmap.h>
#include <ColorControl.h>
#include <Screen.h>
#include <Message.h>

#include "MApplication.h"
#include "MBorder.h"
#include "VGroup.h"
#include "HGroup.h"
#include "MButton.h"
#include "MBViewWrapper.h"
#include "MSlider.h"
#include "MBorder.h"
#include "MTextView.h"
#include "Space.h"

#include "cloud_window.h"
#include "messages.h"
#include "not_implemented.h"
#include "smart_window.h"
#include "settings.h"

// Local messages
#define	MSG_PALETTE_CHANGED			'CL-1'
#define	MSG_PERSISTANCE_CHANGED		'CL-2'
#define	MSG_PRESENCE_CHANGED		'CL-3'
#define	MSG_COLOUR_CHANGED			'CL-4'
#define	MSG_COVER_CHANGED			'CL-5'
#define	MSG_SCALE_CHANGED			'CL-6'

//#define	MSG_RERENDER				'CL-9'




//
// Constructor/destructor
//
cloud_window::cloud_window(BWindow *window, BRect r, char *name, window_type t, ulong mode)
	: smart_window(r, name, t, mode),
	  base_cloud_window(),
	  parent(window)
{
	BRect cloud_rect = BRect(0, 0, CLOUD_WIDTH-1, CLOUD_WIDTH-1);
	cloud_view = new bm_view(cloud_rect, 32);

//	cloud_colour	= DEFAULT_CLOUD_COLOUR;
//	persistance		= DEFAULT_PERSISTANCE;
//	presence		= DEFAULT_PRESENCE;
//	cloud_cover		= DEFAULT_CLOUD_COVER;
//	scale			= DEFAULT_SCALE;


	MButton* generate;
	MSlider* slider_col;
	MSlider* slider_pers;
	MSlider* slider_pres;
	MSlider* slider_cover;
	MSlider* slider_scale;

	generate	= new MButton("Generate", new BMessage(MSG_RERENDER), 0, minimax(-1,-1,-1,-1));
	slider_col	= new MSlider("Colour", 0, MAX_VALUE, 1, new BMessage(MSG_COLOUR_CHANGED));
	slider_pers	= new MSlider("Persistance", 0, MAX_VALUE, 1, new BMessage(MSG_PERSISTANCE_CHANGED));
	slider_pres	= new MSlider("Presence", 0, MAX_VALUE, 1, new BMessage(MSG_PRESENCE_CHANGED));
	slider_cover= new MSlider("Cover", 0, MAX_VALUE, 1, new BMessage(MSG_COVER_CHANGED));
	slider_scale= new MSlider("Scale", 0, MAX_VALUE, 1, new BMessage(MSG_SCALE_CHANGED));

	MView* top_view =
			new HGroup
			(
				new MBViewWrapper(cloud_view),
				new VGroup
				(
					new Space(minimax(100, 1, 110, 1)),
					new MBorder
					(
						M_LABELED_BORDER, 12, "Options",
						new VGroup
						(
							slider_col,
							new Space(minimax(8, 8, 8, 8)),
							slider_pers,
							new Space(minimax(8, 8, 8, 8)),
							slider_pres,
							new Space(minimax(8, 8, 8, 8)),
							slider_cover,
							new Space(minimax(8, 8, 8, 8)),
							slider_scale,
							new Space( /*minimax(6, 6, 6, 6)*/ ),
							0
						)
					),
					new Space(),
					generate,
					new Space(minimax(4, 4, 4, 4)),
					0
				),
				0
			);


	MSlider* slider[] = { slider_pers, slider_pres, slider_col, slider_cover, slider_scale };

	for (uint32 i=0; i<sizeof(slider)/sizeof(slider[0]); i++) {
		slider[i]->SetHashMarkCount(10);
		slider[i]->SetHashMarks(B_HASH_MARKS_BOTTOM);
	}

	slider_pers->SetValue(MAX_VALUE * persistance);
	slider_pres->SetValue(MAX_VALUE * presence/10.0);
	slider_col->SetValue(MAX_VALUE * (cloud_colour - 191)/64.0);
	slider_cover->SetValue(MAX_VALUE * (cloud_cover+4) / 8.0);
	slider_scale->SetValue(MAX_VALUE * (scale/0.2));

	Lock();
	// top_view is an MView, so we need to cast it to a BView.
	AddChild(dynamic_cast<BView*> (top_view));
	Unlock();


	the_thread = 0;
}


cloud_window::~cloud_window()
{
	BMessage msg(MSG_CLOUD_CLOSED);
	parent->PostMessage(&msg);
}


//
// Deal with window close gadget
//
bool cloud_window::QuitRequested()
{
	set_quit_requested();

	if (the_thread) {
		status_t status;
		wait_for_thread(the_thread, &status);
	}

	return TRUE;
}


//
// Deal with incoming messages to window
//
void cloud_window::MessageReceived(BMessage *message)
{
	int32 i;

	switch (message->what)
	{
		//
		// Local messages
		//
		case MSG_COLOUR_CHANGED:
			if (B_OK == message->FindInt32("be:value", &i)) {
				//cloud_colour = 191 + uint8 (64.0 * float(i)/MAX_VALUE);
				changed_colour(i);
			} else {
				cout << "Couldn't find intensity!?" << endl;
			}
			break;

		case MSG_PERSISTANCE_CHANGED:
			if (B_OK == message->FindInt32("be:value", &i)) {
				//persistance = float(i)/MAX_VALUE;
				changed_persistance(i);
			} else {
				cout << "Couldn't find intensity!?" << endl;
			}
			break;

		case MSG_PRESENCE_CHANGED:
			if (B_OK == message->FindInt32("be:value", &i)) {
				//presence = 10.0*float(i)/MAX_VALUE;
				changed_presence(i);
			} else {
				cout << "Couldn't find presence!?" << endl;
			}
			break;

		case MSG_COVER_CHANGED:
			if (B_OK == message->FindInt32("be:value", &i)) {
				//cloud_cover = -4.0 + 8.0*float(i)/MAX_VALUE;
				changed_cover(i);
			} else {
				cout << "Couldn't find cloud_cover!?" << endl;
			}
			break;

		case MSG_SCALE_CHANGED:
			if (B_OK == message->FindInt32("be:value", &i)) {
				//scale = 0.2 * float(i)/MAX_VALUE;
				changed_scale(i);
			} else {
				cout << "Couldn't find scale!?" << endl;
			}
			break;

		case MSG_RERENDER:
			cout << "Rerender" << endl;
			seed = time(NULL);
			my_settings.set_int(CLOUD_SEED, seed);
			my_settings.set_int(CLOUD_COLOUR, cloud_colour);
			my_settings.set_float(CLOUD_PERSISTANCE, persistance);
			my_settings.set_float(CLOUD_PRESENCE, presence);
			my_settings.set_float(CLOUD_COVER, cloud_cover);
			my_settings.set_float(CLOUD_SCALE, scale);
			if (the_thread == 0) {
				the_thread = spawn_thread(cloud_thread_func, "Render Thread",
						B_NORMAL_PRIORITY, this);
				resume_thread(the_thread);
			}
			break;


		default:
			cout << "Message" << endl;
			MWindow::MessageReceived(message);
			break;
	}
}


//
// Static function to launch thread
//
long cloud_window::cloud_thread_func(void *obj )
{
	cloud_window *my_obj = (cloud_window *) obj;
	my_obj->render_thread();
	return 0;
}



long cloud_window::rerender_thread_func(void *obj )
{
	cloud_window *my_obj = (cloud_window *) obj;
	my_obj->rerender_thread();
	return 0;
}



void cloud_window::rerender_thread(void)
{
	//win->rerender();
	the_thread = 0;
}



//
// This routine does all the hard work of generating and
//  rendering the landscape.
//
void cloud_window::render_thread(void)
{


	base_cloud_window::render_thread();

#if 0
	perlin noise;
	noise.set_seed(seed);
	noise.set_persistance(persistance);
	//noise.set_freq_scale(scale);

	const int bpr	= cloud_view->get_bytes_per_row();
	uint32* p		= (uint32*) cloud_view->get_bitmap_data();


	for (int y=0; y<CLOUD_WIDTH; y++) {
		if (quit_requested) {
			the_thread = 0;
			return;
		}
		for (int x=0; x<CLOUD_WIDTH; x++) {

			float height = noise.perlin_noise_2d(scale*x, scale*y);

			uint32 col;

			uint8 r = 90;
			uint8 g = 150;
			uint8 b = 231;

			float adjusted = noise.exponential(cloud_cover + presence*height);

			float blend = adjusted/255.0;

			//r = r + (cloud_colour - r) * CloudVal;
			//g = g + (cloud_colour - g) * CloudVal;
			//b = b + (cloud_colour - b) * CloudVal;
			r = uint8 (r*(1.0 - blend) + cloud_colour*blend);
			g = uint8 (g*(1.0 - blend) + cloud_colour*blend);
			b = uint8 (b*(1.0 - blend) + cloud_colour*blend);

			if (r > 255)	r = 255;
			if (g > 255)	g = 255;
			if (b > 255)	b = 255;

			col = (r<<16) | (g<<8) | b;

			p[x] = col;

		}
		p += bpr/4;	// Next row...
	}
#endif

	if (LockLooper()) {
		cloud_view->Draw(cloud_view->Bounds());
		Unlock();
	}

	the_thread = 0;
}





