//
// lighting_window.cc
//
// Prefs for light source
//
// Copyright (c) J. Belson	1999.06.30
//

#include <ColorControl.h>
#include <Message.h>
#include <iostream.h>

#include "MBorder.h"
#include "MBViewWrapper.h"
#include "MSlider.h"
#include "MStringView.h"
#include "Space.h"
#include "HGroup.h"
#include "VGroup.h"
#include "Colour.h"
#include "ConvertColourRGB.h"
#include "lighting_window.h"
#include "messages.h"
#include "settings.h"
#include "vector3d.h"


const uint32 MSG_AMB_PALETTE_CHANGED	= 'LW-1';
const uint32 MSG_AMBIENT_CHANGED		= 'LW-2';
const uint32 MSG_DIR_PALETTE_CHANGED	= 'LW-3';
const uint32 MSG_DIRECTIONAL_CHANGED	= 'LW-4';
const uint32 MSG_LIGHT_MOVED			= 'LW-5';
//const uint32 MSG_LIGHT_ROTATION_CHANGED	= 'LW-6';
//const uint32 MSG_LIGHT_LIFT_CHANGED		= 'LW-7';


//
// Constructor/destructor
//
lighting_window::lighting_window(BWindow *window, BRect r, char *name, window_type t, ulong mode)
	: smart_window(r, name, t, mode),
	  parent(window)
{
	sv = new sunview(BRect(0, 0, 100, 100), new BMessage(MSG_LIGHT_MOVED));

	aview = new above_view(new BMessage(MSG_LIGHT_MOVED));
	sview = new side_view(new BMessage(MSG_LIGHT_MOVED));

	MSlider*		int_amb;
	MSlider*		int_dir;

	int_amb = new MSlider(
					"Intensity",
					0,
					MAX_VALUE,
					1,
					new BMessage(MSG_AMBIENT_CHANGED));

	int_dir = new MSlider(
					"Intensity",
					0,
					MAX_VALUE,
					1,
					new BMessage(MSG_DIRECTIONAL_CHANGED));

	ConvertColour* conv = new ConvertColourRGB();
	Colour* c_amb = new Colour(BRect(0, 0, 200, 64), conv, new BMessage(MSG_AMB_PALETTE_CHANGED));
	Colour* c_dir = new Colour(BRect(0, 0, 200, 64), conv, new BMessage(MSG_DIR_PALETTE_CHANGED));

	MView* top_view =
		new VGroup(
			new Space(minimax(4, 4, 4, 4)),
			new MBorder(
				M_LABELED_BORDER, 12, "Ambient",
				new VGroup(
					new MBViewWrapper(c_amb),
//					new Space(minimax(8, 8, 8, 8)),
//					int_amb,
					0
				)
			),
			new MBorder(
				M_LABELED_BORDER, 12, "Directional",
				new VGroup(
					new MBViewWrapper(c_dir),
//					int_dir,
					new Space(minimax(8, 8, 8, 8)),
	//				new MBViewWrapper(sv),
		new MBViewWrapper(aview),
		new MBViewWrapper(sview),
					0
				)
			),
			new Space(),
			0
		);


	AddChild(dynamic_cast<BView*>(top_view));

	int r, g, b;
	my_settings.get_int3(AMBIENT_COLOUR, &r, &g, &b);
	c_amb->SetColour(r, g, b);

	my_settings.get_int3(DIRECT_COLOUR, &r, &g, &b);
	c_dir->SetColour(r, g, b);

	float intensity;
	my_settings.get_float(AMBIENT_INTENSITY, &intensity);
	int_amb->SetValue(intensity*MAX_VALUE);

	my_settings.get_float(DIRECT_INTENSITY, &intensity);
	int_dir->SetValue(intensity*MAX_VALUE);

	float i, j, k;
	my_settings.get_float3(LIGHT_VECTOR, &i, &j, &k);
	sv->set_position(i, j, k);
}



lighting_window::~lighting_window()
{
	BMessage msg(MSG_LIGHTING_CLOSED);
	parent->PostMessage(&msg);
}



//
// Deal with window close gadget
//
bool lighting_window::QuitRequested()
{
	//if (win->Lock()) {
	//	win->Quit();
	//}

	return TRUE;
}



//
// Deal with incoming messages to window
//
void lighting_window::MessageReceived(BMessage *message)
{
	switch (message->what) {

		case MSG_LIGHT_MOVED: {
				// Turn the 'lift' into a vector
				float angle = sview->get_lift();
				vector3d tmp(0.0, sin(angle), cos(angle));

				cout << "lift = " << angle << endl;

				// Now rotate the vector to the required position
				angle = aview->get_rotation();
				cout << "rotation = " << angle << endl;

				float old_i = tmp.i;
				float old_j = tmp.j;
				float old_k = tmp.k;
				
				tmp.i = old_i*cos(angle) - old_k*sin(angle);
				tmp.j = old_j;
				tmp.k = old_i*sin(angle) + old_k*cos(angle);
			
			
				tmp.normalise();
			
				my_settings.set_float3(LIGHT_VECTOR, tmp.i, tmp.j, tmp.k);
			
				cout << "Light moved" << endl;
			}
			break;

		case MSG_AMB_PALETTE_CHANGED:
			{
				int8 val1, val2, val3;
				uint8 red, green, blue;
			
				red = green = blue;
			
				if (B_OK == message->FindInt8("red", &val1) &&
					B_OK == message->FindInt8("green", &val2) &&
					B_OK == message->FindInt8("blue", &val3)) {

					red = uint8 (val1);
					green = uint8 (val2);
					blue = uint8 (val3);
					my_settings.set_int3(AMBIENT_COLOUR, red, green, blue);
				}

			}
			break;

		case MSG_DIR_PALETTE_CHANGED:
			{
				int8 val1, val2, val3;
				uint8 red, green, blue;
			
				red = green = blue;

				if (B_OK == message->FindInt8("red", &val1) &&
					B_OK == message->FindInt8("green", &val2) &&
					B_OK == message->FindInt8("blue", &val3)) {

					red = uint8 (val1);
					green = uint8 (val2);
					blue = uint8 (val3);
					my_settings.set_int3(DIRECT_COLOUR, red, green, blue);
				}
			

			}
			break;

		case MSG_AMBIENT_CHANGED:
			{
				int32 i;
				if (B_OK == message->FindInt32("be:value", &i)) {
					float val = float(i)/MAX_VALUE;
					my_settings.set_float(AMBIENT_INTENSITY, val);
				} else {
					cout << "Couldn't find intensity!?" << endl;
				}
			}
			break;

		case MSG_DIRECTIONAL_CHANGED:
			{
				int32 i;
				if (B_OK == message->FindInt32("be:value", &i)) {
					float val = float(i)/MAX_VALUE;
					my_settings.set_float(DIRECT_INTENSITY, val);
				} else {
					cout << "Couldn't find intensity!?" << endl;
				}
			}
			break;


		default:
			MWindow::MessageReceived(message);
			cout << "Got message" << endl;
			break;

	}
}


