//
// main_window.cc
//
// Main control window
//
// Copyright (c) J. Belson	1998.12.12
//


#include <Alert.h>
#include <Bitmap.h>
#include <translation/BitmapStream.h>
#include <Entry.h>
#include <File.h>
#include <Message.h>
#include <MessageQueue.h>
#include <Path.h>
#include <Screen.h>
#include <translation/TranslatorRoster.h>

#include "main_window.h"
#include "numeric_input.h"

#include "hf_fbm.h"
#include "hf_perlin.h"
#include "hf_synthesis.h"
#include "texture.h"


static fcolour col_array[] = {

	fcolour(0.0,	0.0,	0.0),
//	fcolour(0.216,	0.216,	0.9),
//	fcolour(0.235,	0.235,	0.96),
	fcolour(78/256.0,	196/256.0,	74/256.0),
	fcolour(40/256.0,	166/256.0,	40/256.0),
	fcolour(136/256.0,	0.31,		0.31),
	//fcolour(156/256.0,	0.31,		0.31),
	fcolour(0.87,		0.84,		0.96),
	fcolour(1.0,		1.0,		1.0)
};



main_window::main_window(BRect rect, char* title, window_type type, ulong flags)
	: smart_window(rect, title, type, flags),
	  base_main_window()
{
	BRect relief_rect = BRect(0, 0, RELIEF_WIDTH-1, RELIEF_WIDTH-1);
	relief_view = new mouse_bm_view(relief_rect);

	relief_view->reset_camera();

	MCheckBox*	rand_box;
	MMenuBar*	menu_bar;
	MPopup*		fractal_type;
	MButton*	b_update;

	TabGroup*	grp;

	// Create widgets
	rand_box = new MCheckBox("Randomise", new BMessage(MSG_RAND), NULL, true);
	text_fd_subdivision	= new MTextControl("H", "0.85");
	text_fd_synthesis	= new MTextControl("H", "0.85");
	text_seed			= new MTextControl("Seed", "37686");
	text_camera_x		= new numeric_input("X", "0.0");
	text_camera_y		= new numeric_input("Y", "0.0");
	text_camera_h		= new numeric_input("H", "0.0");
	text_target_x		= new numeric_input("X", "0.0");
	text_target_y		= new numeric_input("Y", "0.0");
	text_target_h		= new numeric_input("H", "0.0");

	spin_iterations = new SpinButton("Iterations", SPIN_INTEGER);
	spin_iterations->SetMinimum(1);
	spin_iterations->SetMaximum(6);
	spin_iterations->SetValue(4L);

	spin_synth_iterations = new SpinButton("Iterations", SPIN_INTEGER);
	spin_synth_iterations->SetMinimum(2);
	spin_synth_iterations->SetMaximum(8);
	spin_synth_iterations->SetValue(8L);

	fractal_type	= new MPopup("Method", new BMessage(MSG_METHOD), this,
							"Perlin noise", "Subdivision", "Synthesis", 0);
	b_generate		= new MButton("Generate", new BMessage(MSG_GEN), 0, minimax(-1,-1,-1,-1));
	b_update		= new MButton("Update", new BMessage(MSG_COORD_CHANGE), 0, minimax(-1,-1,-1,-1));

	text_fd_subdivision->SetModificationMessage(new BMessage(MSG_FD));
	text_fd_synthesis->SetModificationMessage(new BMessage(MSG_FD_SYNTHESIS));

	layers =
		new LayeredGroup(
			new VGroup(spin_iterations, 0),
			new VGroup(text_fd_subdivision, text_seed, 0),
			new VGroup(text_fd_synthesis, spin_synth_iterations, 0),
			0
		);


	MView *top_view =
		new VGroup
		(
			menu_bar = new MMenuBar(),
			new HGroup
			(
				//new Space(minimax(6, 6, 6, 6)),
				new MBViewWrapper(relief_view),
				new VGroup
				(
					//new Space(minimax(2, 2, 2, 2)),

					grp = new TabGroup(
						"Fractal",
						new VGroup(
							new MBorder(
								M_NO_BORDER, 12 /*14*/, "",
								new VGroup(
									fractal_type,
									layers,
									0
								)
							),
							new Space(),
							b_generate,
							new Space(minimax(4, 4, 4, 4)),
							0
						),
						"View",
						new VGroup(
							new MBorder(
								M_LABELED_BORDER, 12, "Camera",
								new VGroup(
									text_camera_x, text_camera_y, text_camera_h,
									0
								)
							),
							new MBorder(
								M_LABELED_BORDER, 12, "Target",
								new VGroup(
									text_target_x, text_target_y, text_target_h,
									0
								)
							),
							new Space(),
							b_update,
							new Space(minimax(4, 4, 4, 4)),
							0
						), 0
					), 0
				), 0
			), 0
		);


	// Ensure even spacing
	DivideSame(text_camera_x, text_camera_y, text_camera_h, 0);
	DivideSame(text_target_x, text_target_y, text_target_h, 0);
	DivideSame(text_fd_subdivision, text_seed, fractal_type, spin_iterations, 0);
	DivideSame(text_fd_synthesis, fractal_type, spin_synth_iterations, 0);

	// Disable height widgets for now...
	text_target_h->SetEnabled(false);
	text_camera_h->SetEnabled(false);

	BMenu *menu	= new BMenu("File");
	menu->AddItem(new BMenuItem("About",		new BMessage(MSG_ABOUT)));
	menu->AddItem(new BSeparatorItem);
	menu_save_image = new BMenuItem("Save Image...",new BMessage(MSG_SAVE_IMAGE));
	menu->AddItem(menu_save_image);
	menu_save_image->SetEnabled(false);
	menu->AddItem(new BSeparatorItem);
	menu->AddItem(new BMenuItem("Quit",			new BMessage(MSG_QUIT)));
	menu_bar->AddItem(menu);
	menu = new BMenu("Window");
	menu->AddItem(menu_preview = new BMenuItem("Preview Window", new BMessage(MSG_MENU_PREVIEW)));
	menu->AddItem(menu_render  = new BMenuItem("Render Window",	 new BMessage(MSG_MENU_RENDER)));
	menu_bar->AddItem(menu);
	menu = new BMenu("Settings");

	BMenuItem* mi;
	mi = new BMenuItem("Lighting...",		new BMessage(MSG_MENU_LIGHTING));
	menu->AddItem(mi);
	mi = new BMenuItem("Cloud...",			new BMessage(MSG_MENU_CLOUD));
	menu->AddItem(mi);	//mi->SetEnabled(false);
	mi = new BMenuItem("Trees...",			new BMessage(MSG_MENU_TREES));
	menu->AddItem(mi);	mi->SetEnabled(false);
	mi = new BMenuItem("Palette...",		new BMessage(MSG_MENU_PALETTE));
	menu->AddItem(mi);	mi->SetEnabled(false);
	mi = new BMenuItem("Atmospherics...",	new BMessage(MSG_MENU_ATMOSPHERICS));
	menu->AddItem(mi);
	mi = new BMenuItem("Textures...",		new BMessage(MSG_MENU_TEXTURE));
	menu->AddItem(mi);	mi->SetEnabled(false);
	mi = new BMenuItem("Water...",			new BMessage(MSG_MENU_WATER));
	menu->AddItem(mi);

	menu_bar->AddItem(menu);

	// top_view is an MView, so we need to cast it to a BView.
	AddChild(dynamic_cast<BView*> (top_view));

	fractal_type->SetActive(get_type());

	// Set widgets to current values
	int val;
	my_settings.get_int(PERLIN_ITERATIONS, &val);
	spin_iterations->SetValue((long int) val);
	my_settings.get_int(SYNTHESIS_ITERATIONS, &val);
	spin_synth_iterations->SetValue((long int) val);
	float f;
	char str[10];
	my_settings.get_float(SUBDIVISION_FD, &f);
	sprintf(str, "%1.2f", f);
	text_fd_subdivision->SetText(str);	
	my_settings.get_float(SYNTHESIS_FD, &f);
	sprintf(str, "%1.2f", f);
	text_fd_synthesis->SetText(str);	


	save_panel = new save_image_panel(new BMessenger(this), new BMessage(MSG_DO_SAVE));

}


main_window::~main_window()
{

}



//
// Utilits template for opening tool windows
//
template<class T>
T* main_window::open_tool_window(char* title)
{
	BRect rect(300, 340, 300 + 1, 340 + 1);

	T* win = new T(this, rect, title,
					/*B_TITLED_WINDOW*/ B_FLOATING_WINDOW, B_NOT_RESIZABLE|B_NOT_ZOOMABLE);
	win->Show();

	return win;
}


template<class T>
T* main_window::open_window(char* title)
{
	BRect rect(300, 340, 300 + 1, 340 + 1);

	T* win = new T(this, rect, title,
					B_TITLED_WINDOW, B_NOT_RESIZABLE|B_NOT_ZOOMABLE);
	win->Show();

	return win;
}



//
// Work out angle between camera direction and a unit
//  vector in the z direction
//
float main_window::calculate_camera_angle(void)
{
	int camera_x, camera_z, target_x, target_z;

	// Get current camera and target positions
	relief_view->get_camera_pos(&camera_x, &camera_z);
	relief_view->get_target_pos(&target_x, &target_z);


	// Calculate lift from z=0 plain
	float height_camera = my_map->get_height(camera_x, camera_z);
	if (height_camera < 0.0) {
		height_camera = 0.0;
	}
	float height_target = my_map->get_height(target_x, target_z);
	if (height_target < 0.0) {
		height_target = 0.0;
	}

	// Translate camera to terrain space, ie. origin in centre, -z into screen
	camera_x -= RELIEF_WIDTH/2;
	camera_z -= RELIEF_WIDTH/2;

	target_x -= RELIEF_WIDTH/2;
	target_z -= RELIEF_WIDTH/2;

	my_settings.set_float3(CAMERA_POSITION, camera_x, height_camera, camera_z);
	my_settings.set_float3(TARGET_POSITION, target_x, height_target, target_z);

	return 0;
}



//
// Generate fbm map and render to relief_view
//
void main_window::generate_map(void)
{

	char buf[30];
	sprintf(buf, "%d", int (get_seed()));
	if (LockLooper()) {
		text_seed->SetText(buf);
		b_generate->SetEnabled(false);
		UnlockLooper();

	}

	my_map = new hfield(get_type(), RELIEF_ORDER, get_fd());


	my_map->seed_rand(get_seed());
	my_map->generate(8 /*0.75*/, get_fd());

	if (LockLooper()) {
		b_generate->SetEnabled(true);
		UnlockLooper();
	}

//	const float *height = my_map->get_data();
	const float** height2 = my_map->get_data2();

	const int num = my_map->get_width();
	const int bpr = relief_view->get_bytes_per_row();

	uint32 *p32 = (uint32*) relief_view->get_bitmap_data();

	int i = 0;

	for (int y=0; y<num-1; y++) {
		for (int x=0; x<num-1; x++) {

			uint32 my_col;
			if (height2[x][y] < -4.0) {
				my_col = (55 << 16) | (55 << 8) | 230;
			} else if (height2[x][y] < 0.0) {
				my_col = (60 << 16) | (60 << 8) | (245);
			} else {
				int h = int (10.0 * height2[x][y]);

				if (h > 250)	h = 250;
				if (h < 0)		h = 0;

				float param = h/256.0;

				fcolour c = texture::spline(param,	
									sizeof(col_array)/sizeof(col_array[0]),
									col_array);
				c.clamp();

				my_col = c.get_RGB32();
			}

			i++;
			p32[x] = my_col;
		}
		p32 += bpr/4;
		i++;	// Skip last column to make map width even
	}

	BRect view_rect(0, 0, RELIEF_WIDTH-1, RELIEF_WIDTH-1);
	if (LockLooper()) {
		relief_view->refresh(view_rect);
	
		// Update widets
		relief_view->reset_camera();
		update_text_display();

		update_all(true);
		UnlockLooper();
	}

}





//
// Deal with window close gadget
//
bool main_window::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return TRUE;
}



//
// Deal with incoming messages to window
//
void main_window::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		case MSG_ABOUT:
		{
			char str[256];
			sprintf(str, "Landscape                        \n"
				     	 "Copyright (C) 1998-2001 J. Belson\n"
						 "jon@witchspace.com           ");
			BAlert *the_alert = new BAlert("", str, "OK");
			the_alert->Go();
			break;
		}

		case MSG_LOAD:
			//ni::not_implemented();
			break;

		case MSG_SAVE_IMAGE:
		{
			bool val;
			my_settings.get_bool(IMAGE_TO_SAVE, &val);
			if (!val) {
				BAlert* alert = new BAlert("", "Please render an image first", "OK");
				alert->Go();
			} else if (w_render) {
				save_panel->Show();
			}
		}
			break;

		case B_SAVE_REQUESTED:
		case MSG_DO_SAVE:
			save_image(message);
			break;

		case MSG_QUIT:
			be_app->PostMessage(B_QUIT_REQUESTED);
			break;		// Hah!

		case MSG_RAND:
			cout << "Randomise" << endl;
			randomise = 1 - randomise;
			break;

		case MSG_SMOOTH:
			cout << "Smooth" << endl;
			break;

		case MSG_FD:
			set_fd(atof(text_fd_subdivision->Text()));
			my_settings.set_float(SUBDIVISION_FD, get_fd());
			break;

		case MSG_FD_SYNTHESIS:
			my_settings.set_float(SYNTHESIS_FD, atof(text_fd_synthesis->Text()));
			break;

		case MSG_METHOD:
			{
			int32 method;
			if (message->FindInt32("index", &method) == B_NO_ERROR) {
				switch (method) {
					case 0:
						set_type(HF_PERLIN);
						layers->ActivateLayer(0);
						break;
					case 1:
						set_type(HF_FBM);
						layers->ActivateLayer(1);
						break;
					case 2:
						set_type(HF_SYNTHESIS);
						layers->ActivateLayer(2);
						break;
					default:
						cerr << "Help!! Unknown hf method" << endl;
						exit (1);
				}
			}
			}
			break;

		case MSG_MENU_PREVIEW:
		{
			if (!w_preview) {
				w_preview = open_window<preview_window>("Preview");

				BMessage msg_seed(MSG_NEW_MAP);
				w_preview->PostMessage(&msg_seed);

				menu_preview->SetMarked(true);
			} else {
				w_preview->Activate();
			}
			break;
		}


		case MSG_MENU_RENDER:
			menu_save_image->SetEnabled(true);

			if (!w_render) {
				w_render = open_window<render_window>("Render");

				BMessage msg_seed(MSG_NEW_MAP);
				w_render->PostMessage(&msg_seed);

				menu_render->SetMarked(true);
			} else {
				w_render->Activate();
			}
			break;


		case MSG_MENU_PALETTE:
			if (!w_palette) {
				w_palette = open_tool_window<palette_window>("Palette");
			} else {
				w_palette->Activate();
			}
			break;



		case MSG_MENU_LIGHTING:
			if (!w_lighting) {
				w_lighting = open_tool_window<lighting_window>("Lighting");
			} else {
				w_lighting->Activate();
			}
			break;



		case MSG_MENU_TREES:
			if (!w_trees) {
				w_trees = open_tool_window<trees_window>("Trees");
			} else {
				w_trees->Activate();
			}
			break;



		case MSG_MENU_CLOUD:
			{
			if (!w_cloud) {
				w_cloud = open_window<cloud_window>("Clouds");
			} else {
				w_cloud->Activate();
			}
			break;
		}


		case MSG_MENU_ATMOSPHERICS:
			{
			if (!w_atmospherics) {
				w_atmospherics = open_tool_window<atmospherics_window>("Atmospherics");
			} else {
				w_atmospherics->Activate();
			}
			break;
		}

		case MSG_MENU_TEXTURE:
			{
			if (!w_texture) {
				w_texture = open_tool_window<texture_window>("Texture");
			} else {
				w_texture->Activate();
			}
			break;
		}

		case MSG_MENU_WATER:
			{
			if (!w_water) {
				w_water = open_tool_window<water_window>("Water");
			} else {
				w_water->Activate();
			}
			break;
		}


		case MSG_GEN:
			{
			cout << "Generate" << endl;

			my_settings.set_int(PREVIEW_HF_TYPE, get_type());
			if (randomise) {
				//seed = rand();	//time(NULL);
				set_seed(real_time_clock());
			}

			generate_map();

			}
			break;


		// User clicked on relief view...
		case mouse_bm_view::MSG_NEW_CAMERA:
		case mouse_bm_view::MSG_NEW_TARGET:
			// Eat any other similar messages in queue...
			BMessage* msg;
			MessageQueue()->Lock();
			while ((msg = MessageQueue()->FindMessage(mouse_bm_view::MSG_NEW_CAMERA, 0)) != NULL) {
				MessageQueue()->RemoveMessage(msg);
				delete msg;
			}
			while ((msg = MessageQueue()->FindMessage(mouse_bm_view::MSG_NEW_TARGET, 0)) != NULL) {
				MessageQueue()->RemoveMessage(msg);
				delete msg;
			}
			MessageQueue()->Unlock();

			update_all(false);
			break;


		case MSG_COORD_CHANGE:
		{
			float x, y, h;

			x = atof(text_camera_x->Text());
			y = atof(text_camera_y->Text());
			h = atof(text_camera_h->Text());

			if (x < 0.0)			x = 0.0;
			if (y < 0.0)			y = 0.0;
			if (x > RELIEF_WIDTH-1)	x = RELIEF_WIDTH-1;
			if (y > RELIEF_WIDTH-1)	y = RELIEF_WIDTH-1;

			relief_view->set_camera_pos(int (x), int (y));
			x = atof(text_target_x->Text());
			y = atof(text_target_y->Text());
			h = atof(text_target_h->Text());

			if (x < 0.0)			x = 0.0;
			if (y < 0.0)			y = 0.0;
			if (x > RELIEF_WIDTH-1)	x = RELIEF_WIDTH-1;
			if (y > RELIEF_WIDTH-1)	y = RELIEF_WIDTH-1;
			relief_view->set_target_pos(x, y);

			update_all(false);
		}
			break;


		case MSG_PREVIEW_CLOSED:
			menu_preview->SetMarked(false);
			w_preview = 0;
			break;

		case MSG_RENDER_CLOSED:
			menu_save_image->SetEnabled(false);
			menu_render->SetMarked(false);
			w_render = 0;
			break;

		case MSG_CLOUD_CLOSED:
			w_cloud = 0;
			break;

		case MSG_PALETTE_CLOSED:
			w_palette = 0;
			break;

		case MSG_TREES_CLOSED:
			w_trees = 0;
			break;

		case MSG_LIGHTING_CLOSED:
			w_lighting = 0;
			break;

		case MSG_ATMOSPHERICS_CLOSED:
			w_atmospherics = 0;
			break;

		case MSG_TEXTURE_CLOSED:
			w_texture = 0;
			break;

		case MSG_WATER_CLOSED:
			w_water = 0;
			break;

		case '!spn':
			my_settings.set_int(PERLIN_ITERATIONS, int (spin_iterations->Value()));
			my_settings.set_int(SYNTHESIS_ITERATIONS, int (spin_synth_iterations->Value()));
			break;


		default:
			//cout << "main_window::MessageReceived()" << endl;
			//message->PrintToStream();
			MWindow::MessageReceived(message);
			break;
	}
}



//
// Update camera coord display
//
void main_window::update_text_display(void)
{
	char buf[256];

	int x, y, height;

	relief_view->get_camera_pos(&x, &y);
	height = int (my_map->get_height(x, y));
	if (height < 0) height = 0;

	sprintf(buf, "%.1f", float (x));
	text_camera_x->SetText(buf);
	sprintf(buf, "%.1f", float (y));
	text_camera_y->SetText(buf);
	sprintf(buf, "%.1f", float (height));
	text_camera_h->SetText(buf);

	relief_view->get_target_pos(&x, &y);
	height = int (my_map->get_height(x, y));
	if (height < 0) height = 0;

	sprintf(buf, "%.1f", float (x));
	text_target_x->SetText(buf);
	sprintf(buf, "%.1f", float (y));
	text_target_y->SetText(buf);
	sprintf(buf, "%.1f", float (height));
	text_target_h->SetText(buf);
}



//
// Camera moved, so update any windows
//
void main_window::update_all(bool rerender)
{
	// Display new X, Y, H coordinates
	update_text_display();

	char buf[20];
	sprintf(buf, "%d", int (get_seed()));
	text_seed->SetText(buf);
			
	int x, y;
	// Got a new camera coordinate, so tell other windows..
	relief_view->get_camera_pos(&x, &y);
	float height = my_map->get_height(x, y);
	if (height < 0.0) {
		height = 0.0;
	}

	float angle = calculate_camera_angle();

	// Update global preferences
	my_settings.set_float3(CAMERA_COORDS, RELIEF_WIDTH/2 - x, height+10, RELIEF_WIDTH/2 - y);
	my_settings.set_float3(CAMERA_ANGLE, 0.0, angle, 0.0);
	my_settings.set_int(PREVIEW_SEED, get_seed());

	BMessage msg;
	if (rerender) {
		msg = BMessage(MSG_NEW_MAP);
	} else {
		msg = BMessage(MSG_RERENDER);
	}

	if (w_preview)	w_preview->PostMessage(&msg);

	msg = BMessage(MSG_NEW_MAP);
	if (w_render)	w_render->PostMessage(&msg);
}







//
// Toggle marked/not marked status of menu itme
//
void main_window::set_menu_state(BMessage *msg, bool state)
{
	BMenuItem *i;
	msg->FindPointer("source", (void **) &i);
	BMenu *m = i->Menu();
	long index = m->IndexOf(i);
	m->ItemAt(index)->SetMarked(state);
}




//
// Save rendered image
//
void main_window::save_image(BMessage* msg)
{
	entry_ref	ref;
	status_t	err;
	const char*	name;
	BEntry		entry;
	BPath		path;

	if ((err = msg->FindRef("directory", &ref)) != B_OK) {
		return;
	}
	if ((err = msg->FindString("name", &name)) != B_OK) {
		return;
	}

	if ((err = entry.SetTo(&ref)) != B_OK) {
		return;
	}

	entry.GetPath(&path);
	path.Append(name);

	BFile file(path.Path(), B_WRITE_ONLY | B_CREATE_FILE | B_ERASE_FILE);
	if (file.InitCheck() != B_OK) {
		BAlert *alert = new BAlert(NULL, "Could not create file.", "OK");
		alert->Go();
		return;
	}

	// Recover the necessary data from the message
	translator_id	*id;
	uint32			format;
	ssize_t length = sizeof(translator_id);
	if (msg->FindData("translator_id", B_RAW_TYPE, (const void **) &id, &length) != B_OK) {
		cerr << "FindData() failed" << endl;
		return;
	}
	if (msg->FindInt32("translator_format", (int32 *) &format) != B_OK) {
		cerr << "FindInt32() failed" << endl;
		return;
	}


	BBitmap* bm = (BBitmap*) w_render->get_bitmap();

	BBitmapStream stream((BBitmap*) bm);

	BTranslatorRoster *roster = BTranslatorRoster::Default();
	if (roster->Translate(*id, &stream, 0, &file, format) != B_OK) {
		BAlert *alert = new BAlert(NULL, "Save failed.", "OK");
		alert->Go();
	}
	cout << "done" << endl;

	stream.DetachBitmap(&bm);

}








