//
// main_window.h
//
// Main control window
//
// Copyright (c) J. Belson	1998.12.12
//


#ifndef _MAIN_WINDOW_H_
#define _MAIN_WINDOW_H_


#include <time.h>

#include <MenuItem.h>

#include "LayeredGroup.h"
#include "MWindow.h"
#include "MTextControl.h"
#include "SpinButton.h"


#include "base_main_window.h"


#include "MApplication.h"
#include "MButton.h"
#include "MBViewWrapper.h"
#include "MBorder.h"
#include "MCheckBox.h"
#include "HGroup.h"
#include "MMenuBar.h"
#include "MPopup.h"
#include "MSlider.h"
#include "PropGadget.h"
#include "Space.h"
#include "TabGroup.h"
#include "VGroup.h"

#include "messages.h"
#include "not_implemented.h"

#include "TabGroup.h"


#define	MSG_GEN					'ID-1'
#define	MSG_RAND				'ID-2'
#define	MSG_SMOOTH				'ID-3'
#define MSG_FD					'ID-4'
#define MSG_FD_SYNTHESIS		'ID-5'

#define MSG_MENU_PREVIEW		'MN-0'
#define MSG_MENU_RENDER			'MN-1'
#define	MSG_MENU_LIGHTING		'MN-2'
#define	MSG_MENU_CLOUD			'MN-3'
#define	MSG_MENU_PALETTE		'MN-4'
#define	MSG_MENU_TREES			'MN-5'
#define MSG_MENU_ATMOSPHERICS	'MN-6'
#define MSG_MENU_TEXTURE		'MN-7'
#define MSG_MENU_WATER			'MN-8'

#define MSG_QUIT				'ID-b'
#define	MSG_ABOUT				'ID-c'
#define MSG_LOAD				'ID-d'
#define	MSG_SAVE_IMAGE			'ID-e'
//#define MSG_NEW_CAMERA		'ID-f'
//#define MSG_NEW_TARGET		'ID-g'
#define MSG_UPDATE				'ID-h'
#define	MSG_COORD_CHANGE		'ID-i'
#define MSG_METHOD				'ID-j'

#define	MSG_DO_SAVE				'ID-z'


#include "mouse_bm_view.h"
#include "save_image_panel.h"



class main_window : public smart_window, public base_main_window {


private:
	mouse_bm_view*	relief_view;

	MTextControl* text_fd_subdivision;
	MTextControl* text_fd_synthesis;
	MTextControl* text_seed;

	MTextControl*	text_camera_x;
	MTextControl*	text_camera_y;
	MTextControl*	text_camera_h;
	MTextControl*	text_target_x;
	MTextControl*	text_target_y;
	MTextControl*	text_target_h;
	MButton*		b_generate;
	SpinButton*		spin_iterations;
	SpinButton*		spin_synth_iterations;
	LayeredGroup*	layers;

	// Menu items
	BMenuItem*		menu_save_image;
	BMenuItem*		menu_preview;
	BMenuItem*		menu_render;
	BMenuItem*		menu_lighting;
	BMenuItem*		menu_trees;

	template<class T> T* open_window(char* title);
	template<class T> T* open_tool_window(char* title);

	save_image_panel* save_panel;

	float calculate_camera_angle(void);

	// Update camera coord display
	void update_text_display(void);

	// Camera moved, update any other windows
	void update_all(bool rerender);

	// Save rendered image
	void save_image(BMessage* msg);


public:
	main_window(BRect, char*, window_type, ulong);
	~main_window();

	void set_menu_state(BMessage *msg, bool state);

	virtual void generate_map(void);

	virtual bool QuitRequested();
	virtual void Quit() { smart_window::Quit(); };
	virtual void MessageReceived(BMessage*);

};


#endif	// _MAIN_WINDOW_H_



