//
// mouse_bm_view.h
//
// View that responds to mouse clicks
//
// Copyright (c) J. Belson	1998.12.12
//


#ifndef _MOUSE_BM_VIEW_H_
#define _MOUSE_BM_VIEW_H_

#include <Point.h>

#include "Bitmap.h"
#include "Message.h"
#include "View.h"
#include "Window.h"


// Class that sends mouse position to parent BLooper
class mouse_bm_view : public BView {
	private:
		BLooper *parent;

		BBitmap*bm;
		uchar*	bm_data;
		int		bytes_per_row;

		int		WIDTH;

		enum {
			TRACKING_NONE,
			TRACKING_CAMERA,
			TRACKING_TARGET
		} tracking;

		int camera_x, camera_y;
		int target_x, target_y;

		void draw_line_of_sight(void);

		virtual void Draw(BRect rect) {
			DrawBitmap(bm, rect, rect);
			draw_line_of_sight();
		};

		void update(BPoint point);

		virtual void MouseDown(BPoint point);
		virtual void MouseUp(BPoint point);
		virtual void MouseMoved(BPoint point, uint32 transit, const BMessage *message);


	public:
		mouse_bm_view(BRect rect);

		static const uint32 MSG_NEW_CAMERA = 'ID-f';
		static const uint32 MSG_NEW_TARGET = 'ID-g';

		uchar *get_bitmap_data()	{ return bm_data; }
		int get_bytes_per_row()		{ return bytes_per_row; }

		void refresh(BRect rect) { Draw(rect); }

		void reset_camera();
		void get_camera_pos(int *x, int *y);
		void set_camera_pos(int x, int y);
		void get_target_pos(int *x, int *y);
		void set_target_pos(int x, int y);
};


#endif	// _MOUSE_BM_VIEW_H_

