//
// render_window.cc
//
// Main window
//
// Copyright (c) J. Belson	1999.01.04
//


#include <iostream.h>
#include <stdio.h>

#include <Bitmap.h>
#include <kernel/OS.h>
#include <Screen.h>
#include <Message.h>

#include "MApplication.h"
#include "VGroup.h"
#include "HGroup.h"
#include "MButton.h"
#include "MBViewWrapper.h"
#include "MBorder.h"
#include "PropGadget.h"
#include "MRadioGroup.h"
#include "Space.h"

#include "render_window.h"
#include "renderer_flat.h"
#include "renderer_flat_int.h"
#include "renderer_smooth.h"
#include "renderer_smooth_int.h"
#include "renderer_texture_int.h"

#define	MSG_TEXTURE		'ID-1'
//#define	MSG_RERENDER	'ID-2'
#define	MSG_SMOOTHING	'ID-3'
#define MSG_LIGHTING	'ID-4'
#define	MSG_CLOUDS		'ID-5'
#define	MSG_ATMOSPHERICS 'ID-6'
#define	MSG_SHADOWS		'ID-7'
#define	MSG_REFLECTIONS	'ID-8'
#define MSG_TREES		'ID-9'
#define MSG_SIZE		'ID-a'
#define MSG_UPDATE		'ID-b'
#define MSG_THREADS		'ID-c'


// Valid output window sizes
const struct {
	int x, y;
} w_size[] = { {320, 240}, {640, 480}, {800, 600}, {1024, 768}, {1280, 1024}};


//
// Constructor/destructor
//
render_window::render_window(BWindow *window, BRect r, char *name, window_type t, ulong mode) :
	smart_window(r, name, t, mode),
	base_render_window(),
	parent(window)
{
	// Open output window
	open_output_window();

	system_info s;
	get_system_info(&s);
	num_threads = s.cpu_count;

	MPopup		*popup_update;

	msg_texture		= new BMessage(MSG_TEXTURE);
	msg_lighting	= new BMessage(MSG_LIGHTING);
	msg_shadows		= new BMessage(MSG_SHADOWS);
	msg_reflections	= new BMessage(MSG_REFLECTIONS);
	msg_trees		= new BMessage(MSG_TREES);
	msg_atmospherics= new BMessage(MSG_ATMOSPHERICS);
	msg_clouds		= new BMessage(MSG_CLOUDS);
	msg_smoothing	= new BMessage(MSG_SMOOTHING);
	msg_size		= new BMessage(MSG_SIZE);
	msg_update		= new BMessage(MSG_UPDATE);
	msg_threads		= new BMessage(MSG_UPDATE);
	msg_render		= new BMessage(MSG_RERENDER);



	cb_textures		= new MCheckBox("Textures",	msg_texture, NULL, textures);
	cb_lighting		= new MCheckBox("Lighting",	msg_lighting, NULL, lighting);
	cb_shadows		= new MCheckBox("Shadows",	msg_shadows, NULL, shadows);
	cb_reflections	= new MCheckBox("Reflections",msg_reflections, NULL, reflections);						
	cb_trees		= new MCheckBox("Trees",	msg_trees, NULL, trees);
	cb_atmospherics	= new MCheckBox("Atmospherics", msg_atmospherics, NULL, atmospherics);
	cb_clouds		= new MCheckBox("Clouds", msg_clouds, NULL, clouds);
	cb_smoothing	= new MCheckBox("Smoothing",msg_smoothing, NULL, smoothing);
	popup_size		= new MPopup("Output Size",msg_size, this,
							"320x240", "640x480", "800x600", "1024x768", "1280x1024", 0);
	popup_update	= new MPopup("Update Rate", msg_update, this, "Never", "1 per sec", "2 per sec", "3 per sec", 0);
	popup_threads	= new MPopup("Render threads", msg_threads, this,
							"1", "2", "3", "4", 0);
	MButton* b_render = new MButton("Render", msg_render, 0, minimax(-1,-1,-1,-1));


	MView *top_view =
		new VGroup
		(
			new Space(minimax(4, 4, 4, 4)),
			//new MBViewWrapper(relief_view),
			new MBorder
			(
				M_LABELED_BORDER, 12, "Options",
				new VGroup
				(
					new HGroup(
						new VGroup(
							//new Space(minimax(6, 6, 6, 6)),
							cb_smoothing,
							cb_lighting,
							cb_clouds,
							cb_atmospherics,
							//new Space(minimax(6, 6, 6, 6)),
							0
						),
						new Space(minimax(6, 6, 6, 6)),
						new VGroup(
							//new Space(minimax(6, 6, 6, 6)),
							cb_textures,
							cb_reflections,
							cb_trees,
							cb_shadows,
							0
						),
						0
					),
					0
				)
			),
			new MBorder
			(
				M_LABELED_BORDER, 12, "Output",
				new VGroup
				(
					popup_size,
					popup_update,
					popup_threads,
					0
				)
			),
			b_render,
			new Space(minimax(4, 4, 4, 4)),
			0
		);


	DivideSame(popup_size, popup_update, popup_threads, 0);

	// top_view is an MView, so we need to cast it to a BView.
	AddChild(dynamic_cast<BView*> (top_view));

	popup_size->SetActive(1);
	popup_update->SetActive(3);
	popup_threads->SetActive(num_threads - 1);

	// Disable unimplemented options
	cb_shadows->SetEnabled(false);
	cb_reflections->SetEnabled(false);
	cb_trees->SetEnabled(false);

	the_thread = 0;

}


render_window::~render_window()
{
	//delete relief_view;
	BMessage msg(MSG_RENDER_CLOSED);
	parent->PostMessage(&msg);

}



//
// Deal with window close gadget
//
bool render_window::QuitRequested()
{
	smart_window::QuitRequested();

	// Signal to view that we want it to stop any
	//  rendering activity, then wait for it.
	if (the_thread) {
		//my_view->stop_render();
		status_t status;
		wait_for_thread(the_thread, &status);
	}

	if (win->Lock()) {
		win->Quit();
	}

	return TRUE;
}



//
// Deal with incoming messages to window
//
void render_window::MessageReceived(BMessage *message)
{
	switch (message->what)
	{
		//
		// Local messages
		//
		case MSG_SMOOTHING:
			smoothing = !smoothing;
			cout << "Smoothing" << endl;
			break;

		case MSG_LIGHTING:
			lighting = !lighting;
			my_settings.set_bool(LIGHTING, &lighting);
			break;

		case MSG_CLOUDS:
			clouds = !clouds;
			cout << "Clouds" << endl;
			break;

		case MSG_ATMOSPHERICS:
			atmospherics = !atmospherics;
			my_settings.set_bool(ATMOSPHERICS, atmospherics);
			break;

		case MSG_TEXTURE:
			textures = !textures;
			cb_smoothing->SetEnabled(!textures);
			//my_settings.set_bool(TEXTURES, textures);
			break;

		case MSG_TREES:
//			ni::not_implemented();
			cout << "Trees" << endl;
			break;

		case MSG_SHADOWS:
//			ni::not_implemented();
			cout << "Shadows" << endl;
			break;

		case MSG_REFLECTIONS:
//			ni::not_implemented();
			cout << "Reflections" << endl;
			break;

		case MSG_SIZE:
			cout << "Output size" << endl;
			int32 output_size;
			if (message->FindInt32("index", &output_size) == B_NO_ERROR) {
				if (uint32(output_size) < sizeof(w_size)/sizeof(w_size[0])) {
					w_width  = w_size[output_size].x;
					w_height = w_size[output_size].y;
					open_output_window();
					//launch_render_thread();
				} else {
					cerr << "Illegal output size index " << endl;
				}
			}
			break;

		case MSG_UPDATE:
			int32 rate;
			if (message->FindInt32("index", &rate) == B_NO_ERROR) {
				redraw_rate = rate;
			}
			break;

		case MSG_RERENDER:
			if (the_thread) {
				cout << "render thread already running" << endl;
			} else {
				cout << "Rerender" << endl;
				launch_render_thread();
			}	
			break;

		case MSG_THREADS:
			int32 num;
			if (message->FindInt32("index", &num) == B_NO_ERROR) {
				num_threads = num + 1;
			}
			break;

//		case '!spn':
//			num_threads = int (spin_threads->Value());
//			break;


		//
		// Global messages
		//
		case MSG_NEW_MAP:
			cout << "render got MSG_UPDATE_MAP" << endl;		
			my_settings.get_float3(CAMERA_COORDS, &camera_x, &camera_y, &camera_z);
			my_settings.get_float3(CAMERA_ANGLE, &rot_a, &rot_b, &rot_c);
			my_settings.get_int(PREVIEW_SEED, &seed);
			my_settings.get_float(SYNTHESIS_FD, &fd);
			camera_y = /*-10*/ - camera_y      /*- 20*/; 
			break;

		default:
			//cout << "Message" << endl;
			MWindow::MessageReceived(message);
			break;
	}
}



void render_window::open_output_window(void)
{
	BRect rect;

	if (win) {
		rect = win->Frame();

		if (win->Lock())
			win->Quit();
	} else {
		rect = BRect(40, 40, 40+w_width, 40+w_height);
	}

	rect.right = rect.left + w_width - 1;
	rect.bottom = rect.top + w_height - 1;
	win = new bm_window(rect,	"Render Output", /*B_FLOATING_WINDOW*/ B_TITLED_WINDOW,
					B_NOT_RESIZABLE | B_NOT_CLOSABLE | B_NOT_ZOOMABLE, 32);
	win->Show();
}



void render_window::launch_render_thread(void)
{
	if (the_thread) {
		cerr << "Render thread already running!!" << endl;
	} else {
		the_thread = spawn_thread(render_thread_func, "Render Thread",
				B_NORMAL_PRIORITY, this);
		resume_thread(the_thread);
	}
}



//
// Static function to launch thread
//
long render_window::render_thread_func(void *obj )
{
	render_window *my_obj = (render_window *) obj;
	my_obj->main_render_thread();

	return 0;
}
long render_window::render_strip_thread_func(void *obj)
{
	renderer* my_renderer = (renderer *) obj;
	my_renderer->do_render();

	return 0;
}



//
// This routine does all the hard work of generating and
//  rendering the landscape.
//
void render_window::main_render_thread(void)
{
	cout << "render_window::render_thread()" << endl;

	// Too late to resize now...
	if (LockLooper()) {
		popup_size->SetEnabled(false);
		UnlockLooper();
	}

	// Get requested number of threads (can't change during a render ;^)
	int active_threads = num_threads;

	bigtime_t time = system_time();

	progress *prog = new progress("Initialising");

	win->draw_sky( 1 /*, camera_y*/);

	zbuffer *zb = new zbuffer(w_width, w_height);
	zb->clear_buffer();

	// Start pulse events
	win->set_redraw_rate(redraw_rate);

	float i, j, k;
	my_settings.get_float3(LIGHT_VECTOR, &i, &j, &k);
	vector3d light(i, j, k);
//	vector3d light(0.0, 1.0, 0.0);
	//light.normalise();
cout << "Light vector = " << light << endl;

	//
	// Render clouds
	//
	if (clouds) {
		hf_fbm* cloud_map;
		cloud* my_cloud;
		int cloud_size;

		cloud_map = new hf_fbm(8, 0.65 /*my_prefs.get_cloud_fd()*/ /*fd*/);
		cloud_map->seed_rand(real_time_clock());
//		cloud_map->generate(0.5, fd);
		cloud_size = cloud_map->get_width();

		my_cloud = new cloud(cloud_map);
		// Why minus angle_b????
//		my_cloud->rotate(angle_a, /*-*/ angle_b, angle_c /*0.0 , 0.0, 0.0*/);
//		my_cloud->translate(0.0, 0.0, 0.0);


		// Get camera position
		int cam_x, cam_y, cam_z;
		int tar_x, tar_y, tar_z;

		my_settings.get_float3(CAMERA_POSITION, &cam_x, &cam_y, &cam_z);
		my_settings.get_float3(TARGET_POSITION, &tar_x, &tar_y, &tar_z);

		// Make vector in this direction
		//vector3d heading(tar_x - cam_x, tar_y - cam_y, tar_z - cam_z);
	vector3d heading(cam_x - tar_x, cam_y - tar_y, cam_z - tar_z);

		heading.normalise();

		// Set viewpoint, raising camera slightly
		my_cloud->set_view(point3d(cam_x, cam_y+10, cam_z), /*vector3d(0, 0, 0)*/ heading);




		my_cloud->transform();
		my_cloud->perspective();

		// Scale transformed points to our window...
		my_cloud->viewing_transform(w_width/2, -float(w_width)/cloud_size,
								w_height/2, float(w_width)/cloud_size);

		const triangle3d* t3d	= my_cloud->get_tri3d();
		const point2d* p2d		= my_cloud->get_point2d();
		const vector3d* v3d		= my_cloud->get_normal();

		prog->set_message("Rendering clouds...");

		int num_tri = (cloud_size-1)*(cloud_size-1)*2;

		int strip_width = w_width/active_threads;
	
		// Set up each strip, then spawn a rendering thread
		for (int i=0; i<active_threads; i++) {
			renderer* r;
	
			r = new renderer_smooth(i*strip_width, strip_width, prog);
//			r->set_light(light);
			r->kludge_light(true);

			r->set_blend(true);
			r->set_fog(true);

			r->setup(	t3d, p2d, v3d, num_tri,
						win->get_buffer(),
						win->bytes_per_row,
						w_width, w_height,
						zb);
	
			thread_id id = spawn_thread(render_strip_thread_func,
									"Render Strip Thread",
									B_NORMAL_PRIORITY,
									(void*) r);

			resume_thread(id);

			render_thread[i]	= r;
			render_thread_id[i]	= id;
		}

		for (int i=0; i<active_threads; i++) {
			status_t status;
			wait_for_thread(render_thread_id[i], &status);
			delete render_thread[i];
		}

		delete my_cloud;
		delete cloud_map;
	}


	//
	// Render terrain...
	//
	int type;
	my_settings.get_int(PREVIEW_HF_TYPE, &type);

	my_map = new hfield(hf_type(type), 8, fd);


cout << "seed = " << seed << endl;
	prog->set_message("Generating landscape...");
	my_map->seed_rand(seed);
//	my_map->test_pattern(enumWAVE);
	my_map->generate(8.0, fd);

	my_terrain	= new terrain(my_map);

	// Get camera position
	int cam_x, cam_y, cam_z;
	int tar_x, tar_y, tar_z;

	my_settings.get_float3(CAMERA_POSITION, &cam_x, &cam_y, &cam_z);
	my_settings.get_float3(TARGET_POSITION, &tar_x, &tar_y, &tar_z);


	// Make vector in this direction
	vector3d heading(cam_x - tar_x, cam_y - tar_y, cam_z - tar_z);
	heading.normalise();

	// Set viewpoint, raising camera slightly
	my_terrain->set_view(point3d(cam_x, cam_y+10, cam_z), heading);


	//my_terrain->translate(camera_x, camera_y, camera_z);
	//my_terrain->rotate(angle_a, angle_b, angle_c);
	my_terrain->transform();
	my_terrain->perspective();

	int map_size = my_map->get_width();

	// Scale transformed points to our window...
	my_terrain->viewing_transform(	w_width/2, -float(w_width)/map_size,
									w_height/2, float(w_width)/map_size);


	const triangle3d* t3d	= my_terrain->get_tri3d();
	const point2d* p2d		= my_terrain->get_point2d();
	const vector3d* v3d		= my_terrain->get_normal();
	prog->set_message("Rendering landscape...");

	int num_tri = (map_size-1)*(map_size-1)*2;
	prog->set_maximum(num_tri);

	zb->clear_buffer();



	int strip_width = w_width/active_threads;

	// Set up each strip, then spawn a rendering thread
	for (int i=0; i<active_threads; i++) {
cout << "Thread " << i << "\t" << i*strip_width << ", " << strip_width << endl;

		renderer* r;
		if (textures) {
			r = new renderer_texture_int(i*strip_width, strip_width, prog);
			r->set_light(light);
			r->kludge_light(true);
		}
		else if (smoothing) {
			r = new renderer_smooth_int(i*strip_width, strip_width, prog);
			r->kludge_light(true);
			r->set_light(light);
		}
		else {
			r = new renderer_flat_int(i*strip_width, strip_width, prog);
			r->set_light(light);
		}

		if (atmospherics) {
			r->set_fog(true);
		}
	
		r->setup(
			t3d, p2d, v3d, num_tri,
			win->get_buffer(),
			win->bytes_per_row,
			w_width, w_height,
			zb);

		thread_id id = spawn_thread(render_strip_thread_func,
									"Render Strip Thread",
									B_NORMAL_PRIORITY,
									(void*) r);

		resume_thread(id);

		render_thread[i]	= r;
		render_thread_id[i]	= id;
	}
cout << "Waitong for threads" << endl;
	for (int i=0; i<active_threads; i++) {
		status_t status;
		wait_for_thread(render_thread_id[i], &status);
		delete render_thread[i];
	}

	// Update display
	win->refresh();

	// Disable automatic window update
	win->set_redraw_rate(0);

	float duration = (system_time() - time)/float(1000000);

	char buf[50];
	sprintf(buf, "Render Output - %.1f seconds", duration);

	win->SetTitle(buf);

	delete zb;
	delete my_map;
	delete my_terrain;

	if (prog->Lock()) {
		prog->Quit();
	}


	the_thread = 0;

	// We appear to have rendered an image, so...
	my_settings.set_bool(IMAGE_TO_SAVE, true);

	if (LockLooper()) {
		popup_size->SetEnabled(true);
		UnlockLooper();
	}
}


//
// Access raw image buffer
//
const BBitmap* render_window::get_bitmap(void)
{
	return win->get_bitmap();
}



