//
// save_image_panel.cc
//
// Main control window
//
// Copyright (c) J. Belson	2000.03.09
//


#include <Alert.h>
#include <Button.h>
#include <Menu.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <Window.h>

#include <stdio.h>
#include <string.h>

#include "save_image_panel.h"
#include "smart_window.h"


#define MSG_SETTINGS		'm-01'


save_image_panel::save_image_panel(BMessenger* target, BMessage* msg)
	: BFilePanel(B_SAVE_PANEL, target, NULL, 0, false, msg, NULL, false, true), BHandler("my_handler")
{
	// Store 'what' field of notification field
	what = msg->what;

	if (Window()->Lock()) {
		// Customise here
		BView* background	= Window()->ChildAt(0);
		BMenuBar* menubar	= (BMenuBar*) background->FindView("MenuBar");
		BButton* cancel		= (BButton *) background->FindView("cancel button");
		BButton* save		= (BButton *) background->FindView("default button");

		if (background == 0 || menubar == 0 || cancel == 0 || save == 0) {
			Window()->Unlock();
			return;
		}

		Window()->AddHandler(this);


		BRect r = cancel->Frame();
		float width = be_plain_font->StringWidth("Settings...") + 20.0;
		r.right = r.left + width;	// Resize to hold text...

		float offset = save->Frame().right - cancel->Frame().right;
		r.OffsetBy(-offset + 20, 0.0);

		BButton *settings = new BButton(r, "Settings", "Settings...", new BMessage(MSG_SETTINGS),
			B_FOLLOW_RIGHT | B_FOLLOW_BOTTOM, B_WILL_DRAW | B_NAVIGABLE);
		background->AddChild(settings);
		settings->SetTarget(this);



		BMenu* menu = new BMenu("Format");
		menubar->AddItem(menu);

		// Create menuitem for each save format...
		BTranslatorRoster *roster = BTranslatorRoster::Default();
		translator_id *translators;
		int32 count;

		roster->GetAllTranslators(&translators, &count);
		const translation_format *format;
		int32 format_count;
		
		for (int x = 0; x < count; x++) {
			// Determine which formats this one can write
			roster->GetOutputFormats(translators[x], &format, &format_count);
			for (int y = 0; y < format_count; y++) {

				// Check if this is an image translator
				if (format[y].group == B_TRANSLATOR_BITMAP) {

					// If this format saves to some native format build a menu item for it
					if (format[y].type == B_TRANSLATOR_BITMAP) {
						continue;
					}
					image_menu_item* item = new image_menu_item(format[y].name,
						new BMessage(MSG_SAVE_FORMAT), translators[x], format[y].type);

					item->SetTarget(this);
					menu->AddItem(item);
					break;
				}
			}
		}
		delete [] translators;

		// Select first item as default
		id				= ((image_menu_item*) menu->ItemAt(0))->id;
		this->format	= ((image_menu_item*) menu->ItemAt(0))->format;




		Window()->SetTitle("Save image...");
		Window()->Unlock();
	}

	BMessage* m = new BMessage(what);
	m->AddData("translator_id", B_RAW_TYPE, &(this->id), sizeof(translator_id));
	m->AddInt32("translator_format", this->format);
	SetMessage(m);
}

void save_image_panel::MessageReceived(BMessage *message)
{
	switch (message->what) {
		case MSG_SAVE_FORMAT:
		{
			// 'source' field points to BMenuItem
			image_menu_item* menuitem;
			void* v;
			if (message->FindPointer("source", &v) == B_NO_ERROR) {
				menuitem = (image_menu_item*) v;

				BMessage* msg = new BMessage(what);
				msg->AddData("translator_id", B_RAW_TYPE, &(menuitem->id), sizeof(translator_id));
				msg->AddInt32("translator_format", menuitem->format);
				SetMessage(msg);

				id		= menuitem->id;
				format	= menuitem->format;
			} else {
				puts("Find failed!!");
			}
		}
			break;

		case MSG_SETTINGS:
		{
			BTranslatorRoster *roster = BTranslatorRoster::Default();
			BView *view;
			BRect rect(0, 0, 239, 239);
			
			// Build a window around this translator's configuration view
			status_t err = roster->MakeConfigurationView(id, NULL, &view, &rect);
			if (err != B_OK || view == NULL) {
				BAlert *alert = new BAlert(NULL, strerror(err), "OK");
				alert->Go();
			} else {
				if (configwindow != NULL) {
					if (configwindow->Lock()) configwindow->Quit();
				}
				configwindow = new smart_window(rect, "Translator Settings", B_TITLED_WINDOW,
						B_NOT_ZOOMABLE | B_NOT_RESIZABLE);
				configwindow->AddChild(view);

				// Just to make sure
				//view->MoveTo(0, 0);
				//view->ResizeTo(rect.Width(), rect.Height());
				//configwindow->MoveTo(100, 100);
				configwindow->Show();
			}

		}
		break;

	default:
		BHandler::MessageReceived(message);
		break;
}
}

