//
// Slider.cc
//
// J. Belson	2000.06.08
//


#include "slider.h"

#include <cstdio>
#include <Bitmap.h>
#include <Region.h>


Slider::Slider(
			BRect			frame,
			const char*		name,
			const char*		label,
			ConvertColour*	Conv,
			BMessage*		message,
			uint32			resizingMode,
			uint32			flags)
	: BControl(
		frame, name, label, message, resizingMode, flags),
	Convert(Conv)
{
	// Make a bitmap to render into
	BRect rect = BarFrame();
	rect.InsetBy(1.0, 1.0);

BRect r = rect;
//r.InsetBy(0.0, 1.0);

	bm = new BBitmap(r /*ect*/, B_RGB32);
	bits = (uint32 *) bm->Bits(); 

	// Set bar colour to reasonable default
	col init_l = { 0.0, 0.0, 0.0};
	col init_r = { 1.0, 1.0, 1.0};
	left = init_l;
	right = init_r;

	// Default slider position
	position = Bounds().left;	//frame.left;

	last_thumb = ThumbFrame();

	// Don't erase during update
	SetViewColor(B_TRANSPARENT_COLOR);
}



//
// Draw graded bar
//
void Slider::DrawBar(void)
{
	BRect full_rect = Bounds();

	BRect border_rect = BarFrame();
	BRect frame_rect = border_rect;
	frame_rect.InsetBy(1.0, 1.0);

	int steps = int (frame_rect.Width() + 1.0);

	float a = left.a;
	float b = left.b;
	float c = left.c;

	float a_inc = (right.a - left.a)/steps;
	float b_inc = (right.b - left.b)/steps;
	float c_inc = (right.c - left.c)/steps;

	int height = int (frame_rect.Height() + 0.5);
// Something not quite right here....
//printf("height = %d\n", height);
	for (int i=0; i<steps; i++) {

		// Convert from native format to RGB
		uint8 red, green, blue;
		col colour = { a, b, c };
		Convert->ToRGB(colour, &red, &green, &blue);

		// Draw strip with current colour
		uint32 col;
		#if __POWERPC__
			col = (blue)<<24 | (green)<<16 | (red)<<8;
		#else
			col = (red)<<16 | (green)<<8 | (blue);
		#endif
		for (int y=0; y<height; y++) {
			*(bits + y*bm->BytesPerRow()/4 + i) = col;
		}

		a += a_inc;
		b += b_inc;
		c += c_inc;
	}


	// Erase sections to left and right of bar
	FillRect(BRect(0, border_rect.top, border_rect.left-1, border_rect.bottom), B_SOLID_LOW);
	FillRect(BRect(border_rect.right, border_rect.top, full_rect.right, border_rect.bottom), B_SOLID_LOW);

	// Draw border
	SetHighColor(0, 0, 0);
	StrokeRect(border_rect);

	// Copy graded bitmap to view
	DrawBitmap(bm, frame_rect);
}



//
// Erase old thumb and draw it in new position
//
void Slider::DrawThumb(void)
{
	// Erase thumb sliding strip...
	FillRect(BRect(0, 0, Bounds().right, THUMB_HEIGHT-1), B_SOLID_LOW);

	// Calculate thumb size and draw it
	BRect r = ThumbFrame();
	float middle = r.left + (r.Width())/2.0;
	BPoint p[] = {
			BPoint(middle, r.bottom),
			BPoint(r.left, r.top),
			BPoint(r.right, r.top) };

	FillPolygon(p, 3);

	last_thumb = r;
}



//
// Return current bounding rect for frame
//
BRect Slider::ThumbFrame(void) const
{
	BRect bf = BarFrame();
	BRect rect = Bounds();

	BRect r = BRect(rect.left, bf.top - THUMB_HEIGHT, rect.left + THUMB_WIDTH, bf.top-1);

	r.OffsetBy(position, 0.0);

	return r;
}



//
// Return current bounding rect for bar
//
BRect Slider::BarFrame(void) const
{
	BRect r = Bounds();

	// Make space for the thumb...
	r.InsetBy(THUMB_WIDTH/2, 0.0);
	r.top += THUMB_HEIGHT;

	return r;
}



//
// Refresh display
//
void Slider::Draw(BRect rect)
{
	DrawBar();
	DrawThumb();
}



//
// If mouse is pressed over thumb, start tracking
//
void Slider::MouseDown(BPoint pt)
{
	BRect frame = ThumbFrame();
	frame.InsetBy(-2, -2);

	if (frame.Contains(pt)) {
		SetMouseEventMask(B_POINTER_EVENTS,
			B_LOCK_WINDOW_FOCUS | B_NO_POINTER_HISTORY);
		SetTracking(true);
	}
}



//
// If tracking mouse, update the thumb as the mouse moves
//
void Slider::MouseMoved(BPoint pt, uint32 code, const BMessage *message)
{
	if (IsTracking()) {
		float x = pt.x;

		if (x < 0.0) {
			x = 0.0;
		}
		if (x > BarFrame().Width()) {
			x = BarFrame().Width();
		}
		if (x != position) {
			position = x;
			DrawThumb();
			SetValue(position);
		}
	}
}



//
// Mouse button up, so stop tracking
//
void Slider::MouseUp(BPoint pt)
{
	SetTracking(false);
}



void Slider::AttachedToWindow()
{
	// Inherit view colour from parent
	if (Parent()) {
		SetLowColor(Parent()->ViewColor());
		SetTarget(Parent());
	}
}



void Slider::SetValue(int32 val)
{
	if (val != Value()) {
		BControl::SetValue(val);
		Invoke();
	}
}



// Change the colourspace by supplying new converter
void Slider::SetConverter(ConvertColour* Conv)
{
	Convert = Conv;
}



// Return current position in range 0 - 1.0
float Slider::GetValue(void)
{
	return position/BarFrame().Width();
}



//
// Set new slider position
//
void Slider::Set(float val)
{
	position = val * BarFrame().Width();
	Draw(Bounds());
}



// Set left and right colours for gradation
void Slider::SetBarColour(col l, col r)
{
	left	= l;
	right	= r;

	DrawBar();
}




