//
// Slider.h
//


#ifndef _SLIDER_H_
#define _SLIDER_H_

#include <Control.h>

#include "colour_struct.h"
#include "ConvertColour.h"


class Slider : public BControl {

private:

	float position;		// Current slider position

	BRect last_thumb;	// Previous thumb bounds
	enum {
		THUMB_HEIGHT	= 5,
		THUMB_WIDTH		= 8
	};

	// Bitmap to render gradated bar into
	BBitmap*	bm;
	uint32*		bits;

	// Extreme colours of gradated bar
	col left, right;

	// Pointer to colorspace converting class
	ConvertColour *Convert;

	virtual void DrawBar(void);
	virtual void DrawThumb(void);
	virtual BRect ThumbFrame(void) const;
	virtual BRect BarFrame(void) const;

	virtual	void MouseDown(BPoint pt);
	virtual void MouseMoved(BPoint pt, uint32 code, const BMessage *message);
	virtual	void MouseUp(BPoint pt);

	virtual void AttachedToWindow();

	virtual void SetValue(int32 val);

public:
	Slider(BRect frame,
			const char* name,
			const char* label,
			ConvertColour *Convert,
			BMessage* message,
			uint32 resizingMode = B_FOLLOW_LEFT | B_FOLLOW_TOP,
			uint32 flags = /*B_NAVIGABLE |*/ B_WILL_DRAW | B_FRAME_EVENTS);

	// Set left and right colours for gradation
	void SetBarColour(col l, col r);

	// Return current position in range 0 - 1.0
	float GetValue(void);
	void Set(float val);

	// Change the colourspace by supplying new converter
	void SetConverter(ConvertColour* Conv);

	virtual void Draw(BRect rect);
};

#endif	// _SLIDER_H_


