//
// sunview.h
//
// Allow user to position sun
//
// Copyright (c) J. Belson	2000.05.14
//


#ifndef _SUNVIEW_H_
#define _SUNVIEW_H_


#include "Bitmap.h"
#include "Message.h"
#include "View.h"
#include "Window.h"

#include "vector2d.h"


class above_view : public BView {

private:
	vector2d vector;
	float centre_x, centre_y;

	BMessage* msg;

	bool tracking;

	int MAX_SIZE;
	
	void update(float x, float y);
	float calculate_angle(void);
	void send_signal(void);

	void start_tracking(void);
	void stop_tracking(void);

	virtual void MouseDown(BPoint point);
	virtual void MouseMoved(BPoint point, uint32 transit, const BMessage *message);
	virtual void MouseUp(BPoint point);

public:
	above_view(BMessage* message);


	void get_position(float* x, float* z) {
		vector.normalise();
		*x = vector.i * MAX_SIZE;
		*z = vector.j * MAX_SIZE;
	}

	float get_rotation(void) {
		return calculate_angle();
	}

	virtual void Draw(BRect rect);

};



class side_view : public BView {

private:
	vector2d vector;
	float centre_x, centre_y;
	//float curr_x, curr_y;

	BMessage* msg;

	bool tracking;

	int MAX_SIZE;

	void update(float x, float y);

	void send_signal(void);

	// Calculate angle of lift
	float calculate_angle(void);

	void start_tracking(void);
	void stop_tracking(void);

	virtual void MouseDown(BPoint point);
	virtual void MouseMoved(BPoint point, uint32 transit, const BMessage *message);
	virtual void MouseUp(BPoint point);

public:
	side_view(BMessage* message);

	float get_lift(void) {
		return calculate_angle();
	}

	virtual void Draw(BRect rect);
};









class sunview : public BView {

private:
	BLooper*	parent;
	BMessage*	msg;

	float	width, height;	// Size of playarea
	float	x, y;			// Centre of 'sun'
	bool	tracking;		// Are we currently tracking the mouse?

	enum {
		RADIUS	= 6
	};

	// True if point is within 'sun'
	bool in_range(BPoint p);

	void draw_sun(void);

	void start_tracking(void);
	void stop_tracking(void);

	virtual void MouseDown(BPoint point);
	virtual void MouseMoved(BPoint point, uint32 transit, const BMessage *message);
	virtual void MouseUp(BPoint point);

public:
	sunview(BRect rect, BMessage* message);

	// Set position for given vector
	void set_position(float i, float j, float k);

	virtual void Draw(BRect rect);


};

#endif	// _SUNVIEW_H_
