//
// water_window.cc
//
// Prefs for water
//
// Copyright (c) J. Belson	2000.06.18
//

//#include <ColorControl.h>
#include <Message.h>
#include <iostream.h>

#include "MBorder.h"
#include "MBViewWrapper.h"
#include "MSlider.h"
#include "MStringView.h"
#include "Space.h"
#include "HGroup.h"
#include "VGroup.h"
#include "water_window.h"
#include "Colour.h"
#include "ConvertColourRGB.h"
#include "messages.h"
#include "settings.h"

#define MSG_PALETTE_CHANGED			'AW-1'
#define MSG_WAVE_SIZE_CHANGED		'AW-2'


//
// Constructor/destructor
//
water_window::water_window(
				BWindow *window,
				BRect r,
				char *name,
				window_type t,
				ulong mode)
	: smart_window(r, name, t, mode),
	  parent(window)
{
	MSlider*		slider;

	ConvertColour* conv = new ConvertColourRGB();
	Colour* cp = new Colour(BRect(0, 0, 200, 64), conv, new BMessage(MSG_PALETTE_CHANGED));

	slider = new MSlider(
					"Wave Size",
					0,
					MAX_VALUE,
					1,
					new BMessage(MSG_WAVE_SIZE_CHANGED));

	MView* top_view =
		new VGroup(
			new Space(minimax(4, 4, 4, 4)),
			new MBorder(
				M_LABELED_BORDER, 12, "Base Colour",
				new VGroup(
					new HGroup(
						new MBViewWrapper(cp),
						0
					),
					new Space(minimax(8, 8, 8, 8)),
					0
				)
			),
			new Space(),
			new MBorder(
				M_LABELED_BORDER, 12, "Waves",
				slider
			),
			0
		);

	slider->SetLimitLabels("Less", "More");
	slider->SetHashMarkCount(10);
	slider->SetHashMarks(B_HASH_MARKS_BOTTOM);

	float size;
	my_settings.get_float(WAVE_SIZE, &size);
	slider->SetValue(size*MAX_VALUE);


	AddChild(dynamic_cast<BView*>(top_view));

	int r, g, b;
	my_settings.get_int3(WATER_COLOUR, &r, &g, &b);

	cp->SetColour(r, g, b);
}



water_window::~water_window()
{
	BMessage msg(MSG_WATER_CLOSED);
	parent->PostMessage(&msg);
}


//
// Deal with window close gadget
//
bool water_window::QuitRequested()
{
	//if (win->Lock()) {
	//	win->Quit();
	//}

	return TRUE;
}


//
// Deal with incoming messages to window
//
void water_window::MessageReceived(BMessage *message)
{
	switch (message->what) {

		case MSG_PALETTE_CHANGED:
			{
				int8 val;
				uint8 red, green, blue;
			
				red = green = blue = 0;
			
				if (B_OK == message->FindInt8("red", &val)) {
					red = uint8 (val);
				}
			
				if (B_OK == message->FindInt8("green", &val)) {
					green = uint8 (val);
				}
			
				if (B_OK == message->FindInt8("blue", &val)) {
					blue = uint8 (val);
				}
				my_settings.set_int3(WATER_COLOUR, red, green, blue);
			}
			break;

		case MSG_WAVE_SIZE_CHANGED:
			{
				int32 i;
				if (B_OK == message->FindInt32("be:value", &i)) {
					float val = float(i)/MAX_VALUE;
					my_settings.set_float(WAVE_SIZE, val);
				} else {
					cout << "Couldn't find intensity!?" << endl;
				}
			}
			break;

		default:
			MWindow::MessageReceived(message);
			break;

	}
}


