//
// atmospherics_window.h
//
// Qt widget to control atmospheric settings
//
// Copyright (c) J. Belson	2000.10.17
//
// $Author: jon $	:	$Date: 2003/06/06 21:43:57 $
//

#include "atmospherics_window.h"

#include <cstdio>

#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "settings.h"


/**
 * Constructor
 */
atmospherics_window::atmospherics_window( QWidget* parent, const char*, WFlags)
	: widget_atmospherics(parent),
	  base_atmospherics_window()
{
	// Set initial sliders positions appropriately
	update_widgets();

	connect(KColorButtonHaze, SIGNAL(changed(const QColor&)), this, SLOT(slot_colour_changed(const QColor&)));
	connect(sliderHaze, SIGNAL(valueChanged(int)), this, SLOT(slot_slider_changed(int)));
}


void atmospherics_window::closeEvent(QCloseEvent* e)
{
	e->accept();

	emit window_closing();
}


/**
 * Update the widgets to reflect the current state
 */
void atmospherics_window::update_widgets(void)
{
	sliderHaze->setValue(MAX_VALUE*haze_intensity);

	fcolour& col = haze_colour;
	QColor colour = QColor(255*col.r, 255*col.g, 255*col.b);
	KColorButtonHaze->setColor(colour);
}


/**
 * Atmospheric colour changed
 */
void atmospherics_window::slot_colour_changed(const QColor& new_colour)
{
	unsigned char r = new_colour.red();
	unsigned char g = new_colour.green();
	unsigned char b = new_colour.blue();

	fcolour col(fcolour::make_rgb32(r, g, b));
	my_settings->set_colour(HAZE_COLOUR, col);
}


/**
 * Atmospheric intensity changed
 */
void atmospherics_window::slot_slider_changed(int i)
{
	float val = float(i)/MAX_VALUE;
	my_settings->set_float(HAZE_INTENSITY, val);
}

