//
// bm_window.h
//
// Copyright (C) J. Belson	2000.03.20
//
// $Author: jon $	:	$Date: 2003/05/21 11:40:09 $
//

#ifndef _BM_WINDOW_H_
#define _BM_WINDOW_H_

#include <qimage.h>
#include <qtimer.h>
#include <qwidget.h>

#include "colour.h"
#include "types.h"
#include "vector3d.h"


// Defining USE_MMAP causes a shared memory area for
//  rendering into.  This allows the render buffer to
//  be shared between rendering processes (when
//  implemented).
#define USE_MMAP
#undef USE_MMAP

class QImage;


/**
 * A widget for drawing into.  Perform simple anti-aliasing by
 *  having a render space bigger than the display space, and
 *  interpolating during the screen update.
 */
class bm_window : public QWidget
{
	Q_OBJECT

private:
	int width, height;
	int render_width, render_height;

	// Double size image to render into
	QImage* image_full;
	
	// Normal size image for displaying 
	QImage* image_scaled;

	// Perform periodic redraws
	QTimer* redraw_timer;

	// mmap()ed area for rendering into
	void*	render_buffer;
	int		render_buffer_size;

	enum {
		SCALE		= 2		// Ratio between rendered size and displayed size
	};

	// True for antialiased update
	bool smooth_update;

protected:
	void paintEvent(QPaintEvent* e);

public slots:
	void slot_refresh(void);


public:
	bm_window(int width, int height, QWidget* parent = 0, const char* name = 0);
	~bm_window();

	void closeEvent(QCloseEvent *);

	// Direct access to bitmap data
	uint32* get_buffer(void);
	int bytes_per_row(void);

	// Get structure representing rendered image
	const QImage* get_bitmap(void);

	// Window size
	void set_size(int width, int height);

	// Get size of window
	void get_window_size(int *x, int *y);

	// Get size of area to be rendered into
	void get_render_size(int* x, int* y);

	void set_smooth_update(bool b);

	//void clear_zbuffer(void);
	void clear_window(void);

	// Rate at which window contents is updated
	void set_redraw_rate(int rate);
};


#endif	// _BM_WINDOW_H_
