//
// cloud_window.cc
//
// Qt widget to allow user to experiemnt with cloud settings
//
// Copyright (c) J. Belson	2000.11.19
//
// $Author: jon $	:	$Date: 2003/06/07 14:08:01 $
//


#include <ctime>

#include <kcolorbtn.h>
#include <kcolordialog.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "cloud_window.h"
#include "settings.h"


/**
 * Constructor
 */
cloud_window::cloud_window(QWidget* p)
	: widget_clouds(p),
	  base_cloud_window()
{
	// Set initial sliders positions appropriately
	update_widgets();

	connect(kColorButtonColour, SIGNAL(changed(const QColor&)), this, SLOT(slot_colour(const QColor&)));
	connect(sliderPersistence, SIGNAL(valueChanged(int)), this, SLOT(slot_persistence(int)));
	connect(sliderPresence, SIGNAL(valueChanged(int)), this, SLOT(slot_presence(int)));
	connect(sliderCover, SIGNAL(valueChanged(int)), this, SLOT(slot_cover(int)));
	connect(sliderScale, SIGNAL(valueChanged(int)), this, SLOT(slot_scale(int)));
	connect(pushButtonGenerate, SIGNAL(clicked()), this, SLOT(slot_render()));

	// A bm_view gets created automatically by the gui code, so
	//  we can just point to that.
	cloud_view = cloud_view_ui;
}


/**
 * User closed window, notify anyone who is listening
 */
void cloud_window::closeEvent(QCloseEvent* e)
{
	e->accept();

	emit window_closing();
}


/**
 * Cloud colour changed
 */
void cloud_window::slot_colour(const QColor& col)
{
	changed_colour(col);
}


/**
 * Cloud persistence changed
 */
void cloud_window::slot_persistence(int val)
{
	changed_persistence(val);
}


/**
 * Cloud presence changed
 */
void cloud_window::slot_presence(int val)
{
	changed_presence(val);
}


/**
 * Cloud cover changed
 */
void cloud_window::slot_cover(int val)
{
	changed_cover(val);
}


/**
 * Cloud scale changed
 */
void cloud_window::slot_scale(int val)
{
	changed_scale(val);
}


/**
 * Render cloud preview
 */
void cloud_window::slot_render(void)
{
	seed = time(NULL);
	my_settings->set_int(CLOUD_SEED, seed);
	my_settings->set_colour(CLOUD_COLOUR, cloud_colour);
	my_settings->set_float(CLOUD_PERSISTENCE, persistence);
	my_settings->set_float(CLOUD_PRESENCE, presence);
	my_settings->set_float(CLOUD_COVER, cloud_cover);
	my_settings->set_float(CLOUD_SCALE, scale);

	base_cloud_window::render_thread();

	cloud_view->repaint(0, 0, CLOUD_WIDTH, CLOUD_WIDTH, false);
}


/**
 * Update the widgets to reflect the current state
 */
void cloud_window::update_widgets(void)
{
	QColor colour = QColor(255*cloud_colour.r, 255*cloud_colour.g, 255*cloud_colour.b);
	kColorButtonColour->setColor(colour);
	sliderPersistence->setValue(int (MAX_VALUE * persistence));
	sliderPresence->setValue(int (MAX_VALUE * presence/10.0));
	sliderCover->setValue(int (MAX_VALUE * (cloud_cover+4) / 8.0));
	sliderScale->setValue(int (MAX_VALUE * (scale/0.2)));
}

