//
// light_vector.h
//
// Custom widget to specify light direction
//
// Copyright (C) J. Belson	2000.12.11
//
// $Author: jon $	:	$Date: 2003/05/26 13:58:06 $
//


#ifndef _LIGHT_VECTOR_H_
#define _LIGHT_VECTOR_H_

#include <qwidget.h>

#include "vector2d.h"


/**
 * Qt widget allowing light direction to be adjusted (as viewed
 *  from above).
 */
class above_view : public QWidget {

Q_OBJECT

private:
	vector2d vector;

	int centre_x, centre_y;

	int MAX_SIZE;

	void update(float x, float y);

	float calculate_angle(void);

	void send_signal(void);

public:
	above_view(QWidget* parent=0, const char* name=0);

	virtual void mouseMoveEvent(QMouseEvent* me);
	virtual void mousePressEvent(QMouseEvent* me);
	virtual void paintEvent(QPaintEvent* pe);
	virtual void resizeEvent(QResizeEvent* re);

	float get_rotation(void)
;
signals:
	void position_changed(void);
};



/**
 * Qt widget allowing light direction to be adjusted (as viewed
 *  from the side).
 */
class side_view : public QWidget {

Q_OBJECT

private:
	vector2d vector;
	float centre_x, centre_y;

	int MAX_SIZE;

	void update(float x, float y);

	void send_signal(void);

	// Calculate angle of lift
	float calculate_angle(void);

public:
	side_view(QWidget* parent=0, const char* name=0);

	virtual void mouseMoveEvent(QMouseEvent* me);
	virtual void mousePressEvent(QMouseEvent* me);
	virtual void paintEvent(QPaintEvent* pe);
	virtual void resizeEvent(QResizeEvent* re);

	virtual QSize sizeHint(void) {
		return QSize(MAX_SIZE, MAX_SIZE);
	}

	float get_lift(void);

public slots:
	//void set_lift(float radians);

signals:
	void position_changed(void);
};



#endif	// _LIGHT_VECTOR_H_

