//
// lighting_window.cc
//
// Prefs for light source
//
// Copyright (c) J. Belson	2000.12.11
//
// $Author: jon $	: 	$Date: 2003/05/30 11:04:04 $
//

#include "lighting_window.h"
#include "vector3d.h"

#include <cstdio>
#include <cmath>

#include <qcolor.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "settings.h"
#include "types.h"

using std::cout;
using std::endl;


/**
 * Constructor
 */
lighting_window::lighting_window(QWidget* parent)
	: widget_lighting(parent),
	  base_lighting_window()
{
	update_widgets();

	connect(kColorButtonAmbient, SIGNAL(changed(const QColor&)), this, SLOT(slot_ambient_changed(const QColor&)));
	connect(kColorButtonDirectional, SIGNAL(changed(const QColor&)), this, SLOT(slot_directional_changed(const QColor&)));

	connect(aboveView, SIGNAL(position_changed()), this, SLOT(slot_light_moved()));
	connect(sideView, SIGNAL(position_changed()), this, SLOT(slot_light_moved()));
}


/**
 * User closed window, notify anyone who is listening
 */
void lighting_window::closeEvent(QCloseEvent* e)
{
	e->accept();

	emit window_closing();
}


/**
 * Update colour of ambient light
 */
void lighting_window::slot_ambient_changed(const QColor& new_colour)
{
	uint8 r = new_colour.red();
	uint8 g = new_colour.green();
	uint8 b = new_colour.blue();

	fcolour col(r/255.0, g/255.0, b/255.0);
	my_settings->set_colour(AMBIENT_COLOUR, col);
}


/**
 * Update colour of directional light
 */
void lighting_window::slot_directional_changed(const QColor& new_colour)
{
	uint8 r = new_colour.red();
	uint8 g = new_colour.green();
	uint8 b = new_colour.blue();

	fcolour col(r/255.0, g/255.0, b/255.0);
	my_settings->set_colour(DIRECT_COLOUR, col);
}


/**
 * Calculate new vector representing rays from light
 */
void lighting_window::slot_light_moved(void)
{
	// Turn the 'lift' into a vector
	float angle = sideView->get_lift();
	vector3d tmp(0.0, std::sin(angle), std::cos(angle));

	// Now rotate the vector to the required position
	angle = aboveView->get_rotation();

	float old_i = tmp.i;
	float old_j = tmp.j;
	float old_k = tmp.k;
	
	tmp.i = old_i*std::cos(angle) - old_k*std::sin(angle);
	tmp.j = old_j;
	tmp.k = old_i*std::sin(angle) + old_k*std::cos(angle);

	tmp.normalise();
cout << "light vector = " << tmp << endl;
	my_settings->set_float3(LIGHT_RAY_VECTOR, tmp.i, tmp.j, tmp.k);
}


/**
 * Update the widgets to reflect the current state
 */
void lighting_window::update_widgets(void)
{
	uint8 r, g, b;

	ambient.get_rgb8(&r, &g, &b);
	kColorButtonAmbient->setColor(QColor(r, g, b));

	direct.get_rgb8(&r, &g, &b);
	kColorButtonDirectional->setColor(QColor(r, g, b));

}
