//
// main_window.h
//
// Display and control the main application window
//
// Copyright (C) J. Belson
//
// $Author: jon $	:	$Date: 2003/07/26 17:01:33 $
//


#ifndef _MAIN_WINDOW_H_
#define _MAIN_WINDOW_H_

#include <kmainwindow.h>
#include <kmenubar.h>
#include <qvariant.h>
#include <qwidget.h>

#include "base_main_window.h"
#include "widget_main.h"


// Forward declarations
class atmospherics_window;
class lighting_window;
class mouse_bm_view;
class preview_window;
class render_window;
class settings;
class settings_window;
class texture_window;
class widget_perlin;
class widget_perlin2;
class widget_multifractal;
class widget_ridged_perlin;

class KCheckBox;
class QPopupMenu;
class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class MyCustomWidget;
class QComboBox;
class QGroupBox;
class QLabel;
class QLineEdit;
class QPushButton;
class QTabWidget;
class QWidgetStack;



/**
 * Main window with Qt-specific code
 */
class main_window : public widget_main, public base_main_window
{
	Q_OBJECT

public:
	main_window(QWidget*);
	void closeEvent(QCloseEvent *);

	void open_windows(void);

public slots:
	void slot_about_qt();
	void slot_exit();
	void slot_save_image();

	void slot_generate();
	void slot_hf_type(int i);

	void slot_preview_win();
	void slot_render_win();
	void slot_settings_win();

	// Callbacks to indicate user want to move camera/target
	void slot_camera_moved();
	void slot_target_moved();

	// Slots for child windows to signal that they're closing
	void slot_render_closed();
	void slot_preview_closed();
	void slot_settings_closed();

	// User entered new data for camera or target
	void slot_text_changed(void);

	void slot_perlin_iterations(int val);
	void slot_perlin_scale(int val);
	void slot_perlin_persistence(int val);
	void slot_perlin2_iterations(int val);
	void slot_perlin2_lacunarity(void);
	void slot_perlin2_h(void);
	void slot_perlin2_scale(int val);
	void slot_multifractal_iterations(int val);
	void slot_multifractal_lacunarity(void);
	void slot_multifractal_h(void);
	void slot_multifractal_scale(int val);
	void slot_ridged_iterations(int val);
	void slot_ridged_lacunarity(void);
	void slot_ridged_h(void);
	void slot_ridged_scale(int val);
	void slot_ridged_gain(int val);

	virtual void generate_map(void);

	// Store current window position
	void save_position(void);

private:

	enum {
		RELIEF_SIZE = 256
	};


	render_window*			w_render;
	preview_window*			w_preview;
	settings_window*		w_settings;

	widget_perlin*			w_perlin;
	widget_perlin2*			w_perlin2;
	widget_multifractal*	w_multifractal;
	widget_ridged_perlin*	w_ridged;

	// Calculate camera position in terrain space
	void calculate_camera_angle(void);

	// Calculate target position in terrain space
	void calculate_target_angle(void);

	// Update camera coord display
	void update_text_display(void);

	void update_camera_text(void);
	void update_target_text(void);

	void update_widgets(void);

	enum Update {
		CAMERA,
		TARGET,
		CAMERA_AND_TARGET
	};

	// Camera moved, update any other windows
	void update_all(bool want_rerender, Update update);

	virtual void showEvent(QShowEvent *);

};

#endif
