//
// Qt widget which displays a representation of a camera view
//
// Copyright (C) J. Belson	2000.03.09
//
// $Author: jon $	:	$Date: 2003/06/08 15:50:43 $
//

#ifndef _MOUSE_BM_VIEW_H_
#define _MOUSE_BM_VIEW_H_

#include <qimage.h>
#include <qwidget.h>


/** 
 * Qt specific widget which displays a representation of a
 *  camera view.
 */
class mouse_bm_view : public QWidget
{
	Q_OBJECT

private:
	// Relief rendered into here
	QImage* image;

	const int WIDTH;

	enum {
		TRACKING_NONE,
		TRACKING_CAMERA,
		TRACKING_TARGET
	} tracking;

	int camera_x, camera_y;
	int target_x, target_y;

	void draw_line_of_sight(void);

	// Camera or target has moved, so update and inform parent window
	void update(int x, int y);

	// Track mouse movement
	virtual void mousePressEvent(QMouseEvent* e);
	virtual void mouseReleaseEvent(QMouseEvent* e);
	virtual void mouseMoveEvent(QMouseEvent* e);


protected:
	void paintEvent(QPaintEvent* e);


public:
	mouse_bm_view(QWidget* parent=0, const char* name = 0, int width=256, int height=256);
	~mouse_bm_view();

	// Return pointer to displayable image
	QImage* get_bitmap_data(void) {	return image; }

//	QSizePolicy sizePolicy(void) const;

	void reset_camera();
	void get_camera_pos(int *x, int *y);
	void set_camera_pos(int x, int y);
	void get_target_pos(int *x, int *y);
	void set_target_pos(int x, int y);

public slots:


signals:
	void signal_camera_moved(void);
	void signal_target_moved(void);

};

#endif	// _MOUSE_BM_VIEW_H_
