//
// preview_window.cc
//
// Copyright (C) J. Belson	2000.03.27
//
// $Author: jon $	:	$Date: 2003/06/08 15:50:43 $
//

#include <stdio.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qgroupbox.h>
#include <qpushbutton.h>

#include "bm_window.h"
#include "preview_window.h"
#include "settings.h"


/**
 * Constructor.
 * Connect widgets to slots and open the OpenGL window
 */
preview_window::preview_window(QWidget* /*p*/)
	: widget_preview(0, 0),
	  base_preview_window()
{
	connect(comboBoxDetail, SIGNAL(activated(int)), this, SLOT(slot_detail(int)));
	connect(checkBoxSmoothing, SIGNAL(clicked()), this, SLOT(slot_smoothing()));
	connect(checkBoxLighting, SIGNAL(clicked()), this, SLOT(slot_lighting()));
	connect(checkBoxFogging, SIGNAL(clicked()), this, SLOT(slot_fogging()));
	connect(checkBoxTexture, SIGNAL(clicked()), this, SLOT(slot_texture()));
	connect(checkBoxShadows, SIGNAL(clicked()), this, SLOT(slot_shadows()));
	connect(checkBoxDrawSky, SIGNAL(clicked()), this, SLOT(slot_draw_sky()));
	connect(checkBoxDrawLand, SIGNAL(clicked()), this, SLOT(slot_draw_land()));
	connect(checkBoxTrackCamera, SIGNAL(clicked()), this, SLOT(slot_track_camera()));
	connect(pushButtonUpdate, SIGNAL(clicked()), this, SLOT(slot_update()));

	comboBoxDetail->setCurrentItem(detail - 6);
	checkBoxSmoothing->setChecked(smoothing);
	checkBoxLighting->setChecked(lighting);
	checkBoxTexture->setChecked(texture);
	checkBoxShadows->setChecked(shadows);
	checkBoxFogging->setChecked(fogging);
	checkBoxShadows->setChecked(shadows);
	checkBoxTrackCamera->setChecked(track_camera);
	checkBoxDrawSky->setChecked(draw_sky);
	checkBoxDrawLand->setChecked(draw_land);

	// Not implemeted
	checkBoxShadows->setEnabled(false);

	// Open output window...
	win = new GLwindow(WIDTH, HEIGHT);
	win->setFixedSize(WIDTH, HEIGHT);
	//win->resize(WIDTH, HEIGHT);
	win->setCaption("Preview Output");

	// Move the window to the appropriate place
	int x, y;
	my_settings->get_int2(WINDOW_POS_PREVIEW_OUTPUT, &x, &y);
	if (x != -1 && y != -1) {
		win->move(x, y);
	}

	win->show();
}



void preview_window::slot_close()
{
	puts("Close me");
}


/**
 * Window closing, so save position and delete
 * @note I'm not /entirely/ sure that 'delete this' is the best way
 *  of doing things.
 */
void preview_window::closeEvent(QCloseEvent *)
{
	// Save window position
	my_settings->set_int2(WINDOW_POS_PREVIEW, x(), y());
	my_settings->set_int2(WINDOW_POS_PREVIEW_OUTPUT, win->x(), win->y());

	delete win;
	delete this;
}


/**
 * Hide OpenGL window when preview window is iconified
 */
void preview_window::hideEvent(QHideEvent* /*he*/)
{
	win->hide();
}


/**
 * Show OpenGL window when preview window is iconified
 */
void preview_window::showEvent(QShowEvent* /*se*/)
{
	win->show();
}


void preview_window::slot_exit()
{
	close();
}


/**
 * Adjust number of polygons used for display
 */
void preview_window::slot_detail(int num)
{
	need_recalc = true;
	detail = 6 + num;
	my_settings->set_int(PREVIEW_DETAIL, detail);
}


/**
 * Create new terrain object and render
 */
void preview_window::create_and_render(void)
{
	need_recalc = true;
	my_settings->get_int(PREVIEW_SEED, &seed);

	base_preview_window::render_thread();
}


/**
 * Render current terrain object. Called when the viewpoint
 *  has changed and we want to show the updated preview
 */
void preview_window::render(void)
{
	my_settings->get_int(PREVIEW_SEED, &seed);

	if (track_camera) {
		base_preview_window::render_thread();
	}
}


/**
 * Re-render from the current settings
 */
void preview_window::slot_update(void)
{
	create_and_render();
}


/**
 * Toggle drawing sky
 */
void preview_window::slot_draw_sky(void)
{
	draw_sky = !draw_sky;
}


/**
 * Toggle drawing land
 */
void preview_window::slot_draw_land(void)
{
	draw_land = !draw_land;
}


/**
 * Toggle automatic updating of preview when camera moved
 */
void preview_window::slot_track_camera(void)
{
	track_camera = !track_camera;

	need_recalc = true;
}


/**
 * Toggle Gouraud shading
 */
void preview_window::slot_smoothing()
{
	smoothing = !smoothing;
	(smoothing)	? win->smooth()
				: win->flat();
}


/**
 * Toggle lighting
 */
void preview_window::slot_lighting()
{
	lighting = !lighting;
	win->set_lighting(lighting);
}


/**
 * Toggle fog
 */
void preview_window::slot_fogging()
{
	fogging = !fogging;
	win->set_fogging(fogging);
}


/**
 * Toggle procedural texturing
 */
void preview_window::slot_texture()
{
	texture = !texture;
	win->set_texture(texture);
	need_recalc = true;
}


/**
 * Toggle shadows
 */
void preview_window::slot_shadows()
{
	puts("Unimplemented");
}


/**
 * Save current window position
 */
void preview_window::save_position(void)
{
	my_settings->set_int2(WINDOW_POS_PREVIEW, x(), y());
	my_settings->set_int2(WINDOW_POS_PREVIEW_OUTPUT, win->x(), win->y());
}

