//
// progress.cc
//
// Simple pop-up window that displays a
//  progress message
//
// Copyright (c) J. Belson	2000.10.21
//
// $Author: jon $	:	$Date: 2003/05/19 21:41:42 $
//


#include <qpushbutton.h>
#include <qlabel.h>

#include "progress.h"
#include "stopwatch.h"


/**
 * Constructor
 */
progress::progress(const char *message)
	: widget_progress(),
	  quit_requested(false)
{
	set_message(message);

	timer = new stopwatch();
	timer->start();

	connect(pushButtonCancel, SIGNAL(clicked()), this, SLOT(slot_cancel()));
}


/**
 * Destructor
 */
progress::~progress()
{
	delete timer;
}


/**
 * Window closed, treat in the same way as the cancel button
 */
void progress::closeEvent(QCloseEvent *)
{
	quit_requested = true;
}


/**
 * Allow other classes to check whether cancel was clicked 
 */
bool progress::cancelled(void)
{
	return quit_requested;
}


/**
 * Update status information
 */
void progress::set_message(const char *message)
{
	labelAction->setText( QString(message) );
}


/**
 * Cancel button pressed, set flag to indicate that rendering
 *  should be stopped.
 */
void progress::slot_cancel(void)
{
	quit_requested = true;
}


/**
 * Display elapsed time since widget created.
 * Give the KApplication a chance to process a few events.
 */
void progress::update_status(void)
{
	// Display elapsed rendering time
	QString msg;
	time_t t = timer->get_time();
	int mins = int (t/6000);
	if (mins > 0) {
		t -= mins*6000;
		msg = QString::number(mins) + "m ";
	}
	int secs = int (t/100);
	int csecs = int (t%100);
	msg += QString::number(secs) + ".";
	msg += QString::number(csecs) + "s";
	labelTime->setText(msg);

	// Work round the lack of thread support by
	//  giving the event loop a chance to catch up
	kApp->processEvents();
}
