//
// render_window.h
//
// Copyright (C) J. Belson	2000.10.08
//
// $Author: jon $	: $Date: 2003/05/27 16:35:20 $
//


#ifndef _RENDER_WINDOW_H_
#define _RENDER_WINDOW_H_

#include <cstdio>

#include <qvariant.h>
#include <qwidget.h>

#include "base_render_window.h"
#include "bm_window.h"
#include "progress.h"
#include "widget_render.h"


class QVBoxLayout; 
class QHBoxLayout; 
class QGridLayout; 
class QCheckBox;
class QComboBox;
class QGroupBox;
class QLabel;
class QPushButton;
class QImage;



/**
 * Window to control rendering.
 */
class render_window : public widget_render, public base_render_window
{
	Q_OBJECT

private:

	enum {
		DEFAULT_DETAIL	= 5
	};

	QWidget*		parent;
	progress*		prog;
	bm_window*		win;

	int				detail;			// Detail of rendered image
	int				w_size_index;	// Size of rendered image
	bool			rendering;		// Render thread currently running?

	// Generating and render the landscape.
	virtual void main_render_thread(void);

	// Handle display window, using w_width/w_height
	virtual void open_output_window(void);

	virtual void xrun(void);

	static void launch_thread(void* arg);

public:
	render_window(QWidget* parent);

	void closeEvent(QCloseEvent *);

	// Main window generated a new map...
	void new_map(void);

	void hideEvent(QHideEvent* he);
	void showEvent(QShowEvent* se);

	const QImage* get_bitmap(void);

	// Move window to (x, y)
	virtual void set_position(int x, int y);

	// Store current window position
	virtual void save_position(void);

	// Render option widgets should be disabled while rendering
	virtual void disable_widgets(void);
	virtual void enable_widgets(void);


public slots:
	void slot_exit();

	void slot_render();

	void slot_size(int current);
	void slot_update(int num);
	void slot_detail(int num);

	// Rendering options
	void slot_lighting();
	void slot_sky();
	void slot_ground();
	void slot_atmospherics();
	void slot_textures();
	void slot_reflections();
	void slot_trees();
	void slot_shadows();
	void slot_backface_cull(void);
	void slot_frustrum_cull(void);

	void slot_close();


};

#endif
