//
// settings_window.cc
//
// Display configuration window
//
// Copyright (C) J. Belson	2002.02.07
//
// $Author: jon $	:	$Date: 2003/05/27 16:35:20 $
//


#include <cstdio>

#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <klocale.h>
#include <kiconloader.h>

#include "atmospherics_window.h"
#include "cloud_window.h"
#include "debug_window.h"
#include "lighting_window.h"
#include "settings.h"
#include "settings_window.h"
#include "sky_window.h"
#include "texture_window.h"
#include "water_window.h"


/**
 * Constructor
 */
settings_window::settings_window(QWidget* parent, const char* name, bool modal)
	: KDialogBase(KDialogBase::TreeList, i18n("Configure"), Ok|Default,
			Try /*Ok*/, parent, name, modal)
{
	my_settings = settings::get_instance();

	setShowIconsInTreeList(true);

	addWaterPage();
	addAtmosphericsPage();
	addLightingPage();
	addSkyPage();
	addCloudsPage();
	addTreesPage();
	addTexturesPage();
	addDebugPage();
}


/**
 * Create water settings pane
 */
void settings_window::addWaterPage(void)
{
	QFrame* page = addPage(i18n("Water"), i18n("Water Settings"),
						BarIcon("misc", KIcon::SizeSmall));

	water_window* widget = new water_window(page);
	widget->show();

	QVBoxLayout* layout = new QVBoxLayout(page, 0, 6);
	layout->addWidget(widget);

	panes[catWATER] = widget;
}


/**
 * Create atmospherics settings pane
 */
void settings_window::addAtmosphericsPage(void)
{
	QFrame* page = addPage(i18n("Atmospherics"), i18n("Atmospherics Settings"),
						BarIcon("misc", KIcon::SizeSmall));
	atmospherics_window* widget = new atmospherics_window(page);
	widget->show();

	QVBoxLayout* layout = new QVBoxLayout(page, 0, 6);
	layout->addWidget(widget);

	panes[catATMOSPHERICS] = widget;
}


/**
 * Create lighting settings pane
 */
void settings_window::addLightingPage(void)
{
	QFrame* page = addPage(i18n("Lighting"), i18n("Lighting Settings"),
						BarIcon("misc", KIcon::SizeSmall));

	lighting_window* widget = new lighting_window(page);
	widget->show();

	QVBoxLayout* layout = new QVBoxLayout(page, 0, 6);
	layout->addWidget(widget);

	panes[catLIGHTING] = widget;
}


/**
 * Create sky settings pane
 */
void settings_window::addSkyPage(void)
{
	QFrame* page = addPage(i18n("Sky"), i18n("Sky Settings"),
						BarIcon("misc", KIcon::SizeSmall));

	sky_window* widget = new sky_window(page);
	widget->show();

	QVBoxLayout* layout = new QVBoxLayout(page, 0, 6);
	layout->addWidget(widget);

	panes[catSKY] = widget;
}


/**
 * Create clouds settings pane
 */
void settings_window::addCloudsPage(void)
{
	QFrame* page = addPage(i18n("Clouds"), i18n("Cloud Settings"),
						BarIcon("misc", KIcon::SizeSmall));

	cloud_window* widget = new cloud_window(page);
	widget->show();

	QVBoxLayout* layout = new QVBoxLayout(page, 0, 6);
	layout->addWidget(widget);

	panes[catCLOUDS] = widget;
}


/**
 * Create trees settings pane
 */
void settings_window::addTreesPage(void)
{
	//QFrame* page = addPage(i18n("Trees"), i18n("Tree Settings"),
	//					BarIcon("misc", KIcon::SizeSmall));

	//QVBoxLayout* layout = new QVBoxLayout(page, 0, 6);
	//layout->addWidget(widget);

	panes[catTREES] = 0;
}


/**
 * Create textures settings pane
 */
void settings_window::addTexturesPage(void)
{
	QFrame* page = addPage(i18n("Textures"), i18n("Texture Settings"),
						BarIcon("misc", KIcon::SizeSmall));

	texture_window* widget = new texture_window(page);
	widget->show();

	QVBoxLayout* layout = new QVBoxLayout(page, 0, 6);
	layout->addWidget(widget);

	panes[catTEXTURES] = widget;
}


/**
 * Create debug settings pane
 */
void settings_window::addDebugPage(void)
{
	QFrame* page = addPage(i18n("Debug"), i18n("Debug Settings"),
						BarIcon("misc", KIcon::SizeSmall));

	debug_window* widget = new debug_window(page);
	widget->show();

	QVBoxLayout* layout = new QVBoxLayout(page, 0, 6);
	layout->addWidget(widget);

	panes[catDEBUG] = widget;
}


/**
 * User clicked Okay button, so save window position and tidy up
 */
void settings_window::slotOk()
{
	save_position();

	delete this;
}


/**
 * Reload defaults for currently selected catagory
 */
void settings_window::slotDefault()
{
	// Which pane is active?
	pref_category cat = pref_category (activePageIndex());

	my_settings->load_defaults(cat);

	// Tell the pane to update itself with the new settings
	panes[cat]->load_settings();
	panes[cat]->update_widgets();
}


/**
 * Save  current window position and destroy self
 */
void settings_window::closeEvent(QCloseEvent *)
{
	// Save window position
	save_position();

	delete this;
}


/**
 * Store current window position
 */
void settings_window::save_position(void)
{
	puts("settings_window::save_position");

	my_settings->set_int2(WINDOW_POS_SETTINGS, x(), y());
}



