//
// sky_window.cc
//
// Widget to control sky settings
//
// Copyright (c) J. Belson	2001.08.08
//
// $Author: jon $	:	$Date: 2003/05/27 16:35:20 $
//

#include <stdio.h>
#include <kcolorbutton.h>
#include <qcheckbox.h>
#include <qgroupbox.h>

#include "settings.h"
#include "sky_window.h"


/**
 * Constructor
 */
sky_window::sky_window(QWidget* p)
	: widget_sky(p)
{
	update_widgets();

	connect(	kColorButtonZenith, SIGNAL(changed(const QColor&)),
				this, SLOT(slot_zenith_colour_changed(const QColor&)));
	connect(	kColorButtonHorizon, SIGNAL(changed(const QColor&)),
				this, SLOT(slot_horizon_colour_changed(const QColor&)));
	connect(	checkBoxRealistic, SIGNAL(toggled(bool)),
				this, SLOT(slot_realistic(bool)));
}


/**
 * User closed window, notify anyone who is listening
 */
void sky_window::closeEvent(QCloseEvent *e)
{
	e->accept();
	emit window_closing();
}


/** 
 * User changed colour of zenith
 */
void sky_window::slot_zenith_colour_changed(const QColor& new_colour)
{
	unsigned char r = new_colour.red();
	unsigned char g = new_colour.green();
	unsigned char b = new_colour.blue();

	fcolour col(r/255.0, g/255.0, b/255.0);
	my_settings->set_colour(SKY_ZENITH_COLOUR, col);
}


/**
 * User changed colour of horizon
 */
void sky_window::slot_horizon_colour_changed(const QColor& new_colour)
{
	unsigned char r = new_colour.red();
	unsigned char g = new_colour.green();
	unsigned char b = new_colour.blue();

	fcolour col(r/255.0, g/255.0, b/255.0);
	my_settings->set_colour(SKY_HORIZON_COLOUR, col);
}


/**
 * Toggle realistic sky option
 */
void sky_window::slot_realistic(bool b)
{
	my_settings->set_bool(SKY_REALISTIC, b);
	groupBoxColour->setEnabled(!b);
}


/**
 * Update the widgets to reflect the current state
 */
void sky_window::update_widgets(void)
{
	uint8 r, g, b;

	zenith.get_rgb8(&r, &g, &b);
	kColorButtonZenith->setColor(QColor(r, g, b));

	horizon.get_rgb8(&r, &g, &b);
	kColorButtonHorizon->setColor(QColor(r, g, b));

	bool sky;
	my_settings->get_bool(SKY_REALISTIC, &sky);
	checkBoxRealistic->setChecked(sky);
	groupBoxColour->setEnabled(!sky);
}

