//
// water_window.cc
//
// Qt widget to control water settings
//
// Copyright (c) J. Belson	2000.11.19
//
// $Author: jon $	:	$Date: 2003/05/27 16:35:20 $
//

#include <cstdio>

#include <kcolordialog.h>
#include <kcolorbtn.h>
#include <qslider.h>
#include <qlineedit.h>

#include "settings.h"
#include "widget_water.h"
#include "water_window.h"


/**
 * Constructor
 */
water_window::water_window(QWidget* parent)
	: widget_water(parent),
	  base_water_window()
{
	// Settings were read in the base constructor, display them
	update_widgets();

	connect(kColorButtonWater, SIGNAL(changed(const QColor&)), this, SLOT(slot_colour_changed(const QColor&)));
	connect(sliderHeight, SIGNAL(valueChanged(int)), this, SLOT(slot_height_changed(int)));
	connect(sliderScale, SIGNAL(valueChanged(int)), this, SLOT(slot_scale_changed(int)));
	connect(sliderReflectivity, SIGNAL(valueChanged(int)), this, SLOT(slot_reflectivity_changed(int)));
	connect(lineEditLevel, SIGNAL(returnPressed()), this, SLOT(slot_level_changed()));
}

 
/**
 * Window closing, notify any interested parties
 */
void water_window::closeEvent(QCloseEvent *e)
{
	e->accept();

	emit window_closing();
}


/**
 * Water colour changed
 */
void water_window::slot_colour_changed(const QColor& new_colour)
{
	unsigned char r = new_colour.red();
	unsigned char g = new_colour.green();
	unsigned char b = new_colour.blue();

	my_settings->set_colour(WATER_COLOUR, fcolour(r/255.0, g/255.0, b/255.0));
}


/**
 * Wave height changed
 */
void water_window::slot_height_changed(int val)
{
	float size = float(val)/MAX_VALUE;

	my_settings->set_float(WAVE_HEIGHT, size);
}


/**
 * Wave scale changed
 */
void water_window::slot_scale_changed(int val)
{
	float size = float(val)/MAX_VALUE;

	my_settings->set_float(WAVE_SCALE, size);
}


/**
 * Water reflectivity changed
 */
void water_window::slot_reflectivity_changed(int val)
{
	float size = float(val)/MAX_VALUE;

	my_settings->set_float(WATER_REFLECTIVITY, size);
}


/**
 * Water level changed
 */
void water_window::slot_level_changed(void)
{
	bool b;
	float val = lineEditLevel->text().toFloat(&b);

	if (!b) {
		// Invalid value, reset to current water_level setting
		QString str;
		str.setNum(water_level);
		lineEditLevel->setText(str);
		
		std::printf("Illegal water level value\n");
	} else {
		water_level = val;
		my_settings->set_float(WATER_LEVEL, water_level);

		std::printf("Setting water level to %f\n", water_level);
	}
}


/**
 * Update the widgets to reflect the current state
 */
void water_window::update_widgets(void)
{
	uint8 r, g, b;
	water_colour.get_rgb8(&r, &g, &b);

	kColorButtonWater->setColor(QColor(r, g, b));
	sliderHeight->setValue(int (wave_height*MAX_VALUE));
	sliderScale->setValue(int (wave_scale*MAX_VALUE));
	sliderReflectivity->setValue(int (water_reflectivity*MAX_VALUE));

	QString str;
	str.setNum(water_level);
	lineEditLevel->setText(str);

}

